@@ -13,21 +13,21 @@ import webbrowser
 from collections import namedtuple
 from tkinter import messagebox, ttk
 from typing import (
-    Dict,# pylint disable=unused-import
+    Dict,  # pylint disable=unused-import
     Iterable,
-    List,# pylint disable=unused-import
-    Optional,# pylint disable=unused-import
+    List,  # pylint disable=unused-import
+    Optional,  # pylint disable=unused-import
     Tuple,  # pylint disable=unused-import
-    Type,# pylint disable=unused-import
-    Union, # pylint disable=unused-import
-)  
+    Type,  # pylint disable=unused-import
+    Union,  # pylint disable=unused-import
+)
 
 
 import thonny
 from thonny import get_workbench, rst_utils, tktextext, ui_utils, get_runner
 from thonny.common import ToplevelResponse, read_source
 from thonny.misc_utils import levenshtein_damerau_distance, running_on_mac_os
-from thonny.ui_utils import scrollbar_style
+from thonny.ui_utils import scrollbar_style, CommonDialog
 from thonny.running import CPythonProxy
 import subprocess
 
@@ -80,32 +80,19 @@ class AssistantView(tktextext.TextFrame):
         self.text.tag_configure("body", font="ItalicTkDefaultFont")
 
         main_font = tk.font.nametofont("TkDefaultFont")
-        
+
         # Underline on font looks better than underline on tag
         italic_underline_font = main_font.copy()
-        italic_underline_font.configure(slant="italic", 
-                                        size=main_font.cget("size"),
-                                        underline=True)
-        
-        
-        
-        self.text.tag_configure(
-            "feedback_link",
-            justify="right",
-            font=italic_underline_font,
-        )
+        italic_underline_font.configure(slant="italic", size=main_font.cget("size"), underline=True)
+
+        self.text.tag_configure("feedback_link", justify="right", font=italic_underline_font)
+        self.text.tag_bind("feedback_link", "<ButtonRelease-1>", self._ask_feedback, True)
+        self.text.tag_configure("python_errors_link", justify="right", font=italic_underline_font)
         self.text.tag_bind(
-            "feedback_link", "<ButtonRelease-1>", self._ask_feedback, True
-        )
-        self.text.tag_configure(
             "python_errors_link",
-            justify="right",
-            font=italic_underline_font,
-        )
-        self.text.tag_bind(
-            "python_errors_link", "<ButtonRelease-1>",
+            "<ButtonRelease-1>",
             lambda e: get_workbench().open_url("errors.rst"),
-            True
+            True,
         )
 
         get_workbench().bind("ToplevelResponse", self.handle_toplevel_response, True)
@@ -115,6 +102,13 @@ class AssistantView(tktextext.TextFrame):
     def handle_toplevel_response(self, msg: ToplevelResponse) -> None:
         # Can be called by event system or by Workbench
         # (if Assistant wasn't created yet but an error came)
+        if not msg.get("user_exception") and msg.get("command_name") in [
+            "execute_system_command",
+            "execute_source",
+        ]:
+            # Shell commands may be used to investigate the problem, don't clear assistance
+            return
+
         self._clear()
 
         if not isinstance(get_runner().get_backend_proxy(), CPythonProxy):
@@ -133,7 +127,7 @@ class AssistantView(tktextext.TextFrame):
         if msg.get("user_exception"):
             if not msg["user_exception"].get("message", None):
                 msg["user_exception"]["message"] = "<no message>"
-                
+
             self._exception_info = msg["user_exception"]
             self._explain_exception(msg["user_exception"])
             if get_workbench().get_option("assistance.open_assistant_on_errors"):
@@ -145,9 +139,7 @@ class AssistantView(tktextext.TextFrame):
             self.main_file_path = msg["filename"]
             source = read_source(msg["filename"])
             self._start_program_analyses(
-                msg["filename"],
-                source,
-                _get_imported_user_files(msg["filename"], source),
+                msg["filename"], source, _get_imported_user_files(msg["filename"], source)
             )
         else:
             self.main_file_path = None
@@ -161,10 +153,12 @@ class AssistantView(tktextext.TextFrame):
             )
             + "\n"
         )
-        
-        if (error_info.get("lineno") is not None 
+
+        if (
+            error_info.get("lineno") is not None
             and error_info.get("filename")
-            and os.path.exists(error_info["filename"])):
+            and os.path.exists(error_info["filename"])
+        ):
             rst += "`%s, line %d <%s>`__\n\n" % (
                 os.path.basename(error_info["filename"]),
                 error_info["lineno"],
@@ -174,8 +168,7 @@ class AssistantView(tktextext.TextFrame):
         helpers = [
             helper_class(error_info)
             for helper_class in (
-                _error_helper_classes.get(error_info["type_name"], [])
-                + _error_helper_classes["*"]
+                _error_helper_classes.get(error_info["type_name"], []) + _error_helper_classes["*"]
             )
         ]
 
@@ -217,7 +210,7 @@ class AssistantView(tktextext.TextFrame):
                 # It looks cleaner if it is not.
                 False,  # i==0
             )
-        
+
         self._current_snapshot["exception_suggestions"] = [
             dict(sug._asdict()) for sug in suggestions
         ]
@@ -254,18 +247,16 @@ class AssistantView(tktextext.TextFrame):
         self._analyzer_instances = []
         self.text.clear()
 
-    def _start_program_analyses(
-        self, main_file_path, main_file_source, imported_file_paths
-    ):
+    def _start_program_analyses(self, main_file_path, main_file_source, imported_file_paths):
 
         for cls in _program_analyzer_classes:
             analyzer = cls(self._accept_warnings)
             if analyzer.is_enabled():
                 self._analyzer_instances.append(analyzer)
-        
+
         if not self._analyzer_instances:
             return
-        
+
         self._append_text("\nAnalyzing your code ...", ("em",))
 
         # save snapshot of current source
@@ -274,16 +265,15 @@ class AssistantView(tktextext.TextFrame):
         self._current_snapshot["imported_files"] = {
             name: read_source(name) for name in imported_file_paths
         }
-        
+
         # start the analysis
         for analyzer in self._analyzer_instances:
             analyzer.start_analysis(main_file_path, imported_file_paths)
 
-
     def _accept_warnings(self, analyzer, warnings):
         if analyzer.cancelled:
             return
-        
+
         self._accepted_warning_sets.append(warnings)
         if len(self._accepted_warning_sets) == len(self._analyzer_instances):
             self._present_warnings()
@@ -307,14 +297,14 @@ class AssistantView(tktextext.TextFrame):
                     + "`debugging techniques <debugging.rst>`__.\n\n",
                     ("em",),
                 )
-        
-        
 
         if self.text.get("1.0", "end").strip():
             self._append_feedback_link()
-        
+
         if self._exception_info:
-            self._append_text("General advice on dealing with errors.\n", ("a", "python_errors_link"))
+            self._append_text(
+                "General advice on dealing with errors.\n", ("a", "python_errors_link")
+            )
 
     def _present_warnings(self):
         warnings = [w for ws in self._accepted_warning_sets for w in ws]
@@ -348,8 +338,7 @@ class AssistantView(tktextext.TextFrame):
                 self._format_file_url(dict(filename=filename)),
             )
             file_warnings = sorted(
-                by_file[filename],
-                key=lambda x: (x.get("lineno", 0), -x.get("relevance", 1)),
+                by_file[filename], key=lambda x: (x.get("lineno", 0), -x.get("relevance", 1))
             )
 
             for i, warning in enumerate(file_warnings):
@@ -381,9 +370,7 @@ class AssistantView(tktextext.TextFrame):
             explanation_rst = ""
 
         if warning.get("more_info_url"):
-            explanation_rst += (
-                "\n\n`More info online <%s>`__" % warning["more_info_url"]
-            )
+            explanation_rst += "\n\n`More info online <%s>`__" % warning["more_info_url"]
 
         explanation_rst = explanation_rst.strip()
         topic_class = "toggle" if explanation_rst else "empty"
@@ -392,7 +379,8 @@ class AssistantView(tktextext.TextFrame):
 
         return (
             ".. topic:: %s\n" % title
-            + "    :class: " + topic_class
+            + "    :class: "
+            + topic_class
             + ("" if last else ", tight")
             + "\n"
             + "    \n"
@@ -408,22 +396,16 @@ class AssistantView(tktextext.TextFrame):
 
     def _ask_feedback(self, event=None):
 
-        all_snapshots = self._snapshots_per_main_file[
-            self._current_snapshot["main_file_path"]
-        ]
+        all_snapshots = self._snapshots_per_main_file[self._current_snapshot["main_file_path"]]
 
         # TODO: select only snapshots which are not sent yet
         snapshots = all_snapshots
 
-        ui_utils.show_dialog(
-            FeedbackDialog(get_workbench(), self.main_file_path, snapshots)
-        )
-    
+        ui_utils.show_dialog(FeedbackDialog(get_workbench(), self.main_file_path, snapshots))
+
     def _get_rst_prelude(self):
-        return (".. default-role:: code\n\n"
-                + ".. role:: light\n\n"
-                + ".. role:: remark\n\n"
-                )
+        return ".. default-role:: code\n\n" + ".. role:: light\n\n" + ".. role:: remark\n\n"
+
 
 class AssistantRstText(rst_utils.RstText):
     def configure_tags(self):
@@ -463,9 +445,7 @@ class ErrorHelper(Helper):
         self.last_frame_ast = None
         if self.last_frame.source:
             try:
-                self.last_frame_ast = ast.parse(
-                    self.last_frame.source, self.last_frame.filename
-                )
+                self.last_frame_ast = ast.parse(self.last_frame.source, self.last_frame.filename)
             except SyntaxError:
                 pass
 
@@ -475,10 +455,10 @@ class ErrorHelper(Helper):
             self.last_frame_module_source = self.last_frame.source
             self.last_frame_module_ast = self.last_frame_ast
         elif self.last_frame.filename is not None:
-            self.last_frame_module_source = read_source(self.last_frame.filename)
             try:
+                self.last_frame_module_source = read_source(self.last_frame.filename)
                 self.last_frame_module_ast = ast.parse(self.last_frame_module_source)
-            except SyntaxError:
+            except Exception:
                 pass
 
         self.intro_confidence = 1
@@ -500,20 +480,20 @@ class GenericErrorHelper(ErrorHelper):
                 + "about your problem. They may add support for "
                 + "such cases in future Thonny versions.",
                 1,
-            ),
+            )
         ]
-        
+
         if error_info["message"].lower() != "invalid syntax":
-            self.suggestions.append(Suggestion(
-                "generic-search-the-web",
-                "Search the web",
-                "Try performing a web search for\n\n``Python %s: %s``"
-                % (
-                    self.error_info["type_name"],
-                    rst_utils.escape(self.error_info["message"]),
-                ),
-                1,
-            ))
+            self.suggestions.append(
+                Suggestion(
+                    "generic-search-the-web",
+                    "Search the web",
+                    "Try performing a web search for\n\n``Python %s: %s``"
+                    % (self.error_info["type_name"], rst_utils.escape(self.error_info["message"])),
+                    1,
+                )
+            )
+
 
 class ProgramAnalyzer:
     def __init__(self, on_completion):
@@ -522,7 +502,7 @@ class ProgramAnalyzer:
 
     def is_enabled(self):
         return True
-    
+
     def start_analysis(self, main_file_path, imported_file_paths):
         raise NotImplementedError()
 
@@ -551,7 +531,7 @@ class LibraryErrorHelper(ErrorHelper):
         return []
 
 
-class FeedbackDialog(tk.Toplevel):
+class FeedbackDialog(CommonDialog):
     def __init__(self, master, main_file_path, all_snapshots):
         super().__init__(master=master)
         main_frame = ttk.Frame(self)
@@ -582,17 +562,13 @@ class FeedbackDialog(tk.Toplevel):
             + "information and the data you enter or approve on this form.",
             wraplength=550,
         )
-        intro_label.grid(
-            row=1, column=0, columnspan=3, sticky="nw", padx=padx, pady=(15, 15)
-        )
+        intro_label.grid(row=1, column=0, columnspan=3, sticky="nw", padx=padx, pady=(15, 15))
 
         tree_label = ttk.Label(
             main_frame,
             text="Which messages were helpful (H) or confusing (C)?       Click on  [  ]  to mark!",
         )
-        tree_label.grid(
-            row=2, column=0, columnspan=3, sticky="nw", padx=padx, pady=(15, 0)
-        )
+        tree_label.grid(row=2, column=0, columnspan=3, sticky="nw", padx=padx, pady=(15, 0))
         tree_frame = ui_utils.TreeFrame(
             main_frame,
             columns=["helpful", "confusing", "title", "group", "symbol"],
@@ -642,9 +618,7 @@ class FeedbackDialog(tk.Toplevel):
         )
 
         comments_label = ttk.Label(main_frame, text="Any comments? Enhancement ideas?")
-        comments_label.grid(
-            row=6, column=0, columnspan=3, sticky="nw", padx=padx, pady=(15, 0)
-        )
+        comments_label.grid(row=6, column=0, columnspan=3, sticky="nw", padx=padx, pady=(15, 0))
         self.comments_text_frame = tktextext.TextFrame(
             main_frame,
             vertical_scrollbar_style=scrollbar_style("Vertical"),
@@ -659,9 +633,7 @@ class FeedbackDialog(tk.Toplevel):
             borderwidth=1,
             relief="groove",
         )
-        self.comments_text_frame.grid(
-            row=7, column=0, columnspan=3, sticky="nsew", padx=padx
-        )
+        self.comments_text_frame.grid(row=7, column=0, columnspan=3, sticky="nsew", padx=padx)
 
         url_font = tk.font.nametofont("TkDefaultFont").copy()
         url_font.configure(underline=1, size=url_font.cget("size"))
@@ -675,14 +647,10 @@ class FeedbackDialog(tk.Toplevel):
         preview_link.bind("<1>", self._preview_submission_data, True)
         preview_link.grid(row=8, column=0, sticky="nw", padx=15, pady=15)
 
-        submit_button = ttk.Button(
-            main_frame, text="Submit", width=10, command=self._submit_data
-        )
+        submit_button = ttk.Button(main_frame, text="Submit", width=10, command=self._submit_data)
         submit_button.grid(row=8, column=0, sticky="ne", padx=0, pady=15)
 
-        cancel_button = ttk.Button(
-            main_frame, text="Cancel", width=7, command=self._close
-        )
+        cancel_button = ttk.Button(main_frame, text="Cancel", width=7, command=self._close)
         cancel_button.grid(row=8, column=1, sticky="ne", padx=(10, 15), pady=15)
 
         self.protocol("WM_DELETE_WINDOW", self._close)
@@ -828,9 +796,7 @@ class FeedbackDialog(tk.Toplevel):
             except Exception as e:
                 return str(e)
 
-        result = ui_utils.run_with_waiting_dialog(
-            self, do_work, description="Uploading"
-        )
+        result = ui_utils.run_with_waiting_dialog(self, do_work, description="Uploading")
         if result == b"OK":
             if self.snapshots:
                 last_timestamp = self.snapshots[-1]["timestamp"]
@@ -838,7 +804,6 @@ class FeedbackDialog(tk.Toplevel):
             messagebox.showinfo(
                 "Done!",
                 "Thank you for the feedback!\n\nLet us know again when Assistant\nhelps or confuses you!",
-                parent=get_workbench()
             )
             self._close()
         else:
@@ -846,7 +811,6 @@ class FeedbackDialog(tk.Toplevel):
                 "Problem",
                 "Something went wrong:\n%s\n\nIf you don't mind, then try again later!"
                 % result[:1000],
-                parent=get_workbench()
             )
 
     def _select_unsent_snapshots(self, all_snapshots):
@@ -869,9 +833,7 @@ class FeedbackDialog(tk.Toplevel):
                 _last_feedback_timestamps[self.main_file_path], "%Y-%m-%dT%H:%M:%S"
             )
         else:
-            since = datetime.datetime.strptime(
-                self.snapshots[0]["timestamp"], "%Y-%m-%dT%H:%M:%S"
-            )
+            since = datetime.datetime.strptime(self.snapshots[0]["timestamp"], "%Y-%m-%dT%H:%M:%S")
 
         if since.date() == datetime.date.today() or (
             datetime.datetime.now() - since
@@ -879,9 +841,7 @@ class FeedbackDialog(tk.Toplevel):
             since_str = since.strftime("%X")
         else:
             # date and time without yer
-            since_str = since.strftime("%c").replace(
-                str(datetime.date.today().year), ""
-            )
+            since_str = since.strftime("%c").replace(str(datetime.date.today().year), "")
 
         # remove seconds
         if since_str.count(":") == 2:
@@ -924,12 +884,7 @@ def name_similarity(a, b):
     # if names differ at final isolated digits,
     # then they are probably different vars, even if their
     # distance is small (eg. location_1 and location_2)
-    if (
-        a[-1].isdigit()
-        and not a[-2].isdigit()
-        and b[-1].isdigit()
-        and not b[-2].isdigit()
-    ):
+    if a[-1].isdigit() and not a[-2].isdigit() and b[-1].isdigit() and not b[-2].isdigit():
         return 0
 
     # same thing with _ + single char suffixes
@@ -1002,5 +957,4 @@ def init():
     get_workbench().set_default("assistance.open_assistant_on_errors", True)
     get_workbench().set_default("assistance.open_assistant_on_warnings", False)
     get_workbench().set_default("assistance.disabled_checks", [])
-    get_workbench().add_view(AssistantView, "Assistant", "se", visible_by_default=False)
-
+    get_workbench().add_view(AssistantView, _("Assistant"), "se", visible_by_default=False)
