@@ -6,30 +6,29 @@ import tokenize
 import traceback
 from logging import exception
 from tkinter import messagebox, ttk
-from tkinter.messagebox import askyesno
 
-from thonny import get_workbench, ui_utils
+from thonny import get_workbench, ui_utils, get_runner
 from thonny.codeview import CodeView
 from thonny.common import (
     TextRange,
     ToplevelResponse,
     normpath_with_actual_case,
     is_same_path,
+    InlineCommand,
 )
 from thonny.tktextext import rebind_control_a
 from thonny.ui_utils import askopenfilename, asksaveasfilename, select_sequence
 from thonny.misc_utils import running_on_windows
+from _tkinter import TclError
+from thonny.base_file_browser import choose_node_for_file_operations, ask_backend_path
 
-_dialog_filetypes = [
-    ("Python files", ".py .pyw"),
-    ("text files", ".txt"),
-    ("all files", ".*"),
-]
+_dialog_filetypes = [("Python files", ".py .pyw"), ("text files", ".txt"), ("all files", ".*")]
+
+REMOTE_PATH_MARKER = " :: "
 
 
 class Editor(ttk.Frame):
-    def __init__(self, master, filename=None):
-
+    def __init__(self, master):
         ttk.Frame.__init__(self, master)
         assert isinstance(master, EditorNotebook)
 
@@ -48,22 +47,17 @@ class Editor(ttk.Frame):
         self.columnconfigure(0, weight=1)
         self.rowconfigure(0, weight=1)
 
+        self._newlines = None
         self._filename = None
         self._last_known_mtime = None
         self._asking_about_external_change = False
 
-        if filename is not None:
-            self._load_file(filename)
-            self._code_view.text.edit_modified(False)
-
         self._code_view.text.bind("<<Modified>>", self._on_text_modified, True)
         self._code_view.text.bind("<<TextChange>>", self._on_text_change, True)
         self._code_view.text.bind("<Control-Tab>", self._control_tab, True)
 
         get_workbench().bind("DebuggerResponse", self._listen_debugger_progress, True)
-        get_workbench().bind(
-            "ToplevelResponse", self._listen_for_toplevel_response, True
-        )
+        get_workbench().bind("ToplevelResponse", self._listen_for_toplevel_response, True)
 
         self.update_appearance()
 
@@ -83,6 +77,10 @@ class Editor(ttk.Frame):
     def get_title(self):
         if self.get_filename() is None:
             result = "<untitled>"
+        elif is_remote_path(self.get_filename()):
+            path = extract_target_path(self.get_filename())
+            name = path.split("/")[-1]
+            result = "[ " + name + " ]"
         else:
             result = os.path.basename(self.get_filename())
 
@@ -100,6 +98,9 @@ class Editor(ttk.Frame):
         if self._filename is None:
             return
 
+        if is_remote_path(self._filename):
+            return
+
         try:
             self._asking_about_external_change = True
 
@@ -111,10 +112,9 @@ class Editor(ttk.Frame):
 
                 if messagebox.askyesno(
                     "File is gone",
-                    "Looks like '%s' was deleted or moved outside Thonny.\n\n"
+                    "Looks like '%s' was deleted or moved outside if the editor.\n\n"
                     % self._filename
-                    + "Do you want to also close this editor?",
-                    parent=get_workbench()
+                    + "Do you want to also close the editor?",
                 ):
                     self.master.close_editor(self)
                 else:
@@ -126,9 +126,8 @@ class Editor(ttk.Frame):
 
                 if messagebox.askyesno(
                     "External modification",
-                    "Looks like '%s' was modified outside Thonny.\n\n" % self._filename
+                    "Looks like '%s' was modified outside the editor.\n\n" % self._filename
                     + "Do you want to discard current editor content and reload the file from disk?",
-                    parent=get_workbench(),
                 ):
                     self._load_file(self._filename, keep_undo=True)
                 else:
@@ -154,9 +153,29 @@ class Editor(ttk.Frame):
         return result
 
     def _load_file(self, filename, keep_undo=False):
+        try:
+            if is_remote_path(filename):
+                self._load_remote_file(filename)
+            else:
+                self._load_local_file(filename, keep_undo)
+        except SyntaxError as e:
+            assert "encoding" in str(e).lower()
+            messagebox.showerror(
+                "Problem loading file",
+                "This file seems to have problems with encoding.\n\n"
+                + "Make sure it is in UTF-8 or contains proper encoding hint.",
+            )
+            return False
+
+        self.update_appearance()
+
+    def _load_local_file(self, filename, keep_undo=False):
         with tokenize.open(filename) as fp:  # TODO: support also text files
             source = fp.read()
 
+        # get the file file format (DOS/UNIX)
+        self._newlines = fp.newlines
+
         # Make sure Windows filenames have proper format
         filename = normpath_with_actual_case(filename)
         self._filename = filename
@@ -168,104 +187,211 @@ class Editor(ttk.Frame):
         self._code_view.focus_set()
         self.master.remember_recent_file(filename)
 
+    def _load_remote_file(self, filename):
+        self._filename = filename
+        self._newlines = None
+        self._code_view.set_content("")
+        self._code_view.text.set_read_only(True)
+
+        target_filename = extract_target_path(self._filename)
+
+        self.update_title()
+        response = get_runner().send_command(
+            InlineCommand(
+                "read_file", path=target_filename, blocking=True, description=_("Loading") + "..."
+            )
+        )
+
+        if response.get("error"):
+            # TODO: make it softer
+            raise RuntimeError(response["error"])
+
+        content = response["content_bytes"]
+        if content.count(b"\r\n") > content.count(b"\n") / 2:
+            self._newlines = "\r\n"
+        else:
+            self._newlines = "\n"
+
+        self._code_view.text.set_read_only(False)
+        self._code_view.set_content_as_bytes(content)
+        self.get_text_widget().edit_modified(False)
+        self.update_title()
+
     def is_modified(self):
         return self._code_view.text.edit_modified()
 
     def save_file_enabled(self):
         return self.is_modified() or not self.get_filename()
 
-    def save_file(self, ask_filename=False):
+    def save_file(self, ask_filename=False, save_copy=False):
         if self._filename is not None and not ask_filename:
-            get_workbench().event_generate("Save", editor=self, filename=self._filename)
+            save_filename = self._filename
+            get_workbench().event_generate("Save", editor=self, filename=save_filename)
         else:
-            # http://tkinter.unpythonic.net/wiki/tkFileDialog
-            new_filename = asksaveasfilename(
-                master=get_workbench(),
-                filetypes=_dialog_filetypes,
-                defaultextension=".py",
-                initialdir=get_workbench().get_cwd(),
-            )
-
-            # Different tkinter versions may return different values
-            if new_filename in ["", (), None,]:
+            save_filename = self.ask_new_path()
+            if not save_filename:
                 return None
 
-            # Seems that in some Python versions defaultextension
-            # acts funny
-            if new_filename.lower().endswith(".py.py"):
-                new_filename = new_filename[:-3]
+            get_workbench().event_generate(
+                "SaveAs", editor=self, filename=save_filename, save_copy=save_copy
+            )
 
-            if running_on_windows():
-                # may have /-s instead of \-s and wrong case
-                new_filename = os.path.join(
-                    normpath_with_actual_case(os.path.dirname(new_filename)),
-                    os.path.basename(new_filename)
-                )
+        content_bytes = self._code_view.get_content_as_bytes(self._newlines)
 
-            if new_filename.endswith(".py"):
-                base = os.path.basename(new_filename)
-                mod_name = base[:-3].lower()
-                if running_on_windows():
-                    mod_name = mod_name.lower()
+        if is_remote_path(save_filename):
+            result = self.write_remote_file(save_filename, content_bytes, save_copy)
+        else:
+            result = self.write_local_file(save_filename, content_bytes, save_copy)
 
-                if mod_name in [
-                    "math",
-                    "turtle",
-                    "random",
-                    "statistics",
-                    "pygame",
-                    "matplotlib",
-                    "numpy",
-                ]:
+        if not result:
+            return None
 
-                    # More proper name analysis will be performed by ProgramNamingAnalyzer
-                    if not tk.messagebox.askyesno(
-                        "Potential problem",
-                        "If you name your script '%s', " % base
-                        + "you won't be able to import the library module named '%s'"
-                        % mod_name
-                        + ".\n\n"
-                        + "Do you still want to use this name for your script?",
-                        parent=get_workbench(),
-                    ):
-                        return self.save_file(ask_filename)
+        if not save_copy:
+            self._filename = save_filename
 
-            self._filename = new_filename
-            get_workbench().event_generate("SaveAs", editor=self, filename=new_filename)
+        self.update_title()
+        return save_filename
 
-        content = self._code_view.get_content_as_bytes()
+    def write_local_file(self, save_filename, content_bytes, save_copy):
         try:
-            f = open(self._filename, mode="wb")
-            f.write(content)
+            f = open(save_filename, mode="wb")
+            f.write(content_bytes)
             f.flush()
             # Force writes on disk, see https://learn.adafruit.com/adafruit-circuit-playground-express/creating-and-editing-code#1-use-an-editor-that-writes-out-the-file-completely-when-you-save-it
             os.fsync(f)
             f.close()
-            self._last_known_mtime = os.path.getmtime(self._filename)
+            if not save_copy or save_filename == self._filename:
+                self._last_known_mtime = os.path.getmtime(save_filename)
+            get_workbench().event_generate(
+                "LocalFileOperation", path=save_filename, operation="save"
+            )
         except PermissionError:
-            if askyesno(
-                "Permission Error",
-                "Looks like this file or folder is not writable.\n\n"
-                + "Do you want to save under another folder and/or filename?",
-                parent=get_workbench(),
-            ):
-                return self.save_file(True)
-            else:
-                return None
+            messagebox.showerror(
+                "Permission Error", "Looks like this file or folder is not writable."
+            )
+            return False
 
-        self.master.remember_recent_file(self._filename)
+        if not save_copy or save_filename == self._filename:
+            self.master.remember_recent_file(save_filename)
 
-        self._code_view.text.edit_modified(False)
+        if not save_copy or save_filename == self._filename:
+            self._code_view.text.edit_modified(False)
 
-        return self._filename
+        return True
+
+    def write_remote_file(self, save_filename, content_bytes, save_copy):
+        if get_runner().can_do_file_operations():
+            target_filename = extract_target_path(save_filename)
+
+            get_runner().send_command(
+                InlineCommand(
+                    "write_file",
+                    path=target_filename,
+                    content_bytes=content_bytes,
+                    editor_id=id(self),
+                    blocking=True,
+                    description=_("Saving") + "...",
+                )
+            )
+
+            if not save_copy:
+                self._code_view.text.edit_modified(False)
+
+            self.update_title()
+
+            # NB! edit_modified is not falsed yet!
+            get_workbench().event_generate(
+                "RemoteFileOperation", path=target_filename, operation="save"
+            )
+            return True
+        else:
+            messagebox.showwarning("Can't save", "Device is busy, wait and try again!")
+            return False
+
+    def ask_new_path(self):
+        node = choose_node_for_file_operations(self.winfo_toplevel(), "Where to save to?")
+        if not node:
+            return None
+
+        if node == "local":
+            return self.ask_new_local_path()
+        else:
+            assert node == "remote"
+            return self.ask_new_remote_path()
+
+    def ask_new_remote_path(self):
+        target_path = ask_backend_path(self.winfo_toplevel(), "save")
+        if target_path:
+            return make_remote_path(target_path)
+        else:
+            return None
+
+    def ask_new_local_path(self):
+        if self._filename is None:
+            initialdir = get_workbench().get_local_cwd()
+            initialfile = None
+        else:
+            initialdir = os.path.dirname(self._filename)
+            initialfile = os.path.basename(self._filename)
+
+        # http://tkinter.unpythonic.net/wiki/tkFileDialog
+        new_filename = asksaveasfilename(
+            filetypes=_dialog_filetypes,
+            defaultextension=".py",
+            initialdir=initialdir,
+            initialfile=initialfile,
+        )
+
+        # Different tkinter versions may return different values
+        if new_filename in ["", (), None]:
+            return None
+
+        # Seems that in some Python versions defaultextension
+        # acts funny
+        if new_filename.lower().endswith(".py.py"):
+            new_filename = new_filename[:-3]
+
+        if running_on_windows():
+            # may have /-s instead of \-s and wrong case
+            new_filename = os.path.join(
+                normpath_with_actual_case(os.path.dirname(new_filename)),
+                os.path.basename(new_filename),
+            )
+
+        if new_filename.endswith(".py"):
+            base = os.path.basename(new_filename)
+            mod_name = base[:-3].lower()
+            if running_on_windows():
+                mod_name = mod_name.lower()
+
+            if mod_name in [
+                "math",
+                "turtle",
+                "random",
+                "statistics",
+                "pygame",
+                "matplotlib",
+                "numpy",
+            ]:
+
+                # More proper name analysis will be performed by ProgramNamingAnalyzer
+                if not tk.messagebox.askyesno(
+                    "Potential problem",
+                    "If you name your script '%s', " % base
+                    + "you won't be able to import the library module named '%s'" % mod_name
+                    + ".\n\n"
+                    + "Do you still want to use this name for your script?",
+                ):
+                    return self.ask_new_local_path()
+
+        return new_filename
 
     def show(self):
         self.master.select(self)
 
     def update_appearance(self):
         self._code_view.set_gutter_visibility(
-            get_workbench().get_option("view.show_line_numbers")
-            or get_workbench().get_ui_mode() == "simple"
+            get_workbench().get_option("view.show_line_numbers") or get_workbench().in_simple_mode()
         )
         self._code_view.set_line_length_margin(
             get_workbench().get_option("view.recommended_line_length")
@@ -316,13 +442,16 @@ class Editor(ttk.Frame):
         return self.focus_displayof() == self._code_view.text
 
     def _on_text_modified(self, event):
+        self.update_title()
+
+    def update_title(self):
         try:
             self.master.update_editor_title(self)
         except Exception:
             traceback.print_exc()
 
     def _on_text_change(self, event):
-        self.master.update_editor_title(self)
+        self.update_title()
 
     def destroy(self):
         get_workbench().unbind("DebuggerResponse", self._listen_debugger_progress)
@@ -365,6 +494,8 @@ class EditorNotebook(ui_utils.ClosableNotebook):
         # constructor hasn't completed yet
         self.preferred_size_in_pw = None
 
+        get_workbench().bind("WindowFocusIn", self.on_focus_window, True)
+
     def _init_commands(self):
         # TODO: do these commands have to be in EditorNotebook ??
         # Create a module level function install_editor_notebook ??
@@ -373,9 +504,9 @@ class EditorNotebook(ui_utils.ClosableNotebook):
         get_workbench().add_command(
             "new_file",
             "file",
-            "New",
+            _("New"),
             self._cmd_new_file,
-            caption="New",
+            caption=_("New"),
             default_sequence=select_sequence("<Control-n>", "<Command-n>"),
             extra_sequences=["<Control-Greek_nu>"],
             group=10,
@@ -386,9 +517,9 @@ class EditorNotebook(ui_utils.ClosableNotebook):
         get_workbench().add_command(
             "open_file",
             "file",
-            "Open...",
+            _("Open..."),
             self._cmd_open_file,
-            caption="Load",
+            caption=_("Load"),
             default_sequence=select_sequence("<Control-o>", "<Command-o>"),
             extra_sequences=["<Control-Greek_omicron>"],
             group=10,
@@ -397,7 +528,7 @@ class EditorNotebook(ui_utils.ClosableNotebook):
         )
 
         get_workbench().add_command(
-            "recents", "file", "Recent files", group=10, submenu=self._recent_menu
+            "recents", "file", _("Recent files"), group=10, submenu=self._recent_menu
         )
 
         # http://stackoverflow.com/questions/22907200/remap-default-keybinding-in-tkinter
@@ -408,7 +539,7 @@ class EditorNotebook(ui_utils.ClosableNotebook):
         get_workbench().add_command(
             "close_file",
             "file",
-            "Close",
+            _("Close"),
             self._cmd_close_file,
             default_sequence=select_sequence("<Control-w>", "<Command-w>"),
             extra_sequences=["<Control-Greek_finalsmallsigma>"],
@@ -419,7 +550,7 @@ class EditorNotebook(ui_utils.ClosableNotebook):
         get_workbench().add_command(
             "close_files",
             "file",
-            "Close all",
+            _("Close all"),
             self.close_tabs,
             tester=lambda: self.get_current_editor() is not None,
             default_sequence=select_sequence("<Control-W>", "<Command-Alt-w>"),
@@ -429,9 +560,9 @@ class EditorNotebook(ui_utils.ClosableNotebook):
         get_workbench().add_command(
             "save_file",
             "file",
-            "Save",
+            _("Save"),
             self._cmd_save_file,
-            caption="Save",
+            caption=_("Save"),
             default_sequence=select_sequence("<Control-s>", "<Command-s>"),
             extra_sequences=["<Control-Greek_sigma>"],
             tester=self._cmd_save_file_enabled,
@@ -443,7 +574,7 @@ class EditorNotebook(ui_utils.ClosableNotebook):
         get_workbench().add_command(
             "save_file_as",
             "file",
-            "Save as...",
+            _("Save as..."),
             self._cmd_save_file_as,
             default_sequence=select_sequence("<Control-Shift-S>", "<Command-Shift-S>"),
             extra_sequences=["<Control-Greek_SIGMA>"],
@@ -451,24 +582,33 @@ class EditorNotebook(ui_utils.ClosableNotebook):
             group=10,
         )
 
+        get_workbench().add_command(
+            "save_copy",
+            "file",
+            _("Save copy..."),
+            self._cmd_save_copy,
+            tester=lambda: self.get_current_editor() is not None,
+            group=10,
+        )
+
         get_workbench().add_command(
             "rename_file",
             "file",
-            "Rename...",
+            _("Rename..."),
             self._cmd_rename_file,
             tester=self._cmd_rename_file_enabled,
             group=10,
         )
 
-        get_workbench().createcommand(
-            "::tk::mac::OpenDocument", self._mac_open_document
-        )
+        get_workbench().createcommand("::tk::mac::OpenDocument", self._mac_open_document)
 
     def load_startup_files(self):
         """If no filename was sent from command line
         then load previous files (if setting allows)"""
 
-        cmd_line_filenames = [os.path.abspath(name) for name in sys.argv[1:] if os.path.exists(name)]
+        cmd_line_filenames = [
+            os.path.abspath(name) for name in sys.argv[1:] if os.path.exists(name)
+        ]
 
         if len(cmd_line_filenames) > 0:
             filenames = cmd_line_filenames
@@ -516,9 +656,7 @@ class EditorNotebook(ui_utils.ClosableNotebook):
     def _update_recent_menu(self):
         recents = get_workbench().get_option("file.recent_files")
         relevant_recents = [
-            path
-            for path in recents
-            if os.path.exists(path) and not self.file_is_opened(path)
+            path for path in recents if os.path.exists(path) and not self.file_is_opened(path)
         ]
         self._recent_menu.delete(0, "end")
         for path in relevant_recents:
@@ -540,9 +678,7 @@ class EditorNotebook(ui_utils.ClosableNotebook):
         get_workbench().set_option("file.current_file", current_file)
 
         open_files = [
-            editor.get_filename()
-            for editor in self.winfo_children()
-            if editor.get_filename()
+            editor.get_filename() for editor in self.winfo_children() if editor.get_filename()
         ]
         get_workbench().set_option("file.open_files", open_files)
 
@@ -554,16 +690,25 @@ class EditorNotebook(ui_utils.ClosableNotebook):
         new_editor.focus_set()
 
     def _cmd_open_file(self):
-        filename = askopenfilename(
-            master=get_workbench(),
-            filetypes=_dialog_filetypes,
-            initialdir=get_workbench().get_cwd(),
-        )
-        if (
-            filename
-        ):  # Note that missing filename may be "" or () depending on tkinter version
+        node = choose_node_for_file_operations(self.winfo_toplevel(), "Where to open from?")
+        if not node:
+            return None
+
+        if node == "local":
+            path = askopenfilename(
+                filetypes=_dialog_filetypes, initialdir=get_workbench().get_local_cwd()
+            )
+        else:
+            assert node == "remote"
+            target_path = ask_backend_path(self.winfo_toplevel(), "open")
+            if not target_path:
+                return None
+
+            path = make_remote_path(target_path)
+
+        if path:
             # self.close_single_untitled_unmodified_editor()
-            self.show_file(filename)
+            self.show_file(path)
 
     def _control_o(self, event):
         # http://stackoverflow.com/questions/22907200/remap-default-keybinding-in-tkinter
@@ -602,9 +747,7 @@ class EditorNotebook(ui_utils.ClosableNotebook):
             self.update_editor_title(self.get_current_editor())
 
     def _cmd_save_file_enabled(self):
-        return (
-            self.get_current_editor() and self.get_current_editor().save_file_enabled()
-        )
+        return self.get_current_editor() and self.get_current_editor().save_file_enabled()
 
     def _cmd_save_file_as(self):
         if not self.get_current_editor():
@@ -614,6 +757,13 @@ class EditorNotebook(ui_utils.ClosableNotebook):
         self.update_editor_title(self.get_current_editor())
         get_workbench().update_title()
 
+    def _cmd_save_copy(self):
+        if not self.get_current_editor():
+            return
+
+        self.get_current_editor().save_file(ask_filename=True, save_copy=True)
+        self.update_editor_title(self.get_current_editor())
+
     def _cmd_save_file_as_enabled(self):
         return self.get_current_editor() is not None
 
@@ -628,18 +778,11 @@ class EditorNotebook(ui_utils.ClosableNotebook):
             os.remove(old_filename)
 
     def _cmd_rename_file_enabled(self):
-        return (
-            self.get_current_editor()
-            and self.get_current_editor().get_filename() is not None
-        )
+        return self.get_current_editor() and self.get_current_editor().get_filename() is not None
 
     def close_single_untitled_unmodified_editor(self):
         editors = self.winfo_children()
-        if (
-            len(editors) == 1
-            and not editors[0].is_modified()
-            and not editors[0].get_filename()
-        ):
+        if len(editors) == 1 and not editors[0].is_modified() and not editors[0].get_filename():
             self._cmd_close_file()
 
     def _mac_open_document(self, *args):
@@ -651,6 +794,13 @@ class EditorNotebook(ui_utils.ClosableNotebook):
     def get_current_editor(self):
         return self.get_current_child()
 
+    def get_current_editor_content(self):
+        editor = self.get_current_editor()
+        if editor is None:
+            return None
+        else:
+            return editor.get_code_view().get_content()
+
     def get_all_editors(self):
         # When workspace is closing, self.winfo_children()
         # may return an unexplainable tkinter.Frame
@@ -682,6 +832,16 @@ class EditorNotebook(ui_utils.ClosableNotebook):
 
         return editor
 
+    def show_remote_file(self, target_filename):
+        if not get_runner().can_do_file_operations():
+            messagebox.showwarning(
+                "Can't open",
+                "Device is busy, can't read file content.\n" + "Please try again later!",
+            )
+            return None
+        else:
+            return self.show_file(make_remote_path(target_filename))
+
     def show_file_at_line(self, filename, lineno, col_offset=None):
         editor = self.show_file(filename)
         editor.select_line(lineno, col_offset)
@@ -693,18 +853,21 @@ class EditorNotebook(ui_utils.ClosableNotebook):
     def update_editor_title(self, editor, title=None):
         if title is None:
             title = editor.get_title()
-        self.tab(editor, text=title)
+        try:
+            self.tab(editor, text=title)
+        except TclError:
+            pass
 
     def _open_file(self, filename):
-        editor = Editor(self, filename)
+        editor = Editor(self)
+        editor._load_file(filename)
         self.add(editor, text=editor.get_title())
-
         return editor
 
     def get_editor(self, filename, open_when_necessary=False):
         for child in self.winfo_children():
             child_filename = child.get_filename(False)
-            if child_filename and is_same_path(child.get_filename(), filename):
+            if child_filename == filename:
                 return child
 
         if open_when_necessary:
@@ -728,9 +891,7 @@ class EditorNotebook(ui_utils.ClosableNotebook):
             message = "Do you want to save file before closing?"
 
         confirm = messagebox.askyesnocancel(
-            title="Save On Close", message=message, default=messagebox.YES,
-            master=get_workbench(),
-            parent=get_workbench(),
+            title="Save On Close", message=message, default=messagebox.YES
         )
 
         if confirm:
@@ -746,6 +907,10 @@ class EditorNotebook(ui_utils.ClosableNotebook):
         else:
             return True
 
+    def on_focus_window(self, event=None):
+        for editor in self.get_all_editors():
+            editor.check_for_external_changes()
+
 
 def get_current_breakpoints():
     result = {}
@@ -759,6 +924,7 @@ def get_current_breakpoints():
 
     return result
 
+
 def get_saved_current_script_filename(force=True):
     editor = get_workbench().get_editor_notebook().get_current_editor()
     if not editor:
@@ -770,5 +936,18 @@ def get_saved_current_script_filename(force=True):
 
     if editor.is_modified():
         filename = editor.save_file()
-    
+
     return filename
+
+
+def is_remote_path(s):
+    return REMOTE_PATH_MARKER in s
+
+
+def extract_target_path(s):
+    assert is_remote_path(s)
+    return s[s.find(REMOTE_PATH_MARKER) + len(REMOTE_PATH_MARKER) :]
+
+
+def make_remote_path(target_path):
+    return get_runner().get_node_label() + REMOTE_PATH_MARKER + target_path
