@@ -21,9 +21,7 @@ class SyntaxText(EnhancedText):
     def __init__(self, master=None, cnf={}, **kw):
         self._syntax_options = {}
         super().__init__(master=master, cnf=cnf, **kw)
-        self._syntax_theme_change_binding = tk._default_root.bind(
-            "<<SyntaxThemeChanged>>", self._reload_syntax_options, True
-        )
+        get_workbench().bind("SyntaxThemeChanged", self._reload_syntax_options, True)
         self._reload_syntax_options()
 
     def set_syntax_options(self, syntax_options):
@@ -31,12 +29,20 @@ class SyntaxText(EnhancedText):
         for tag_name in self._syntax_options:
             self.tag_reset(tag_name)
 
+        background = syntax_options.get("TEXT", {}).get("background")
+
         # apply new options
         for tag_name in syntax_options:
+            opts = syntax_options[tag_name]
+
+            if tag_name == "string3":
+                # Needs explicit background to override uniline tags
+                opts["background"] = background
+
             if tag_name == "TEXT":
-                self.configure(**syntax_options[tag_name])
+                self.configure(**opts)
             else:
-                self.tag_configure(tag_name, **syntax_options[tag_name])
+                self.tag_configure(tag_name, **opts)
 
         self._syntax_options = syntax_options
 
@@ -44,6 +50,7 @@ class SyntaxText(EnhancedText):
             self.tag_lower("current_line")
 
         self.tag_raise("sel")
+        self.tag_lower("stdout")
 
     def _reload_theme_options(self, event=None):
         super()._reload_theme_options(event)
@@ -55,18 +62,16 @@ class SyntaxText(EnhancedText):
 
     def destroy(self):
         super().destroy()
-        tk._default_root.unbind(
-            "<<SyntaxThemeChanged>>", self._syntax_theme_change_binding
-        )
+        get_workbench().unbind("SyntaxThemeChanged", self._reload_syntax_options)
 
 
 class PythonText(SyntaxText):
     def __init__(self, master=None, cnf={}, **kw):
         if "indent_with_tabs" not in kw:
             kw["indent_with_tabs"] = False
-            
+
         super().__init__(master=master, cnf=cnf, **kw)
-    
+
     def perform_return(self, event):
         # copied from idlelib.EditorWindow (Python 3.4.2)
         # slightly modified
@@ -179,6 +184,7 @@ class CodeViewText(EnhancedTextWithLogging, PythonText):
     """Provides opportunities for monkey-patching by plugins"""
 
     def __init__(self, master=None, cnf={}, **kw):
+
         if "replace_tabs" not in kw:
             kw["replace_tabs"] = True
 
@@ -211,7 +217,6 @@ class CodeViewText(EnhancedTextWithLogging, PythonText):
 
 class CodeView(tktextext.TextFrame):
     def __init__(self, master, propose_remove_line_numbers=False, **text_frame_args):
-
         tktextext.TextFrame.__init__(
             self,
             master,
@@ -228,8 +233,8 @@ class CodeView(tktextext.TextFrame):
 
         assert self._first_line_number is not None
 
-        self._syntax_theme_change_binding = tk._default_root.bind(
-            "<<SyntaxThemeChanged>>", self._reload_theme_options, True
+        self._syntax_theme_change_binding = get_workbench().bind(
+            "SyntaxThemeChanged", self._reload_theme_options, True
         )
 
         self._reload_theme_options()
@@ -245,33 +250,34 @@ class CodeView(tktextext.TextFrame):
         self._gutter.tag_raise("spacer")
 
     def get_content(self):
-        return self.text.get(
-            "1.0", "end-1c"
-        )  # -1c because Text always adds a newline itself
+        return self.text.get("1.0", "end-1c")  # -1c because Text always adds a newline itself
 
-    def detect_encoding(self):
-        encoding, _ = tokenize.detect_encoding(
-            io.BytesIO(self.get_content().encode("ascii", errors="replace")).readline
-        )
+    def detect_encoding(self, data):
+        encoding, _ = tokenize.detect_encoding(io.BytesIO(data).readline)
         return encoding
 
-    def get_content_as_bytes(self):
+    def get_content_as_bytes(self, newlines=None):
+        newlines_windows = "\r\n"
         chars = self.get_content().replace("\r", "")
-        if running_on_windows():
-            chars = chars.replace("\n", "\r\n")
+        if newlines == None and running_on_windows() or newlines == newlines_windows:
+            chars = chars.replace("\n", newlines_windows)
 
-        return chars.encode(self.detect_encoding())
+        return chars.encode(
+            self.detect_encoding(self.get_content().encode("ascii", errors="replace"))
+        )
+
+    def set_content_as_bytes(self, data):
+        encoding = self.detect_encoding(data)
+        chars = data.decode(encoding).replace("\r\n", "\n")
+        self.set_content(chars)
 
     def set_content(self, content, keep_undo=False):
         self.text.direct_delete("1.0", tk.END)
         self.text.direct_insert("1.0", content)
-        self.update_gutter()
 
         if not keep_undo:
             self.text.edit_reset()
 
-        self.text.event_generate("<<TextChange>>")
-
     def _toggle_breakpoint(self, event):
         index = "@%d,%d" % (event.x, event.y)
         start_index = index + " linestart"
@@ -286,19 +292,29 @@ class CodeView(tktextext.TextFrame):
 
         self.update_gutter(clean=True)
 
-    def compute_gutter_line(self, lineno):
-        visual_line_number = self._first_line_number + lineno - 1
-        linestart = str(visual_line_number) + ".0"
+    def _clean_selection(self):
+        self.text.tag_remove("sel", "1.0", "end")
+        self._gutter.tag_remove("sel", "1.0", "end")
 
-        yield from super().compute_gutter_line(lineno)
-        yield " ", ("spacer",)
+    def _text_changed(self, event):
+        self.update_gutter(
+            clean=self.text._last_event_changed_line_count
+            and self.text.tag_ranges("breakpoint_line")
+        )
 
-        if self.text.tag_nextrange(
-            "breakpoint_line", linestart, linestart + " lineend"
-        ):
-            yield BREAKPOINT_SYMBOL, ("breakpoint",)
+    def compute_gutter_line(self, lineno, plain=False):
+        if plain:
+            yield str(lineno) + " ", ()
         else:
-            yield " ", ()
+            visual_line_number = self._first_line_number + lineno - 1
+            linestart = str(visual_line_number) + ".0"
+
+            yield str(lineno), ()
+
+            if self.text.tag_nextrange("breakpoint_line", linestart, linestart + " lineend"):
+                yield BREAKPOINT_SYMBOL, ("breakpoint",)
+            else:
+                yield " ", ()
 
     def select_range(self, text_range):
         self.text.tag_remove("sel", "1.0", tk.END)
@@ -336,9 +352,7 @@ class CodeView(tktextext.TextFrame):
     def get_selected_range(self):
         if self.text.has_selection():
             lineno, col_offset = map(int, self.text.index(tk.SEL_FIRST).split("."))
-            end_lineno, end_col_offset = map(
-                int, self.text.index(tk.SEL_LAST).split(".")
-            )
+            end_lineno, end_col_offset = map(int, self.text.index(tk.SEL_LAST).split("."))
         else:
             lineno, col_offset = map(int, self.text.index(tk.INSERT).split("."))
             end_lineno, end_col_offset = lineno, col_offset
@@ -347,9 +361,7 @@ class CodeView(tktextext.TextFrame):
 
     def destroy(self):
         super().destroy()
-        tk._default_root.unbind(
-            "<<SyntaxThemeChanged>>", self._syntax_theme_change_binding
-        )
+        get_workbench().unbind("SyntaxThemeChanged", self._reload_theme_options)
 
     def _reload_theme_options(self, event=None):
         super()._reload_theme_options(event)
@@ -358,15 +370,16 @@ class CodeView(tktextext.TextFrame):
             opts = _syntax_options["GUTTER"].copy()
             if "background" in opts and "selectbackground" not in opts:
                 opts["selectbackground"] = opts["background"]
+                opts["inactiveselectbackground"] = opts["background"]
             if "foreground" in opts and "selectforeground" not in opts:
                 opts["selectforeground"] = opts["foreground"]
 
             self._gutter.configure(opts)
 
-            if "background" in _syntax_options["GUTTER"]:
-                self._margin_line.configure(
-                    background=_syntax_options["GUTTER"]["background"]
-                )
+            if "background" in opts:
+                background = opts["background"]
+                self._margin_line.configure(background=background)
+                self._gutter.tag_configure("sel", background=background)
 
         if "breakpoint" in _syntax_options:
             self._gutter.tag_configure("breakpoint", _syntax_options["breakpoint"])
@@ -375,9 +388,7 @@ class CodeView(tktextext.TextFrame):
 def set_syntax_options(syntax_options):
     global _syntax_options
     _syntax_options = syntax_options
-
-    assert tk._default_root is not None
-    tk._default_root.event_generate("<<SyntaxThemeChanged>>")
+    get_workbench().event_generate("SyntaxThemeChanged")
 
 
 def get_syntax_options_for_tag(tag, **base_options):
