@@ -6,6 +6,7 @@ import traceback
 
 from thonny import ast_utils, get_workbench, ui_utils
 from thonny.common import TextRange, range_contains_smaller
+import sys
 
 
 class AstView(ui_utils.TreeFrame):
@@ -20,9 +21,8 @@ class AstView(ui_utils.TreeFrame):
         self._current_code_view = None
         self.tree.bind("<<TreeviewSelect>>", self._locate_code)
         self.tree.bind("<<Copy>>", self._copy_to_clipboard)
-        get_workbench().get_editor_notebook().bind(
-            "<<NotebookTabChanged>>", self._update
-        )
+        self.tree.bind("<Map>", self._update, True)
+        get_workbench().get_editor_notebook().bind("<<NotebookTabChanged>>", self._update)
         get_workbench().bind("Save", self._update, True)
         get_workbench().bind("SaveAs", self._update, True)
         get_workbench().bind_class("Text", "<Double-Button-1>", self._update, True)
@@ -43,14 +43,22 @@ class AstView(ui_utils.TreeFrame):
         self._update(None)
 
     def _update(self, event):
+        if not self.winfo_ismapped():
+            return
+
         editor = get_workbench().get_editor_notebook().get_current_editor()
 
         if not editor:
             self._current_code_view = None
             return
 
-        self._current_code_view = editor.get_code_view()
-        self._current_source = self._current_code_view.get_content_as_bytes()
+        new_cw = editor.get_code_view()
+        new_source = new_cw.get_content_as_bytes()
+        if self._current_code_view == new_cw and self._current_source == new_source:
+            return
+
+        self._current_code_view = new_cw
+        self._current_source = new_source
         selection = self._current_code_view.get_selected_range()
 
         self._clear_tree()
@@ -98,9 +106,7 @@ class AstView(ui_utils.TreeFrame):
                 if hasattr(node, "end_lineno") and hasattr(node, "end_col_offset"):
                     self.tree.set(node_id, "end_lineno", node.end_lineno)
                     self.tree.set(node_id, "end_col_offset", node.end_col_offset)
-                    range_str += (
-                        "  -  " + str(node.end_lineno) + "." + str(node.end_col_offset)
-                    )
+                    range_str += "  -  " + str(node.end_lineno) + "." + str(node.end_col_offset)
                 else:
                     # fallback
                     self.tree.set(node_id, "end_lineno", node.lineno)
@@ -134,7 +140,7 @@ class AstView(ui_utils.TreeFrame):
     def _copy_to_clipboard(self, event):
         self.clipboard_clear()
         if self._current_source is not None:
-            pretty_ast = ast_utils.pretty(ast_utils.parse_source(self._current_source))
+            pretty_ast = pretty(ast_utils.parse_source(self._current_source))
             self.clipboard_append(pretty_ast)
 
 
@@ -147,8 +153,7 @@ def _find_closest_containing_node(tree, text_range):
 
     # no suitable child was found
     if hasattr(tree, "lineno") and range_contains_smaller(
-        TextRange(tree.lineno, tree.col_offset, tree.end_lineno, tree.end_col_offset),
-        text_range,
+        TextRange(tree.lineno, tree.col_offset, tree.end_lineno, tree.end_col_offset), text_range
     ):
         return tree
     # nope
@@ -156,5 +161,49 @@ def _find_closest_containing_node(tree, text_range):
         return None
 
 
+def pretty(node, key="/", level=0):
+    """Used for exporting ASTView content"""
+    if isinstance(node, ast.AST):
+        fields = [(key, val) for key, val in ast.iter_fields(node)]
+        value_label = node.__class__.__name__
+        if isinstance(node, ast.Call):
+            # Try to make 3.4 AST-s more similar to 3.5
+            if sys.version_info[:2] == (3, 4):
+                if ("kwargs", None) in fields:
+                    fields.remove(("kwargs", None))
+                if ("starargs", None) in fields:
+                    fields.remove(("starargs", None))
+
+            # TODO: translate also non-None kwargs and starargs
+
+    elif isinstance(node, list):
+        fields = list(enumerate(node))
+        if len(node) == 0:
+            value_label = "[]"
+        else:
+            value_label = "[...]"
+    else:
+        fields = []
+        value_label = repr(node)
+
+    item_text = level * "    " + str(key) + "=" + value_label
+
+    if hasattr(node, "lineno"):
+        item_text += " @ " + str(getattr(node, "lineno"))
+        if hasattr(node, "col_offset"):
+            item_text += "." + str(getattr(node, "col_offset"))
+
+        if hasattr(node, "end_lineno"):
+            item_text += "  -  " + str(getattr(node, "end_lineno"))
+            if hasattr(node, "end_col_offset"):
+                item_text += "." + str(getattr(node, "end_col_offset"))
+
+    lines = [item_text] + [
+        pretty(field_value, field_key, level + 1) for field_key, field_value in fields
+    ]
+
+    return "\n".join(lines)
+
+
 def load_plugin() -> None:
-    get_workbench().add_view(AstView, "AST", "s")
+    get_workbench().add_view(AstView, _("Program tree"), "s")
