@@ -5,12 +5,12 @@ For performance reasons, coloring is updated in 2 phases:
     1. recolor single-line tokens on the modified line(s)
     2. recolor multi-line tokens (triple-quoted strings) in the whole text
 
-First phase may insert wrong tokens inside triple-quoted strings, but the 
-priorities of triple-quoted-string tags are higher and therefore user 
+First phase may insert wrong tokens inside triple-quoted strings, but the
+priorities of triple-quoted-string tags are higher and therefore user
 doesn't see these wrong taggings.
 
 In Shell only current command entry is colored
-    
+
 Regexes are adapted from idlelib
 """
 
@@ -20,6 +20,8 @@ from thonny import get_workbench
 from thonny.codeview import CodeViewText
 from thonny.shell import ShellText
 
+TODO = "COLOR_TODO"
+
 
 class SyntaxColorer:
     def __init__(self, text):
@@ -27,13 +29,14 @@ class SyntaxColorer:
         self._compile_regexes()
         self._config_tags()
         self._update_scheduled = False
-        self._dirty_ranges = set()
         self._use_coloring = True
+        self._multiline_dirty = True
 
     def _compile_regexes(self):
         from thonny.token_utils import (
             BUILTIN,
             COMMENT,
+            COMMENT_WITH_Q3DELIMITER,
             MAGIC_COMMAND,
             STRING3,
             STRING3_DELIMITER,
@@ -59,21 +62,19 @@ class SyntaxColorer:
             + STRING_CLOSED
             + "|"
             + STRING_OPEN,
-            re.S,
-        )  # @UndefinedVariable
+            re.S,  # @UndefinedVariable
+        )
 
+        # need to notice triple-quotes inside comments and magic commands
         self.multiline_regex = re.compile(
-            STRING3 + "|" + COMMENT + "|" + MAGIC_COMMAND
-            # + "|" + STRING_CLOSED # need to include single line strings otherwise '"""' ... '""""' will give wrong result
-            + "|"
-            + STRING_OPEN,  # (seems that it works faster and also correctly with only open strings)
-            re.S,
-        )  # @UndefinedVariable
+            "(" + STRING3 + ")|" + COMMENT_WITH_Q3DELIMITER + "|" + MAGIC_COMMAND,
+            re.S,  # @UndefinedVariable
+        )
 
         self.id_regex = re.compile(r"\s+(\w+)", re.S)  # @UndefinedVariable
 
     def _config_tags(self):
-        self.uniline_tagdefs = {
+        self.uniline_tags = {
             "comment",
             "magic",
             "string",
@@ -83,19 +84,29 @@ class SyntaxColorer:
             "builtin",
             "definition",
         }
-        self.multiline_tagdefs = {"string3", "open_string3"}
+        self.multiline_tags = {"string3", "open_string3"}
+        self._raise_tags()
+
+    def _raise_tags(self):
+        self.text.tag_raise("string3")
+        self.text.tag_raise("open_string3")
+        # yes, unclosed_expression is another plugin's issue,
+        # but it must be higher than *string3
+        self.text.tag_raise("unclosed_expression")
         self.text.tag_raise("sel")
+        """
         tags = self.text.tag_names()
         # take into account that without themes some tags may be undefined
         if "string3" in tags:
             self.text.tag_raise("string3")
         if "open_string3" in tags:
             self.text.tag_raise("open_string3")
+        """
 
-    def schedule_update(self, event, use_coloring=True):
-        self._use_coloring = use_coloring
+    def mark_dirty(self, event=None):
+        start_index = "1.0"
+        end_index = "end"
 
-        # Allow reducing work by remembering only changed lines
         if hasattr(event, "sequence"):
             if event.sequence == "TextInsert":
                 index = self.text.index(event.index)
@@ -103,37 +114,40 @@ class SyntaxColorer:
                 end_row = start_row + event.text.count("\n")
                 start_index = "%d.%d" % (start_row, 0)
                 end_index = "%d.%d" % (end_row + 1, 0)
+                if not event.trivial_for_coloring:
+                    self._multiline_dirty = True
+
             elif event.sequence == "TextDelete":
                 index = self.text.index(event.index1)
                 start_row = int(index.split(".")[0])
                 start_index = "%d.%d" % (start_row, 0)
                 end_index = "%d.%d" % (start_row + 1, 0)
-        else:
-            start_index = "1.0"
-            end_index = "end"
+                if not event.trivial_for_coloring:
+                    self._multiline_dirty = True
 
-        self._dirty_ranges.add((start_index, end_index))
+        self.text.tag_add(TODO, start_index, end_index)
 
-        def perform_update():
-            try:
-                self._update_coloring()
-            finally:
-                self._update_scheduled = False
-                self._dirty_ranges = set()
+    def schedule_update(self):
+        self._use_coloring = get_workbench().get_option("view.syntax_coloring")
 
         if not self._update_scheduled:
             self._update_scheduled = True
-            self.text.after_idle(perform_update)
+            self.text.after_idle(self.perform_update)
+
+    def perform_update(self):
+        try:
+            self._update_coloring()
+        finally:
+            self._update_scheduled = False
 
     def _update_coloring(self):
-        self._update_uniline_tokens("1.0", "end")
-        self._update_multiline_tokens("1.0", "end")
+        raise NotImplementedError()
 
     def _update_uniline_tokens(self, start, end):
         chars = self.text.get(start, end)
 
         # clear old tags
-        for tag in self.uniline_tagdefs:
+        for tag in self.uniline_tags:
             self.text.tag_remove(tag, start, end)
 
         if not self._use_coloring:
@@ -141,14 +155,12 @@ class SyntaxColorer:
 
         for match in self.uniline_regex.finditer(chars):
             for token_type, token_text in match.groupdict().items():
-                if token_text and token_type in self.uniline_tagdefs:
+                if token_text and token_type in self.uniline_tags:
                     token_text = token_text.strip()
                     match_start, match_end = match.span(token_type)
 
                     self.text.tag_add(
-                        token_type,
-                        start + "+%dc" % match_start,
-                        start + "+%dc" % match_end,
+                        token_type, start + "+%dc" % match_start, start + "+%dc" % match_end
                     )
 
                     # Mark also the word following def or class
@@ -162,70 +174,87 @@ class SyntaxColorer:
                                 start + "+%dc" % id_match_end,
                             )
 
+        self.text.tag_remove(TODO, start, end)
+
     def _update_multiline_tokens(self, start, end):
         chars = self.text.get(start, end)
         # clear old tags
-        for tag in self.multiline_tagdefs:
+        for tag in self.multiline_tags:
             self.text.tag_remove(tag, start, end)
 
         if not self._use_coloring:
             return
 
-        # Count number of open multiline strings to be able to detect when string gets closed
-        self.text.number_of_open_multiline_strings = 0
-
-        interesting_token_types = list(self.multiline_tagdefs) + ["string3"]
         for match in self.multiline_regex.finditer(chars):
-            for token_type, token_text in match.groupdict().items():
-                if token_text and token_type in interesting_token_types:
-                    token_text = token_text.strip()
-                    match_start, match_end = match.span(token_type)
-                    if token_type == "string3":
-                        if (
-                            token_text.startswith('"""')
-                            and not token_text.endswith('"""')
-                            or token_text.startswith("'''")
-                            and not token_text.endswith("'''")
-                            or len(token_text) == 3
-                        ):
-                            str_end = int(
-                                float(self.text.index(start + "+%dc" % match_end))
-                            )
-                            file_end = int(float(self.text.index("end")))
+            token_text = match.group(1)
+            if token_text is None:
+                # not string3
+                continue
 
-                            if str_end == file_end:
-                                token_type = "open_string3"
-                                self.text.number_of_open_multiline_strings += 1
-                            else:
-                                token_type = None
-                        elif len(token_text) >= 4 and token_text[-4] == "\\":
-                            token_type = "open_string3"
-                            self.text.number_of_open_multiline_strings += 1
-                        else:
-                            token_type = "string3"
+            match_start, match_end = match.span()
+            if (
+                token_text.startswith('"""')
+                and not token_text.endswith('"""')
+                or token_text.startswith("'''")
+                and not token_text.endswith("'''")
+                or len(token_text) == 3
+            ):
+                token_type = "open_string3"
+            elif len(token_text) >= 4 and token_text[-4] == "\\":
+                token_type = "open_string3"
+            else:
+                token_type = "string3"
 
-                    token_start = start + "+%dc" % match_start
-                    token_end = start + "+%dc" % match_end
-                    # clear uniline tags
-                    for tag in self.uniline_tagdefs:
-                        self.text.tag_remove(tag, token_start, token_end)
-                    # add tag
-                    self.text.tag_add(token_type, token_start, token_end)
+            token_start = start + "+%dc" % match_start
+            token_end = start + "+%dc" % match_end
+            self.text.tag_add(token_type, token_start, token_end)
+
+        self._multiline_dirty = False
+        self._raise_tags()
 
 
 class CodeViewSyntaxColorer(SyntaxColorer):
     def _update_coloring(self):
-        for dirty_range in self._dirty_ranges:
-            self._update_uniline_tokens(*dirty_range)
+        search_start = self.text.index("@0,0")
+        search_end = self.text.index(
+            "@%d,%d lineend" % (self.text.winfo_width(), self.text.winfo_height())
+        )
+
+        while True:
+            res = self.text.tag_nextrange(TODO, search_start, search_end)
+            if res:
+                update_start = res[0]
+                update_end = res[1]
+            else:
+                # maybe the range started earlier
+                res = self.text.tag_prevrange(TODO, search_start)
+                if res and self.text.compare(res[1], ">", search_end):
+                    update_start = search_start
+                    update_end = res[1]
+                else:
+                    break
+
+            if self.text.compare(update_end, ">", search_end):
+                update_end = search_end
+
+            self._update_uniline_tokens(update_start, update_end)
+
+            if update_end == search_end:
+                break
+            else:
+                search_start = update_end
 
         # Multiline tokens need to be searched from the whole source
-        open_before = getattr(self.text, "number_of_open_multiline_strings", 0)
-        self._update_multiline_tokens("1.0", "end")
-        open_after = getattr(self.text, "number_of_open_multiline_strings", 0)
+        # open_before = getattr(self.text, "number_of_open_multiline_strings", 0)
+        if self._multiline_dirty:
+            self._update_multiline_tokens("1.0", "end")
+        # open_after = getattr(self.text, "number_of_open_multiline_strings", 0)
 
+        """
         if open_after == 0 and open_before != 0:
             # recolor uniline tokens after closing last open multiline string
             self._update_uniline_tokens("1.0", "end")
+        """
 
 
 class ShellSyntaxColorer(SyntaxColorer):
@@ -246,12 +275,16 @@ class ShellSyntaxColorer(SyntaxColorer):
             self._update_multiline_tokens(start_index, end_index)
 
 
-def update_coloring(event):
+def update_coloring_on_event(event):
     if hasattr(event, "text_widget"):
         text = event.text_widget
     else:
         text = event.widget
 
+    update_coloring_on_text(text, event)
+
+
+def update_coloring_on_text(text, event=None):
     if not hasattr(text, "syntax_colorer"):
         if isinstance(text, ShellText):
             class_ = ShellSyntaxColorer
@@ -261,16 +294,19 @@ def update_coloring(event):
             return
 
         text.syntax_colorer = class_(text)
+        # mark whole text as unprocessed
+        text.syntax_colorer.mark_dirty()
+    else:
+        text.syntax_colorer.mark_dirty(event)
 
-    text.syntax_colorer.schedule_update(
-        event, get_workbench().get_option("view.syntax_coloring")
-    )
+    text.syntax_colorer.schedule_update()
 
 
 def load_plugin() -> None:
     wb = get_workbench()
 
     wb.set_default("view.syntax_coloring", True)
-    wb.bind("TextInsert", update_coloring, True)
-    wb.bind("TextDelete", update_coloring, True)
-    wb.bind("<<UpdateAppearance>>", update_coloring, True)
+    wb.bind("TextInsert", update_coloring_on_event, True)
+    wb.bind("TextDelete", update_coloring_on_event, True)
+    wb.bind_class("CodeViewText", "<<VerticalScroll>>", update_coloring_on_event, True)
+    wb.bind("<<UpdateAppearance>>", update_coloring_on_event, True)
