@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 """
-Adds debugging commands and features. 
+Adds debugging commands and features.
 """
 
 import ast
@@ -12,25 +12,18 @@ from tkinter import ttk
 from tkinter.messagebox import showinfo
 from typing import List, Union  # @UnusedImport
 
-from thonny import (
-    ast_utils,
-    code,
-    get_runner,
-    get_workbench,
-    memory,
-    misc_utils,
-    ui_utils,
-)
-from thonny.codeview import CodeView, get_syntax_options_for_tag
+from thonny import ast_utils, code, get_runner, get_workbench, memory, misc_utils, ui_utils, running
+from thonny.codeview import CodeView, get_syntax_options_for_tag, SyntaxText
 from thonny.common import DebuggerCommand, InlineCommand
-from thonny.config_ui import ConfigurationPage
 from thonny.memory import VariablesFrame
-from thonny.misc_utils import shorten_repr
+from thonny.misc_utils import shorten_repr, running_on_rpi, running_on_mac_os
 from thonny.tktextext import TextFrame
-from thonny.ui_utils import select_sequence
+from thonny.ui_utils import select_sequence, CommonDialog
 
 _current_debugger = None
 
+RESUME_COMMAND_CAPTION = ""  # Init later when gettext is loaded
+
 
 class Debugger:
     def __init__(self):
@@ -50,14 +43,15 @@ class Debugger:
                 frame_id=self._last_progress_message.stack[-1].id,
                 breakpoints=code.get_current_breakpoints(),
                 cursor_position=self.get_run_to_cursor_breakpoint(),
-            )
-
-            cmd.setdefault(
                 state=self._last_progress_message.stack[-1].event,
                 focus=self._last_progress_message.stack[-1].focus,
+                allow_stepping_into_libraries=get_workbench().get_option(
+                    "debugger.allow_stepping_into_libraries"
+                ),
             )
-
             get_runner().send_command(cmd)
+            if command == "resume":
+                self.clear_last_frame()
         else:
             logging.debug("Bad state for sending debugger command " + str(command))
 
@@ -71,20 +65,28 @@ class Debugger:
         if command == "run_to_cursor":
             return self.get_run_to_cursor_breakpoint() is not None
         elif command == "step_back":
-            return (self._last_progress_message 
-                    and self._last_progress_message["tracer_class"] == "NiceTracer")
+            return (
+                self._last_progress_message
+                and self._last_progress_message["tracer_class"] == "NiceTracer"
+            )
         else:
             return True
 
     def handle_debugger_progress(self, msg):
         self._last_brought_out_frame_id = None
 
+    def handle_debugger_return(self, msg):
+        pass
+
     def close(self) -> None:
         self._last_brought_out_frame_id = None
 
         if get_workbench().get_option("debugger.automatic_stack_view"):
             get_workbench().hide_view("StackView")
 
+    def clear_last_frame(self):
+        pass
+
     def get_frame_by_id(self, frame_id):
         for frame_info in self._last_progress_message.stack:
             if frame_info.id == frame_id:
@@ -111,16 +113,14 @@ class Debugger:
             menu = tk.Menu(get_workbench())
             menu.add(
                 "command",
-                label="Run to cursor",
+                label=_("Run to cursor"),
                 command=lambda: self.check_issue_command("run_to_cursor"),
             )
             menu.add("separator")
-            menu.add(
-                "command", label="Copy", command=create_edit_command_handler("<<Copy>>")
-            )
+            menu.add("command", label="Copy", command=create_edit_command_handler("<<Copy>>"))
             menu.add(
                 "command",
-                label="Select all",
+                label=_("Select all"),
                 command=create_edit_command_handler("<<SelectAll>>"),
             )
             self._editor_context_menu = menu
@@ -165,6 +165,10 @@ class SingleWindowDebugger(Debugger):
             self._last_frame_visualizer.close()
             self._last_frame_visualizer = None
 
+    def clear_last_frame(self):
+        if self._last_frame_visualizer is not None:
+            self._last_frame_visualizer.clear()
+
     def bring_out_frame(self, frame_id, force=False):
         if not force and frame_id == self._last_brought_out_frame_id:
             return
@@ -183,9 +187,7 @@ class SingleWindowDebugger(Debugger):
         if self._last_frame_visualizer is None:
             self._last_frame_visualizer = EditorVisualizer(frame_info)
 
-        self._last_frame_visualizer._update_this_frame(
-            self._last_progress_message, frame_info
-        )
+        self._last_frame_visualizer._update_this_frame(self._last_progress_message, frame_info)
 
         # show variables
         var_view = get_workbench().get_view("VariablesView")
@@ -201,6 +203,12 @@ class SingleWindowDebugger(Debugger):
                 else frame_info.code_name,
             )
 
+    def handle_debugger_return(self, msg):
+        if self._last_frame_visualizer is not None and self._last_frame_visualizer.get_frame_id() == msg.get(
+            "frame_id"
+        ):
+            self._last_frame_visualizer.close()
+
 
 class StackedWindowsDebugger(Debugger):
     def __init__(self):
@@ -251,7 +259,12 @@ class StackedWindowsDebugger(Debugger):
             self._main_frame_visualizer.close()
             self._main_frame_visualizer = None
 
-    def _get_topmost_selected_visualizer(self):
+    def clear_last_frame(self):
+        visualizer = self._get_topmost_visualizer()
+        if visualizer is not None:
+            visualizer.clear()
+
+    def _get_topmost_visualizer(self):
         visualizer = self._main_frame_visualizer
         if visualizer is None:
             return None
@@ -259,6 +272,13 @@ class StackedWindowsDebugger(Debugger):
         while visualizer._next_frame_visualizer is not None:
             visualizer = visualizer._next_frame_visualizer
 
+        return visualizer
+
+    def _get_topmost_selected_visualizer(self):
+        visualizer = self._get_topmost_visualizer()
+        if visualizer is None:
+            return None
+
         topmost_text_widget = visualizer._text
         focused_widget = get_workbench().focus_get()
 
@@ -282,6 +302,14 @@ class StackedWindowsDebugger(Debugger):
         frame_info = self.get_frame_by_id(frame_id)
         var_view.show_globals(frame_info.globals, frame_info.module_name)
 
+    def handle_debugger_return(self, msg):
+        if self._main_frame_visualizer is None:
+            return
+
+        self._main_frame_visualizer.close(msg["frame_id"])
+        if msg["frame_id"] == self._main_frame_visualizer.get_frame_id():
+            self._main_frame_visualizer = None
+
 
 class FrameVisualizer:
     """
@@ -299,14 +327,15 @@ class FrameVisualizer:
         self._expression_box = ExpressionBox(text_frame)
         self._note_box = ui_utils.NoteBox(text_frame.winfo_toplevel())
         self._next_frame_visualizer = None
+        self._prev_frame_visualizer = None
         self._text.set_read_only(True)
         self._line_debug = frame_info.current_statement is None
 
         self._reconfigure_tags()
-    
+
     def _translate_lineno(self, lineno):
         return lineno - self._firstlineno + 1
-        
+
     def _reconfigure_tags(self):
         for tag in ["active_focus", "exception_focus"]:
             conf = get_syntax_options_for_tag(tag).copy()
@@ -316,14 +345,20 @@ class FrameVisualizer:
 
             self._text.tag_configure(tag, **conf)
 
-    def close(self):
+    def close(self, frame_id=None):
         if self._next_frame_visualizer:
-            self._next_frame_visualizer.close()
-            self._next_frame_visualizer = None
+            self._next_frame_visualizer.close(frame_id)
+            if frame_id is None or self._next_frame_visualizer.get_frame_id() == frame_id:
+                self._next_frame_visualizer = None
 
-        self._text.set_read_only(False)
-        self._remove_focus_tags()
-        self._expression_box.clear_debug_view()
+        if frame_id is None or frame_id == self._frame_id:
+            self._text.set_read_only(False)
+            self.clear()
+            # self._expression_box.destroy()
+
+    def clear(self):
+        self.remove_focus_tags()
+        self.hide_expression_box()
         self.close_note()
 
     def get_frame_id(self):
@@ -339,21 +374,19 @@ class FrameVisualizer:
 
         # clear obsolete next frame visualizer
         if self._next_frame_visualizer and (
-            not next_frame_info
-            or self._next_frame_visualizer.get_frame_id() != next_frame_info.id
+            not next_frame_info or self._next_frame_visualizer.get_frame_id() != next_frame_info.id
         ):
             self._next_frame_visualizer.close()
             self._next_frame_visualizer = None
 
         if next_frame_info and not self._next_frame_visualizer:
-            self._next_frame_visualizer = self._create_next_frame_visualizer(
-                next_frame_info
-            )
+            self._next_frame_visualizer = self._create_next_frame_visualizer(next_frame_info)
+            self._next_frame_visualizer._prev_frame_visualizer = self
 
         if self._next_frame_visualizer:
             self._next_frame_visualizer.update_this_and_next_frames(msg)
 
-    def _remove_focus_tags(self):
+    def remove_focus_tags(self):
         for name in [
             "exception_focus",
             "active_focus",
@@ -363,20 +396,25 @@ class FrameVisualizer:
         ]:
             self._text.tag_remove(name, "0.0", "end")
 
+    def hide_expression_box(self):
+        if self._expression_box is not None:
+            self._expression_box.clear_debug_view()
+
     def _update_this_frame(self, msg, frame_info):
         self._frame_info = frame_info
-        self._remove_focus_tags()
+        self.remove_focus_tags()
 
         if frame_info.event == "line":
-            if (frame_info.id in msg["exception_info"]["affected_frame_ids"]
-                and msg["exception_info"]["is_fresh"]):
+            if (
+                frame_info.id in msg["exception_info"]["affected_frame_ids"]
+                and msg["exception_info"]["is_fresh"]
+            ):
                 self._tag_range(frame_info.focus, "exception_focus")
             else:
                 self._tag_range(frame_info.focus, "active_focus")
         else:
             if "statement" in frame_info.event:
-                if (msg["exception_info"]["msg"] is not None
-                    and msg["exception_info"]["is_fresh"]):
+                if msg["exception_info"]["msg"] is not None and msg["exception_info"]["is_fresh"]:
                     stmt_tag = "exception_focus"
                 elif frame_info.event.startswith("before"):
                     stmt_tag = "active_focus"
@@ -386,15 +424,18 @@ class FrameVisualizer:
                 assert "expression" in frame_info.event
                 stmt_tag = "suspended_focus"
 
-            self._tag_range(frame_info.current_statement, stmt_tag)
+            if frame_info.current_statement is not None:
+                self._tag_range(frame_info.current_statement, stmt_tag)
+            else:
+                logging.warning("Missing current_statement: %s", frame_info)
 
         self._expression_box.update_expression(msg, frame_info)
 
-        if (frame_info.id in msg["exception_info"]["affected_frame_ids"] 
-            and msg["exception_info"]["is_fresh"]):
-            self._show_exception(
-                msg["exception_info"]["lines_with_frame_info"], frame_info
-            )
+        if (
+            frame_info.id in msg["exception_info"]["affected_frame_ids"]
+            and msg["exception_info"]["is_fresh"]
+        ):
+            self._show_exception(msg["exception_info"]["lines_with_frame_info"], frame_info)
         else:
             self.close_note()
 
@@ -420,6 +461,8 @@ class FrameVisualizer:
         # For most statements I want to highlight block of whole lines
         # but for pseudo-statements (like header in for-loop) I want to highlight only the indicated range
 
+        self._text.tag_raise(tag)
+
         line_prefix = self._text.get(
             "%d.0" % self._translate_lineno(text_range.lineno),
             "%d.%d" % (self._translate_lineno(text_range.lineno), text_range.col_offset),
@@ -438,9 +481,7 @@ class FrameVisualizer:
             first_line, first_col, last_line = self._get_text_range_block(text_range)
 
             for lineno in range(first_line, last_line + 1):
-                self._text.tag_add(
-                    tag, "%d.%d" % (lineno, first_col), "%d.0" % (lineno + 1)
-                )
+                self._text.tag_add(tag, "%d.%d" % (lineno, first_col), "%d.0" % (lineno + 1))
 
         self._text.update_idletasks()
         self._text.see("%d.0" % (last_line))
@@ -456,9 +497,7 @@ class FrameVisualizer:
     def _get_text_range_block(self, text_range):
         first_line = text_range.lineno - self._firstlineno + 1
         last_line = (
-            text_range.end_lineno
-            - self._firstlineno
-            + (1 if text_range.end_col_offset > 0 else 0)
+            text_range.end_lineno - self._firstlineno + (1 if text_range.end_col_offset > 0 else 0)
         )
         first_line_content = self._text.get("%d.0" % first_line, "%d.end" % first_line)
         if first_line_content.strip().startswith("elif "):
@@ -490,9 +529,7 @@ class FrameVisualizer:
     def bring_out_this_frame(self):
         pass
 
-    def show_note(
-        self, *content_items: Union[str, List], target=None, focus=None
-    ) -> None:
+    def show_note(self, *content_items: Union[str, List], target=None, focus=None) -> None:
         if target is None:
             target = self._text
 
@@ -504,15 +541,13 @@ class FrameVisualizer:
 
 class EditorVisualizer(FrameVisualizer):
     """
-    Takes care of stepping in the editor 
-    (main module in case of StackedWindowsDebugger) 
+    Takes care of stepping in the editor
+    (main module in case of StackedWindowsDebugger)
     """
 
     def __init__(self, frame_info):
         self.editor = (
-            get_workbench()
-            .get_editor_notebook()
-            .show_file(frame_info.filename, set_focus=False)
+            get_workbench().get_editor_notebook().show_file(frame_info.filename, set_focus=False)
         )
         FrameVisualizer.__init__(self, self.editor.get_code_view(), frame_info)
         self._firstlineno = 1
@@ -525,15 +560,13 @@ class EditorVisualizer(FrameVisualizer):
             self._decorate_editor_title("   <<< REPLAYING >>> ")
 
     def _decorate_editor_title(self, suffix):
-        self.editor.master.update_editor_title(
-            self.editor, self.editor.get_title() + suffix
-        )
+        self.editor.master.update_editor_title(self.editor, self.editor.get_title() + suffix)
 
     def bring_out_this_frame(self):
         get_workbench().focus_set()
 
-    def close(self):
-        super().close()
+    def clear(self):
+        super().clear()
         self._decorate_editor_title("")
 
 
@@ -573,8 +606,17 @@ class ExpressionBox(tk.Text):
         event = frame_info.event
 
         if frame_info.current_root_expression is not None:
+            with open(frame_info.filename, "rb") as fp:
+                whole_source = fp.read()
+
+            lines = whole_source.splitlines()
+            if len(lines) < frame_info.current_root_expression.end_lineno:
+                # it must be on a synthetical line which is not actually present in the editor
+                self.clear_debug_view()
+                return
+
             self._load_expression(
-                frame_info.filename, frame_info.current_root_expression
+                whole_source, frame_info.filename, frame_info.current_root_expression
             )
             for subrange, value in frame_info.current_evaluations:
                 self._replace(subrange, value)
@@ -583,8 +625,10 @@ class ExpressionBox(tk.Text):
                 self._highlight_range(
                     focus,
                     event,
-                    (frame_info.id in msg["exception_info"]["affected_frame_ids"]
-                     and msg["exception_info"]["is_fresh"]),
+                    (
+                        frame_info.id in msg["exception_info"]["affected_frame_ids"]
+                        and msg["exception_info"]["is_fresh"]
+                    ),
                 )
 
             self._update_position(frame_info.current_root_expression)
@@ -599,9 +643,7 @@ class ExpressionBox(tk.Text):
 
     def get_focused_text(self):
         if self._last_focus:
-            start_mark = self._get_mark_name(
-                self._last_focus.lineno, self._last_focus.col_offset
-            )
+            start_mark = self._get_mark_name(self._last_focus.lineno, self._last_focus.col_offset)
             end_mark = self._get_mark_name(
                 self._last_focus.end_lineno, self._last_focus.end_col_offset
             )
@@ -610,7 +652,8 @@ class ExpressionBox(tk.Text):
             return ""
 
     def clear_debug_view(self):
-        self.place_forget()
+        if self.winfo_ismapped():
+            self.place_forget()
         self._main_range = None
         self._last_focus = None
         self._clear_expression()
@@ -643,14 +686,10 @@ class ExpressionBox(tk.Text):
         self.tag_bind(
             object_tag,
             sequence,
-            lambda _: get_workbench().event_generate(
-                "ObjectSelect", object_id=value.id
-            ),
+            lambda _: get_workbench().event_generate("ObjectSelect", object_id=value.id),
         )
 
-    def _load_expression(self, filename, text_range):
-        with open(filename, "rb") as fp:
-            whole_source = fp.read()
+    def _load_expression(self, whole_source, filename, text_range):
 
         root = ast_utils.parse_source(whole_source, filename)
         main_node = ast_utils.find_expression(root, text_range)
@@ -717,9 +756,7 @@ class ExpressionBox(tk.Text):
             return
 
         start_index = self._get_mark_name(text_range.lineno, text_range.col_offset)
-        end_index = self._get_mark_name(
-            text_range.end_lineno, text_range.end_col_offset
-        )
+        end_index = self._get_mark_name(text_range.end_lineno, text_range.end_col_offset)
         self.tag_add(tag, start_index, end_index)
 
         if has_exception:
@@ -728,9 +765,7 @@ class ExpressionBox(tk.Text):
     def _update_position(self, text_range):
         self._codeview.update_idletasks()
         text_line_number = text_range.lineno - self._codeview._first_line_number + 1
-        bbox = self._codeview.text.bbox(
-            str(text_line_number) + "." + str(text_range.col_offset)
-        )
+        bbox = self._codeview.text.bbox(str(text_line_number) + "." + str(text_range.col_offset))
 
         if isinstance(bbox, tuple):
             x = bbox[0] - self._codeview.text.cget("padx") + 6
@@ -755,9 +790,9 @@ class ExpressionBox(tk.Text):
         self["width"] = max(map(len, lines))
 
 
-class DialogVisualizer(tk.Toplevel, FrameVisualizer):
+class DialogVisualizer(CommonDialog, FrameVisualizer):
     def __init__(self, master, frame_info):
-        tk.Toplevel.__init__(self, master)
+        super().__init__(master)
 
         self.transient(master)
         if misc_utils.running_on_windows():
@@ -796,9 +831,7 @@ class DialogVisualizer(tk.Toplevel, FrameVisualizer):
         self.main_frame.grid(sticky=tk.NSEW)
         self.rowconfigure(0, weight=1)
         self.columnconfigure(0, weight=1)
-        self.main_pw = ui_utils.AutomaticPanedWindow(
-            self.main_frame, orient=tk.VERTICAL
-        )
+        self.main_pw = ui_utils.AutomaticPanedWindow(self.main_frame, orient=tk.VERTICAL)
         self.main_pw.grid(sticky=tk.NSEW, padx=10, pady=10)
         self.main_frame.rowconfigure(0, weight=1)
         self.main_frame.columnconfigure(0, weight=1)
@@ -828,14 +861,16 @@ class DialogVisualizer(tk.Toplevel, FrameVisualizer):
 
     def _on_close(self):
         showinfo(
-            "Can't close yet", 
-            'Use "Stop" command if you want to cancel debugging',
-            parent=self
+            _("Can't close yet"),
+            _('Use "Stop" command if you want to cancel debugging'),
+            parent=None if running_on_mac_os() else self,
         )
 
-    def close(self):
-        FrameVisualizer.close(self)
-        self.destroy()
+    def close(self, frame_id=None):
+        super().close()
+
+        if frame_id is None or frame_id == self._frame_id:
+            self.destroy()
 
 
 class FunctionCallDialog(DialogVisualizer):
@@ -871,9 +906,7 @@ class ModuleLoadDialog(DialogVisualizer):
 class StackView(ui_utils.TreeFrame):
     def __init__(self, master):
         super().__init__(
-            master,
-            ("function", "location", "id"),
-            displaycolumns=("function", "location"),
+            master, ("function", "location", "id"), displaycolumns=("function", "location")
         )
 
         # self.tree.configure(show="tree")
@@ -885,28 +918,38 @@ class StackView(ui_utils.TreeFrame):
         self.tree.heading("location", text="Location", anchor=tk.W)
 
         get_workbench().bind("DebuggerResponse", self._update_stack, True)
-        get_workbench().bind(
-            "ToplevelResponse", lambda e=None: self._clear_tree(), True
-        )
+        get_workbench().bind("ToplevelResponse", lambda e=None: self._clear_tree(), True)
+        get_workbench().bind("debugger_return_response", self._handle_debugger_return, True)
 
     def _update_stack(self, msg):
         self._clear_tree()
+
+        node_id = None
         for frame in msg.stack:
             lineno = frame.focus.lineno
 
             node_id = self.tree.insert("", "end")
             self.tree.set(node_id, "function", frame.code_name)
             self.tree.set(
-                node_id,
-                "location",
-                "%s, line %s" % (os.path.basename(frame.filename), lineno),
+                node_id, "location", "%s, line %s" % (os.path.basename(frame.filename), lineno)
             )
             self.tree.set(node_id, "id", frame.id)
 
         # select last frame
-        self.tree.see(node_id)
-        self.tree.selection_add(node_id)
-        self.tree.focus(node_id)
+        if node_id is not None:
+            self.tree.see(node_id)
+            self.tree.selection_add(node_id)
+            self.tree.focus(node_id)
+
+    def _handle_debugger_return(self, msg):
+        delete = False
+        for iid in self.tree.get_children():
+            if self.tree.set(iid, "id") == msg["frame_id"]:
+                # start deleting from this frame
+                delete = True
+
+            if delete:
+                self.tree.delete(iid)
 
     def on_select(self, event):
         iid = self.tree.focus()
@@ -926,6 +969,7 @@ class ExceptionView(TextFrame):
             undo=False,
             read_only=True,
             font="TkDefaultFont",
+            text_class=SyntaxText,
             foreground=get_syntax_options_for_tag("stderr")["foreground"],
             highlightthickness=0,
             padx=5,
@@ -946,8 +990,7 @@ class ExceptionView(TextFrame):
     def _show_description(self):
         self.text.configure(foreground=get_syntax_options_for_tag("TEXT")["foreground"])
         self.text.direct_insert(
-            "end",
-            "If last command raised an exception then this view will show the stacktrace.",
+            "end", "If last command raised an exception then this view will show the stacktrace."
         )
 
     def set_exception(self, exception_lines_with_frame_info):
@@ -960,20 +1003,14 @@ class ExceptionView(TextFrame):
             self._show_description()
             return
 
-        self.text.configure(
-            foreground=get_syntax_options_for_tag("stderr")["foreground"]
-        )
+        self.text.configure(foreground=get_syntax_options_for_tag("stderr")["foreground"])
         for line, frame_id, filename, lineno in exception_lines_with_frame_info:
 
             if frame_id is not None:
                 frame_tag = "frame_%d" % frame_id
 
-                def handle_frame_click(
-                    event, frame_id=frame_id, filename=filename, lineno=lineno
-                ):
-                    get_runner().send_command(
-                        InlineCommand("get_frame_info", frame_id=frame_id)
-                    )
+                def handle_frame_click(event, frame_id=frame_id, filename=filename, lineno=lineno):
+                    get_runner().send_command(InlineCommand("get_frame_info", frame_id=frame_id))
                     if os.path.exists(filename):
                         get_workbench().get_editor_notebook().show_file(
                             filename, lineno, set_focus=False
@@ -987,9 +1024,7 @@ class ExceptionView(TextFrame):
                     end = len(line)
 
                 self.text.direct_insert("end", line[:start])
-                self.text.direct_insert(
-                    "end", line[start:end], ("hyperlink", frame_tag)
-                )
+                self.text.direct_insert("end", line[start:end], ("hyperlink", frame_tag))
                 self.text.direct_insert("end", line[end:])
 
             else:
@@ -1035,89 +1070,151 @@ def _start_debug_enabled():
 def _request_debug(command_name):
     # Don't assume Debug command gets issued after this
     # This may just call the %cd command
-    # or the user may deny saving current editor 
+    # or the user may deny saving current editor
+    if get_workbench().in_simple_mode():
+        get_workbench().show_view("VariablesView")
+
     get_runner().execute_current(command_name)
 
+
 def _debug_accepted(event):
     # Called when proxy accepted the debug command
     global _current_debugger
     cmd = event.command
     if cmd.get("name") in ["Debug", "FastDebug"]:
         assert _current_debugger is None
-        if get_workbench().get_option("debugger.single_window"):
-            _current_debugger = SingleWindowDebugger()
-        else:
+        if get_workbench().get_option("debugger.frames_in_separate_windows"):
             _current_debugger = StackedWindowsDebugger()
+        else:
+            _current_debugger = SingleWindowDebugger()
+
 
 def _handle_debugger_progress(msg):
     global _current_debugger
     assert _current_debugger is not None
     _current_debugger.handle_debugger_progress(msg)
+    _update_run_or_resume_button()
+
 
 def _handle_toplevel_response(msg):
     global _current_debugger
     if _current_debugger is not None:
         _current_debugger.close()
-        _current_debugger = None    
+        _current_debugger = None
 
-class DebuggerConfigurationPage(ConfigurationPage):
-    def __init__(self, master):
-        super().__init__(master)
-        self.add_checkbox(
-            "debugger.single_window",
-            "Use editors and Stack view for presenting call frames",
-            tooltip="Presents the concept of stack like most professional IDE-s",
-            row=1,
-            columnspan=3
-        )
-        self.add_checkbox(
-            "debugger.automatic_stack_view",
-            "Open and close Stack view automatically",
-            tooltip="Opens the Stack view on first call and "
-            + "closes it when program returns to main frame.",
-            row=2,
-            columnspan=3
-        )
-        
-        if get_workbench().get_option("run.birdseye_port", None):
-            port_label = ttk.Label(self, text="Birdseye port", anchor="w")
-            port_label.grid(row=3, column=0, sticky="w", pady=5)
-            self.add_entry("run.birdseye_port", row=3, column=1, width=5, pady=15, padx=5)
-            comment_label = ttk.Label(self, text="(restart Thonny after changing this)", anchor="w")
-            comment_label.grid(row=3, column=2, sticky="w", pady=5)
-            
-        self.columnconfigure(2, weight=1)
-        
+    _update_run_or_resume_button()
+
+
+def _handle_debugger_return(msg):
+    global _current_debugger
+    assert _current_debugger is not None
+    _current_debugger.handle_debugger_return(msg)
+
+
+def _run_or_resume():
+    state = get_runner().get_state()
+    if state == "waiting_debugger_command":
+        _issue_debugger_command("resume")
+    elif state == "waiting_toplevel_command":
+        get_runner().cmd_run_current_script()
+
+
+def _run_or_resume_enabled():
+    return get_runner().cmd_run_current_script_enabled() or _debugger_command_enabled("resume")
+
+
+def _update_run_or_resume_button():
+    if not get_workbench().in_simple_mode():
+        return
+
+    state = get_runner().get_state()
+    if state == "waiting_debugger_command":
+        caption = RESUME_COMMAND_CAPTION
+        image = get_workbench().get_image("resume")
+    elif state == "waiting_toplevel_command":
+        caption = running.RUN_COMMAND_CAPTION
+        image = get_workbench().get_image("run-current-script")
+    else:
+        return
+
+    button = get_workbench().get_toolbar_button("runresume")
+    button.configure(text=caption, image=image)
 
 
 def get_current_debugger():
     return _current_debugger
 
 
+def run_preferred_debug_command():
+    preferred_debugger = get_workbench().get_option("debugger.preferred_debugger").lower()
+    if preferred_debugger == "faster":
+        return _request_debug("FastDebug")
+    elif preferred_debugger == "birdseye":
+        from thonny.plugins.birdseye_frontend import debug_with_birdseye
+
+        return debug_with_birdseye()
+    else:
+        return _request_debug("Debug")
+
+
 def load_plugin() -> None:
 
-    get_workbench().set_default("debugger.single_window", False)
+    global RESUME_COMMAND_CAPTION
+    RESUME_COMMAND_CAPTION = _("Resume")
+
+    get_workbench().set_default("debugger.frames_in_separate_windows", True)
     get_workbench().set_default("debugger.automatic_stack_view", True)
+    get_workbench().set_default(
+        "debugger.preferred_debugger", "faster" if running_on_rpi() else "nicer"
+    )
+    get_workbench().set_default("debugger.allow_stepping_into_libraries", False)
 
     get_workbench().add_command(
-        "debug",
+        "runresume",
         "run",
-        "Debug current script (nicer)",
-        lambda: _request_debug("Debug"),
-        caption="Debug",
+        _("Run / resume"),
+        _run_or_resume,
+        caption=running.RUN_COMMAND_CAPTION,
+        tester=_run_or_resume_enabled,
+        default_sequence=None,
+        group=10,
+        image="run-current-script",
+        include_in_menu=False,
+        include_in_toolbar=get_workbench().in_simple_mode(),
+        alternative_caption=RESUME_COMMAND_CAPTION,
+    )
+
+    get_workbench().add_command(
+        "debug_preferred",
+        "run",
+        _("Debug current script"),
+        run_preferred_debug_command,
+        caption=_("Debug"),
         tester=_start_debug_enabled,
-        default_sequence="<Control-F5>",
         group=10,
         image="debug-current-script",
+        include_in_menu=False,
         include_in_toolbar=True,
     )
 
     get_workbench().add_command(
-        "debuglite",
+        "debug_nicer",
         "run",
-        "Debug current script (faster)",
+        _("Debug current script (nicer)"),
+        lambda: _request_debug("Debug"),
+        caption="Debug (nicer)",
+        tester=_start_debug_enabled,
+        default_sequence="<Control-F5>",
+        group=10,
+        # image="debug-current-script",
+    )
+
+    get_workbench().add_command(
+        "debug_faster",
+        "run",
+        _("Debug current script (faster)"),
         lambda: _request_debug("FastDebug"),
-        caption="Fast-debug",
+        caption="Debug (faster)",
         tester=_start_debug_enabled,
         default_sequence="<Shift-F5>",
         group=10,
@@ -1126,9 +1223,9 @@ def load_plugin() -> None:
     get_workbench().add_command(
         "step_over",
         "run",
-        "Step over",
+        _("Step over"),
         lambda: _issue_debugger_command("step_over"),
-        caption="Over",
+        caption=_("Over"),
         tester=lambda: _debugger_command_enabled("step_over"),
         default_sequence="<F6>",
         group=30,
@@ -1139,9 +1236,9 @@ def load_plugin() -> None:
     get_workbench().add_command(
         "step_into",
         "run",
-        "Step into",
+        _("Step into"),
         lambda: _issue_debugger_command("step_into"),
-        caption="Into",
+        caption=_("Into"),
         tester=lambda: _debugger_command_enabled("step_into"),
         default_sequence="<F7>",
         group=30,
@@ -1152,9 +1249,9 @@ def load_plugin() -> None:
     get_workbench().add_command(
         "step_out",
         "run",
-        "Step out",
+        _("Step out"),
         lambda: _issue_debugger_command("step_out"),
-        caption="Out",
+        caption=_("Out"),
         tester=lambda: _debugger_command_enabled("step_out"),
         group=30,
         image="step-out",
@@ -1164,20 +1261,20 @@ def load_plugin() -> None:
     get_workbench().add_command(
         "resume",
         "run",
-        "Resume",
+        RESUME_COMMAND_CAPTION,
         lambda: _issue_debugger_command("resume"),
-        caption="Resume",
+        caption=RESUME_COMMAND_CAPTION,
         tester=lambda: _debugger_command_enabled("resume"),
         default_sequence="<F8>",
         group=30,
         image="resume",
-        include_in_toolbar=True,
+        include_in_toolbar=not get_workbench().in_simple_mode(),
     )
 
     get_workbench().add_command(
         "run_to_cursor",
         "run",
-        "Run to cursor",
+        _("Run to cursor"),
         lambda: _issue_debugger_command("run_to_cursor"),
         tester=lambda: _debugger_command_enabled("run_to_cursor"),
         default_sequence=select_sequence("<Control-F8>", "<Control-F8>"),
@@ -1189,18 +1286,17 @@ def load_plugin() -> None:
     get_workbench().add_command(
         "step_back",
         "run",
-        "Step back",
+        _("Step back"),
         lambda: _issue_debugger_command("step_back"),
-        caption="Back",
+        caption=_("Back"),
         tester=lambda: _debugger_command_enabled("step_back"),
         default_sequence=select_sequence("<Control-b>", "<Command-b>"),
         group=30,
     )
 
-    get_workbench().add_view(StackView, "Stack", "se")
-    get_workbench().add_view(ExceptionView, "Exception", "s")
-    get_workbench().add_configuration_page("Debugger", DebuggerConfigurationPage)
+    get_workbench().add_view(StackView, _("Stack"), "se")
+    get_workbench().add_view(ExceptionView, _("Exception"), "s")
     get_workbench().bind("DebuggerResponse", _handle_debugger_progress, True)
     get_workbench().bind("ToplevelResponse", _handle_toplevel_response, True)
+    get_workbench().bind("debugger_return_response", _handle_debugger_return, True)
     get_workbench().bind("CommandAccepted", _debug_accepted, True)
-    
