@@ -20,12 +20,7 @@ from urllib.request import urlopen, urlretrieve
 
 import thonny
 from thonny import get_runner, get_workbench, running, tktextext, ui_utils
-from thonny.common import (
-    InlineCommand,
-    normpath_with_actual_case,
-    is_same_path,
-    path_startswith,
-)
+from thonny.common import InlineCommand, normpath_with_actual_case, is_same_path, path_startswith
 from thonny.ui_utils import (
     AutoScrollbar,
     SubprocessDialog,
@@ -34,19 +29,20 @@ from thonny.ui_utils import (
     lookup_style_option,
     open_path_in_system_file_manager,
     scrollbar_style,
+    CommonDialog,
 )
 
 PIP_INSTALLER_URL = "https://bootstrap.pypa.io/get-pip.py"
 
 
-class PipDialog(tk.Toplevel):
+class PipDialog(CommonDialog):
     def __init__(self, master):
         self._state = "idle"  # possible values: "listing", "fetching", "idle"
         self._process = None
         self._active_distributions = {}
         self.current_package_data = None
 
-        tk.Toplevel.__init__(self, master)
+        super().__init__(master)
 
         main_frame = ttk.Frame(self)
         main_frame.grid(sticky=tk.NSEW, ipadx=15, ipady=15)
@@ -79,10 +75,7 @@ class PipDialog(tk.Toplevel):
         self.search_box.bind("<Return>", self._on_search, False)
 
         self.search_button = ttk.Button(
-            header_frame,
-            text="Find package from PyPI",
-            command=self._on_search,
-            width=25,
+            header_frame, text=_("Find package from PyPI"), command=self._on_search, width=25
         )
         self.search_button.grid(row=1, column=1, sticky="nse", padx=(10, 0))
 
@@ -104,7 +97,7 @@ class PipDialog(tk.Toplevel):
             listframe,
             activestyle="dotbox",
             width=20,
-            height=18,
+            height=20,
             selectborderwidth=0,
             relief="flat",
             # highlightthickness=4,
@@ -112,7 +105,7 @@ class PipDialog(tk.Toplevel):
             # highlightcolor="green",
             borderwidth=0,
         )
-        self.listbox.insert("end", " <INSTALL>")
+        self.listbox.insert("end", " <" + _("INSTALL") + ">")
         self.listbox.bind("<<ListboxSelect>>", self._on_listbox_select, True)
         self.listbox.grid(row=0, column=0, sticky="nsew")
         list_scrollbar = AutoScrollbar(
@@ -140,7 +133,7 @@ class PipDialog(tk.Toplevel):
             vertical_scrollbar_class=AutoScrollbar,
             vertical_scrollbar_style=scrollbar_style("Vertical"),
             horizontal_scrollbar_style=scrollbar_style("Horizontal"),
-            width=60,
+            width=70,
             height=10,
         )
         info_text_frame.configure(borderwidth=0)
@@ -149,15 +142,19 @@ class PipDialog(tk.Toplevel):
         link_color = lookup_style_option("Url.TLabel", "foreground", "red")
         self.info_text.tag_configure("url", foreground=link_color, underline=True)
         self.info_text.tag_bind("url", "<ButtonRelease-1>", self._handle_url_click)
+        self.info_text.tag_bind("url", "<Enter>", lambda e: self.info_text.config(cursor="hand2"))
+        self.info_text.tag_bind("url", "<Leave>", lambda e: self.info_text.config(cursor=""))
+        self.info_text.tag_configure("install_reqs", foreground=link_color, underline=True)
         self.info_text.tag_bind(
-            "url", "<Enter>", lambda e: self.info_text.config(cursor="hand2")
+            "install_reqs", "<ButtonRelease-1>", self._handle_install_requirements_click
         )
         self.info_text.tag_bind(
-            "url", "<Leave>", lambda e: self.info_text.config(cursor="")
+            "install_reqs", "<Enter>", lambda e: self.info_text.config(cursor="hand2")
         )
-        self.info_text.tag_configure(
-            "install_file", foreground=link_color, underline=True
+        self.info_text.tag_bind(
+            "install_reqs", "<Leave>", lambda e: self.info_text.config(cursor="")
         )
+        self.info_text.tag_configure("install_file", foreground=link_color, underline=True)
         self.info_text.tag_bind(
             "install_file", "<ButtonRelease-1>", self._handle_install_file_click
         )
@@ -181,15 +178,16 @@ class PipDialog(tk.Toplevel):
         self.command_frame.grid(row=2, column=0, sticky="w")
 
         self.install_button = ttk.Button(
-            self.command_frame, text=" Upgrade ", command=self._on_click_install
+            self.command_frame, text=" " + _("Upgrade") + " ", command=self._on_click_install
         )
 
         self.install_button.grid(row=0, column=0, sticky="w", padx=0)
 
         self.uninstall_button = ttk.Button(
             self.command_frame,
-            text="Uninstall",
+            text=_("Uninstall"),
             command=lambda: self._perform_action("uninstall"),
+            width=len(_("Uninstall")),
         )
 
         self.uninstall_button.grid(row=0, column=1, sticky="w", padx=(5, 0))
@@ -203,7 +201,7 @@ class PipDialog(tk.Toplevel):
 
         self.advanced_button.grid(row=0, column=2, sticky="w", padx=(5, 0))
 
-        self.close_button = ttk.Button(info_frame, text="Close", command=self._on_close)
+        self.close_button = ttk.Button(info_frame, text=_("Close"), command=self._on_close)
         self.close_button.grid(row=2, column=3, sticky="e")
 
     def _on_click_install(self):
@@ -241,24 +239,25 @@ class PipDialog(tk.Toplevel):
 
     def _install_pip(self):
         self._clear()
-        self.info_text.direct_insert("end", "Installing pip\n\n", ("caption",))
+        self.info_text.direct_insert("end", ("Installing pip") + "\n\n", ("caption",))
         self.info_text.direct_insert(
             "end",
-            "pip, a required module for managing packages is missing or too old.\n\n"
-            + "Downloading pip installer (about 1.5 MB), please wait ...\n",
+            (
+                "pip, a required module for managing packages is missing or too old.\n\n"
+                + "Downloading pip installer (about 1.5 MB), please wait ..."
+            )
+            + "\n",
         )
         self.update()
         self.update_idletasks()
 
         installer_filename, _ = urlretrieve(PIP_INSTALLER_URL)
 
-        self.info_text.direct_insert("end", "Installing pip, please wait ...\n")
+        self.info_text.direct_insert("end", ("Installing pip, please wait ...") + "\n")
         self.update()
         self.update_idletasks()
 
-        proc, _ = self._create_python_process(
-            [installer_filename], stderr=subprocess.PIPE
-        )
+        proc, _ = self._create_python_process([installer_filename], stderr=subprocess.PIPE)
         out, err = proc.communicate()
         os.remove(installer_filename)
 
@@ -276,25 +275,30 @@ class PipDialog(tk.Toplevel):
         self._clear()
         self.info_text.direct_insert("end", error)
         self.info_text.direct_insert(
-            "end", "You seem to have problems with pip\n\n", ("caption",)
+            "end", ("You seem to have problems with pip" + "\n\n"), ("caption",)
         )
         self.info_text.direct_insert(
             "end",
-            "pip, a required module for managing packages is missing or too old for Thonny.\n\n"
-            + "If your system package manager doesn't provide recent pip (9.0.0 or later), "
-            + "then you can install newest version by downloading ",
+            (
+                "pip, a required module for managing packages is missing or too old for Thonny.\n\n"
+                + "If your system package manager doesn't provide recent pip (9.0.0 or later), "
+                + "then you can install newest version by downloading"
+            )
+            + " ",
         )
         self.info_text.direct_insert("end", PIP_INSTALLER_URL, ("url",))
         self.info_text.direct_insert(
             "end",
-            " and running it with "
+            " "
+            + ("and running it with")
+            + " "
             + self._get_interpreter()
-            + " (probably needs admin privileges).\n\n",
+            + " "
+            + ("(probably needs admin privileges).")
+            + "\n\n",
         )
 
-        self.info_text.direct_insert(
-            "end", self._instructions_for_command_line_install()
-        )
+        self.info_text.direct_insert("end", self._instructions_for_command_line_install())
         self._set_state("disabled", True)
 
     def _instructions_for_command_line_install(self):
@@ -345,67 +349,89 @@ class PipDialog(tk.Toplevel):
     def _show_instructions(self):
         self._clear()
         if self._read_only():
-            self.info_text.direct_insert("end", "Browse the packages\n", ("caption",))
+            self.info_text.direct_insert("end", _("Browse the packages") + "\n", ("caption",))
             self.info_text.direct_insert(
                 "end",
-                "With current interpreter you can only browse the packages here.\n"
-                + "Use 'Tools → Open system shell...' for installing, upgrading or uninstalling.\n\n",
+                _(
+                    "With current interpreter you can only browse the packages here.\n"
+                    + "Use 'Tools → Open system shell...' for installing, upgrading or uninstalling."
+                )
+                + "\n\n",
             )
 
             if self._get_target_directory():
-                self.info_text.direct_insert(
-                    "end", "Packages' directory\n", ("caption",)
-                )
+                self.info_text.direct_insert("end", _("Packages' directory") + "\n", ("caption",))
                 self.info_text.direct_insert(
                     "end", self._get_target_directory(), ("target_directory")
                 )
         else:
-            self.info_text.direct_insert("end", "Install from PyPI\n", ("caption",))
+            self.info_text.direct_insert("end", _("Install from PyPI") + "\n", ("caption",))
             self.info_text.direct_insert(
                 "end",
-                "If you don't know where to get the package from, "
-                + "then most likely you'll want to search the Python Package Index. "
-                + "Start by entering the name of the package in the search box above and pressing ENTER.\n\n",
+                _(
+                    "If you don't know where to get the package from, "
+                    + "then most likely you'll want to search the Python Package Index. "
+                    + "Start by entering the name of the package in the search box above and pressing ENTER."
+                )
+                + "\n\n",
             )
 
             self.info_text.direct_insert(
-                "end", "Install from local file\n", ("caption",)
+                "end", _("Install from requirements file") + "\n", ("caption",)
             )
-            self.info_text.direct_insert("end", "Click ")
-            self.info_text.direct_insert("end", "here", ("install_file",))
+            self.info_text.direct_insert("end", _("Click" + " "))
+            self.info_text.direct_insert("end", _("here"), ("install_reqs",))
             self.info_text.direct_insert(
                 "end",
-                " to locate and install the package file (usually with .whl, .tar.gz or .zip extension).\n\n",
+                " "
+                + _("to locate requirements.txt file and install the packages specified in it.")
+                + "\n\n",
             )
 
-            self.info_text.direct_insert("end", "Upgrade or uninstall\n", ("caption",))
+            self.info_text.direct_insert("end", _("Install from local file") + "\n", ("caption",))
+            self.info_text.direct_insert("end", _("Click") + " ")
+            self.info_text.direct_insert("end", _("here"), ("install_file",))
             self.info_text.direct_insert(
-                "end", "Start by selecting the package from the left.\n\n"
+                "end",
+                " "
+                + _(
+                    "to locate and install the package file (usually with .whl, .tar.gz or .zip extension)."
+                )
+                + "\n\n",
+            )
+
+            self.info_text.direct_insert("end", _("Upgrade or uninstall") + "\n", ("caption",))
+            self.info_text.direct_insert(
+                "end", _("Start by selecting the package from the left.") + "\n\n"
             )
 
             if self._get_target_directory():
-                self.info_text.direct_insert("end", "Target:  ", ("caption",))
-                if self._targets_virtual_environment():
+                self.info_text.direct_insert("end", _("Target:") + "  ", ("caption",))
+                if self._should_install_to_site_packages():
                     self.info_text.direct_insert(
-                        "end", "virtual environment\n", ("caption",)
+                        "end", _("virtual environment") + "\n", ("caption",)
                     )
                 else:
                     self.info_text.direct_insert(
-                        "end", "user site packages\n", ("caption",)
+                        "end", _("user site packages") + "\n", ("caption",)
                     )
 
                 self.info_text.direct_insert(
                     "end",
-                    "This dialog lists all available packages,"
-                    + " but allows upgrading and uninstalling only packages from ",
-                )
-                self.info_text.direct_insert(
-                    "end", self._get_target_directory(), ("url")
+                    _(
+                        "This dialog lists all available packages,"
+                        + " but allows upgrading and uninstalling only packages from"
+                    )
+                    + " ",
                 )
+                self.info_text.direct_insert("end", self._get_target_directory(), ("url"))
                 self.info_text.direct_insert(
                     "end",
-                    ". New packages will be also installed into this directory."
-                    + " Other locations must be managed by alternative means.",
+                    ". "
+                    + _(
+                        "New packages will be also installed into this directory."
+                        + " Other locations must be managed by alternative means."
+                    ),
                 )
 
         self._select_list_item(0)
@@ -427,9 +453,9 @@ class PipDialog(tk.Toplevel):
         active_dist = self._get_active_dist(name)
         if active_dist is not None:
             self.name_label["text"] = active_dist["project_name"]
-            self.info_text.direct_insert("end", "Installed version: ", ("caption",))
+            self.info_text.direct_insert("end", _("Installed version:") + " ", ("caption",))
             self.info_text.direct_insert("end", active_dist["version"] + "\n")
-            self.info_text.direct_insert("end", "Installed to: ", ("caption",))
+            self.info_text.direct_insert("end", _("Installed to:") + " ", ("caption",))
             self.info_text.direct_insert(
                 "end", normpath_with_actual_case(active_dist["location"]), ("url",)
             )
@@ -449,12 +475,12 @@ class PipDialog(tk.Toplevel):
 
             if active_dist is not None:
                 # existing package in target directory
-                self.install_button["text"] = "Upgrade"
+                self.install_button["text"] = _("Upgrade")
                 self.install_button["state"] = "disabled"
                 self.uninstall_button.grid(row=0, column=1)
             else:
                 # new package
-                self.install_button["text"] = "Install"
+                self.install_button["text"] = _("Install")
                 self.uninstall_button.grid_remove()
 
     def _show_package_info(self, name, data, error_code=None):
@@ -476,47 +502,54 @@ class PipDialog(tk.Toplevel):
 
         if error_code is not None:
             if error_code == 404:
-                write("Could not find the package from PyPI.")
+                write(_("Could not find the package from PyPI."))
                 if not self._get_active_version(name):
                     # new package
-                    write("\nPlease check your spelling!" + "\nYou need to enter ")
-                    write("exact package name", "bold")
+                    write(
+                        "\n"
+                        + _("Please check your spelling!")
+                        + "\n"
+                        + _("You need to enter")
+                        + " "
+                    )
+                    write(_("exact package name"), "bold")
                     write("!")
 
             else:
                 write(
-                    "Could not find the package info from PyPI. Error code: "
+                    _("Could not find the package info from PyPI.")
+                    + " "
+                    + _("Error code:")
+                    + " "
                     + str(error_code)
                 )
 
             return
 
         info = data["info"]
-        self.name_label["text"] = info[
-            "name"
-        ]  # search name could have been a bit different
+        self.name_label["text"] = info["name"]  # search name could have been a bit different
         latest_stable_version = _get_latest_stable_version(data["releases"].keys())
         if latest_stable_version is not None:
-            write_att("Latest stable version", latest_stable_version)
+            write_att(_("Latest stable version"), latest_stable_version)
         else:
-            write_att("Latest version", data["info"]["version"])
-        write_att("Summary", info["summary"])
-        write_att("Author", info["author"])
-        write_att("Homepage", info["home_page"], "url")
+            write_att(_("Latest version"), data["info"]["version"])
+        write_att(_("Summary"), info["summary"])
+        write_att(_("Author"), info["author"])
+        write_att(_("Homepage"), info["home_page"], "url")
         if info.get("bugtrack_url", None):
-            write_att("Bugtracker", info["bugtrack_url"], "url")
+            write_att(_("Bugtracker"), info["bugtrack_url"], "url")
         if info.get("docs_url", None):
-            write_att("Documentation", info["docs_url"], "url")
+            write_att(_("Documentation"), info["docs_url"], "url")
         if info.get("package_url", None):
-            write_att("PyPI page", info["package_url"], "url")
+            write_att(_("PyPI page"), info["package_url"], "url")
         if info.get("requires_dist", None):
             # Available only when release is created by a binary wheel
             # https://github.com/pypa/pypi-legacy/issues/622#issuecomment-305829257
-            write_att("Requires", ", ".join(info["requires_dist"]))
+            write_att(_("Requires"), ", ".join(info["requires_dist"]))
 
-        if self._get_active_version(
+        if self._get_active_version(name) != latest_stable_version or not self._get_active_version(
             name
-        ) != latest_stable_version or not self._get_active_version(name):
+        ):
             self.install_button["state"] = "normal"
         else:
             self.install_button["state"] = "disabled"
@@ -526,10 +559,7 @@ class PipDialog(tk.Toplevel):
         if dist is None:
             return False
         else:
-            return (
-                normpath_with_actual_case(dist["location"])
-                != self._get_target_directory()
-            )
+            return normpath_with_actual_case(dist["location"]) != self._get_target_directory()
 
     def _normalize_name(self, name):
         # looks like (in some cases?) pip list gives the name as it was used during install
@@ -544,13 +574,11 @@ class PipDialog(tk.Toplevel):
         if isinstance(name_or_index, int):
             index = name_or_index
         else:
-            normalized_items = list(
-                map(self._normalize_name, self.listbox.get(0, "end"))
-            )
+            normalized_items = list(map(self._normalize_name, self.listbox.get(0, "end")))
             try:
                 index = normalized_items.index(self._normalize_name(name_or_index))
             except Exception:
-                exception("Can't find package name from the list: " + name_or_index)
+                exception(_("Can't find package name from the list:") + " " + name_or_index)
                 return
 
         old_state = self.listbox["state"]
@@ -583,12 +611,12 @@ class PipDialog(tk.Toplevel):
             args.append(name)
         elif action == "uninstall":
             if name in ["pip", "setuptools"] and not messagebox.askyesno(
-                "Really uninstall?",
-                "Package '{}' is required for installing and uninstalling other packages.\n\n".format(
-                    name
-                )
-                + "Are you sure you want to uninstall it?",
-                parent=get_workbench()
+                _("Really uninstall?"),
+                _(
+                    "Package '{}' is required for installing and uninstalling other packages."
+                ).format(name)
+                + "\n\n"
+                + _("Are you sure you want to uninstall it?"),
             ):
                 return
             args = ["uninstall", "-y", name]
@@ -625,22 +653,34 @@ class PipDialog(tk.Toplevel):
 
         filename = askopenfilename(
             master=self,
-            filetypes=[("Package", ".whl .zip .tar.gz"), ("all files", ".*")],
-            initialdir=get_workbench().get_cwd(),
+            filetypes=[(_("Package"), ".whl .zip .tar.gz"), (_("all files"), ".*")],
+            initialdir=get_workbench().get_local_cwd,
         )
-        if (
-            filename
-        ):  # Note that missing filename may be "" or () depending on tkinter version
-            self._install_local_file(filename)
+        if filename:  # Note that missing filename may be "" or () depending on tkinter version
+            self._install_local_file(filename, False)
+
+    def _handle_install_requirements_click(self, event):
+        if self._get_state() != "idle":
+            return
+
+        filename = askopenfilename(
+            master=self,
+            filetypes=[("requirements", ".txt"), (_("all files"), ".*")],
+            initialdir=get_workbench().get_local_cwd,
+        )
+        if filename:  # Note that missing filename may be "" or () depending on tkinter version
+            self._install_local_file(filename, True)
 
     def _handle_target_directory_click(self, event):
         if self._get_target_directory():
             open_path_in_system_file_manager(self._get_target_directory())
 
-    def _install_local_file(self, filename):
+    def _install_local_file(self, filename, is_requirements_file):
         args = ["install"]
         if self._use_user_install():
             args.append("--user")
+        if is_requirements_file:
+            args.append("-r")
         args.append(filename)
 
         proc, cmd = self._create_pip_process(args)
@@ -703,23 +743,23 @@ class PipDialog(tk.Toplevel):
     def _get_interpreter(self):
         pass
 
-    def _targets_virtual_environment(self):
+    def _should_install_to_site_packages(self):
         raise NotImplementedError()
 
     def _use_user_install(self):
-        return not self._targets_virtual_environment()
+        return not self._should_install_to_site_packages()
 
     def _get_target_directory(self):
         raise NotImplementedError()
 
     def _get_title(self):
-        return "Manage packages for " + self._get_interpreter()
+        return _("Manage packages for %s") % self._get_interpreter()
 
     def _confirm_install(self, package_data):
         return True
 
     def _read_only(self):
-        if self._targets_virtual_environment():
+        if self._should_install_to_site_packages():
             return False
         else:
             # readonly if not in a virtual environment
@@ -741,16 +781,12 @@ class BackendPipDialog(PipDialog):
         assert self._get_state() in [None, "idle"]
         self._set_state("listing")
 
-        get_workbench().bind(
-            "get_active_distributions_response", self._complete_update_list, True
-        )
+        get_workbench().bind("get_active_distributions_response", self._complete_update_list, True)
         self._last_name_to_show = name_to_show
         get_runner().send_command(InlineCommand("get_active_distributions"))
 
     def _complete_update_list(self, msg):
-        get_workbench().unbind(
-            "get_active_distributions_response", self._complete_update_list
-        )
+        get_workbench().unbind("get_active_distributions_response", self._complete_update_list)
         if "error" in msg:
             self.info_text.direct_delete("1.0", "end")
             self.info_text.direct_insert("1.0", msg["error"])
@@ -773,15 +809,15 @@ class BackendPipDialog(PipDialog):
 
         if name.lower().startswith("thonny"):
             return messagebox.askyesno(
-                "Confirmation",
-                "Looks like you are installing a Thonny-related package.\n"
-                + "If you meant to install a Thonny plugin, then you should\n"
-                + "choose 'Tools → Manage plugins...' instead\n"
-                + "\n"
-                + "Are you sure you want to install '"
-                + name
-                + "' for the back-end?",
-                parent=get_workbench(),
+                _("Confirmation"),
+                _(
+                    "Looks like you are installing a Thonny-related package.\n"
+                    + "If you meant to install a Thonny plugin, then you should\n"
+                    + "choose 'Tools → Manage plugins...' instead\n"
+                    + "\n"
+                    + "Are you sure you want to install %s for the back-end?"
+                )
+                % name,
             )
         else:
             return True
@@ -793,13 +829,16 @@ class BackendPipDialog(PipDialog):
             self._provide_pip_install_instructions(error)
 
     def _get_target_directory(self):
-        if self._targets_virtual_environment():
+        if self._should_install_to_site_packages():
             return normpath_with_actual_case(self._backend_proxy.get_site_packages())
         else:
             usp = self._backend_proxy.get_user_site_packages()
             os.makedirs(usp, exist_ok=True)
             return normpath_with_actual_case(usp)
 
+    def _should_install_to_site_packages(self):
+        return self._targets_virtual_environment()
+
     def _targets_virtual_environment(self):
         return get_runner().using_venv()
 
@@ -848,6 +887,9 @@ class PluginsPipDialog(PipDialog):
     def _get_interpreter(self):
         return sys.executable.replace("thonny.exe", "python.exe")
 
+    def _should_install_to_site_packages(self):
+        return self._targets_virtual_environment()
+
     def _targets_virtual_environment(self):
         # https://stackoverflow.com/a/42580137/261181
         return (
@@ -864,33 +906,37 @@ class PluginsPipDialog(PipDialog):
     def _confirm_install(self, package_data):
         name = package_data["info"]["name"]
         reqs = package_data["info"].get("requires_dist", None)
-        
-        other_version_text = (
+
+        other_version_text = _(
             "NB! There may be another version available "
             + "which is compatible with current Thonny version. "
             + "Click on '...' button to choose the version to install."
         )
-        
+
         if name.lower().startswith("thonny-") and not reqs:
             showerror(
-                "Thonny plugin without requirements",
-                "Looks like you are trying to install an outdated Thonny\n"
-                + "plug-in (it doesn't specify required Thonny version).\n\n"
-                + "If you still want it, then please install it from the command line."
-                + "\n\n" + other_version_text,
-                parent=get_workbench(),
+                _("Thonny plugin without requirements"),
+                _(
+                    "Looks like you are trying to install an outdated Thonny\n"
+                    + "plug-in (it doesn't specify required Thonny version).\n\n"
+                    + "If you still want it, then please install it from the command line."
+                )
+                + "\n\n"
+                + other_version_text,
             )
             return False
         elif reqs:
             conflicts = self._conflicts_with_thonny_version(reqs)
             if conflicts:
                 showerror(
-                    "Unsuitable requirements",
-                    "This package requires different Thonny version:\n\n  "
+                    _("Unsuitable requirements"),
+                    _("This package requires different Thonny version:")
+                    + "\n\n  "
                     + "\n  ".join(conflicts)
-                    + "\n\nIf you still want it, then please install it from the command line."
-                    + "\n\n" + other_version_text,
-                    parent=get_workbench(),
+                    + "\n\n"
+                    + _("If you still want it, then please install it from the command line.")
+                    + "\n\n"
+                    + other_version_text,
                 )
                 return False
 
@@ -917,8 +963,11 @@ class PluginsPipDialog(PipDialog):
         banner.grid(row=0, column=0, sticky="nsew")
 
         banner_msg = (
-            "This dialog is for managing Thonny plug-ins and their dependencies.\n"
-            + "If you want to install packages for your own programs then choose 'Tools → Manage packages...'\n"
+            _(
+                "This dialog is for managing Thonny plug-ins and their dependencies.\n"
+                + "If you want to install packages for your own programs then choose 'Tools → Manage packages...'"
+            )
+            + "\n"
         )
 
         runner = get_runner()
@@ -927,11 +976,15 @@ class PluginsPipDialog(PipDialog):
             and runner.get_local_executable() is not None
             and is_same_path(self._get_interpreter(), get_runner().get_local_executable())
         ):
-            banner_msg += "(In this case Thonny's back-end uses same interpreter, so both dialogs manage same packages.)\n"
+            banner_msg += (
+                _(
+                    "(In this case Thonny's back-end uses same interpreter, so both dialogs manage same packages.)"
+                )
+                + "\n"
+            )
 
-        banner_msg += (
-            "\n"
-            + "NB! You need to restart Thonny after installing / upgrading / uninstalling a plug-in."
+        banner_msg += "\n" + _(
+            "NB! You need to restart Thonny after installing / upgrading / uninstalling a plug-in."
         )
 
         banner_text = tk.Label(banner, text=banner_msg, background=bg, justify="left")
@@ -940,20 +993,20 @@ class PluginsPipDialog(PipDialog):
         PipDialog._create_widgets(self, parent)
 
     def _get_title(self):
-        return "Thonny plug-ins"
+        return _("Thonny plug-ins")
 
     def _handle_outdated_or_missing_pip(self, error):
         return self._provide_pip_install_instructions(error)
 
 
-class DetailsDialog(tk.Toplevel):
+class DetailsDialog(CommonDialog):
     def __init__(self, master, package_metadata, selected_version):
-        tk.Toplevel.__init__(self, master)
+        super().__init__(master)
         self.result = None
         self._closed = False
         self._version_data = None
         self._package_name = package_metadata["info"]["name"]
-        self.title("Advanced install / upgrade / downgrade")
+        self.title(_("Advanced install / upgrade / downgrade"))
 
         self.rowconfigure(0, weight=1)
         self.columnconfigure(0, weight=1)
@@ -962,10 +1015,8 @@ class DetailsDialog(tk.Toplevel):
         main_frame.rowconfigure(0, weight=1)
         main_frame.columnconfigure(0, weight=1)
 
-        version_label = ttk.Label(main_frame, text="Desired version")
-        version_label.grid(
-            row=0, column=0, columnspan=2, padx=20, pady=(15, 0), sticky="w"
-        )
+        version_label = ttk.Label(main_frame, text=_("Desired version"))
+        version_label.grid(row=0, column=0, columnspan=2, padx=20, pady=(15, 0), sticky="w")
 
         def version_sort_key(s):
             # Trying to massage understandable versions into valid StrictVersions
@@ -990,32 +1041,25 @@ class DetailsDialog(tk.Toplevel):
             selected_version, self._start_fetching_version_info
         )
         self.version_combo = ttk.Combobox(
-            main_frame,
-            textvariable=self.version_var,
-            values=version_strings,
-            exportselection=False,
+            main_frame, textvariable=self.version_var, values=version_strings, exportselection=False
         )
 
         self.version_combo.state(["!disabled", "readonly"])
-        self.version_combo.grid(
-            row=1, column=0, columnspan=2, pady=(0, 15), padx=20, sticky="ew"
-        )
+        self.version_combo.grid(row=1, column=0, columnspan=2, pady=(0, 15), padx=20, sticky="ew")
 
         self.requires_label = ttk.Label(main_frame, text="")
-        self.requires_label.grid(
-            row=2, column=0, columnspan=2, pady=(0, 15), padx=20, sticky="ew"
-        )
+        self.requires_label.grid(row=2, column=0, columnspan=2, pady=(0, 15), padx=20, sticky="ew")
 
         self.update_deps_var = tk.IntVar()
         self.update_deps_var.set(0)
         self.update_deps_cb = ttk.Checkbutton(
-            main_frame, text="Upgrade dependencies", variable=self.update_deps_var
+            main_frame, text=_("Upgrade dependencies"), variable=self.update_deps_var
         )
         self.update_deps_cb.grid(row=3, column=0, columnspan=2, padx=20, sticky="w")
 
-        self.ok_button = ttk.Button(main_frame, text="Install", command=self._ok)
+        self.ok_button = ttk.Button(main_frame, text=_("Install"), command=self._ok)
         self.ok_button.grid(row=4, column=0, pady=15, padx=(20, 0), sticky="se")
-        self.cancel_button = ttk.Button(main_frame, text="Cancel", command=self._cancel)
+        self.cancel_button = ttk.Button(main_frame, text=_("Cancel"), command=self._cancel)
         self.cancel_button.grid(row=4, column=1, pady=15, padx=(5, 20), sticky="se")
 
         # self.resizable(height=tk.FALSE, width=tk.FALSE)
@@ -1067,11 +1111,11 @@ class DetailsDialog(tk.Toplevel):
             and "requires_dist" in info["info"]
             and isinstance(info["info"]["requires_dist"], list)
         ):
-            reqs = "Requires:\n  * " + "\n  * ".join(info["info"]["requires_dist"])
+            reqs = _("Requires:") + "\n  * " + "\n  * ".join(info["info"]["requires_dist"])
         elif error_code:
-            reqs = "Error code: " + str(error_code)
+            reqs = _("Error code:") + " " + str(error_code)
             if "error" in info:
-                reqs += "\nError: " + info["error"]
+                reqs += "\n" + _("Error:") + " " + info["error"]
         else:
             reqs = ""
 
@@ -1079,11 +1123,7 @@ class DetailsDialog(tk.Toplevel):
         self._set_state("idle")
 
     def _ok(self, event=None):
-        self.result = (
-            self.version_var.get(),
-            self._version_data,
-            bool(self.update_deps_var.get()),
-        )
+        self.result = (self.version_var.get(), self._version_data, bool(self.update_deps_var.get()))
         self._closed = True
         self.destroy()
 
@@ -1105,9 +1145,7 @@ def _fetch_url_future(url, timeout=10):
 def _get_latest_stable_version(version_strings):
     versions = []
     for s in version_strings:
-        if s.replace(
-            ".", ""
-        ).isnumeric():  # Assuming stable versions have only dots and numbers
+        if s.replace(".", "").isnumeric():  # Assuming stable versions have only dots and numbers
             versions.append(
                 LooseVersion(s)
             )  # LooseVersion __str__ doesn't change the version string
@@ -1149,9 +1187,7 @@ def _start_fetching_package_info(name, version_str, completion_handler):
                 completion_handler(name, json.loads(raw_data))
             except urllib.error.HTTPError as e:
                 completion_handler(
-                    name,
-                    {"info": {"name": name}, "error": str(e), "releases": {}},
-                    e.code,
+                    name, {"info": {"name": name}, "error": str(e), "releases": {}}, e.code
                 )
         else:
             tk._default_root.after(200, poll_fetch_complete)
@@ -1189,11 +1225,11 @@ def load_plugin() -> None:
     get_workbench().add_command(
         "backendpipgui",
         "tools",
-        "Manage packages...",
+        _("Manage packages..."),
         open_backend_pip_gui,
         tester=open_backend_pip_gui_enabled,
         group=80,
     )
     get_workbench().add_command(
-        "pluginspipgui", "tools", "Manage plug-ins...", open_frontend_pip_gui, group=180
+        "pluginspipgui", "tools", _("Manage plug-ins..."), open_frontend_pip_gui, group=180
     )
