@@ -5,29 +5,21 @@ import tkinter as tk
 from datetime import datetime
 from tkinter import ttk
 
-from thonny import codeview, get_workbench, ui_utils
-from thonny.base_file_browser import BaseFileBrowser
+from thonny import codeview, get_workbench, ui_utils, THONNY_USER_DIR
+from thonny.base_file_browser import BaseLocalFileBrowser
 from thonny.plugins.coloring import SyntaxColorer
-from thonny.ui_utils import lookup_style_option
+from thonny.ui_utils import lookup_style_option, CommonDialog
 
 
-class ReplayWindow(tk.Toplevel):
+class ReplayWindow(CommonDialog):
     def __init__(self):
-        tk.Toplevel.__init__(
-            self,
-            get_workbench(),
-            background=lookup_style_option("TFrame", "background"),
-        )
+        super().__init__(get_workbench(), background=lookup_style_option("TFrame", "background"))
         ui_utils.set_zoomed(self, True)
 
         self.main_pw = ReplayerPanedWindow(self, orient=tk.HORIZONTAL, sashwidth=10)
-        self.center_pw = ReplayerPanedWindow(
-            self.main_pw, orient=tk.VERTICAL, sashwidth=10
-        )
+        self.center_pw = ReplayerPanedWindow(self.main_pw, orient=tk.VERTICAL, sashwidth=10)
         self.right_frame = ttk.Frame(self.main_pw)
-        self.right_pw = ReplayerPanedWindow(
-            self.right_frame, orient=tk.VERTICAL, sashwidth=10
-        )
+        self.right_pw = ReplayerPanedWindow(self.right_frame, orient=tk.VERTICAL, sashwidth=10)
         self.editor_notebook = ReplayerEditorNotebook(self.center_pw)
         shell_book = ttk.Notebook(self.main_pw)
         self.shell = ShellFrame(shell_book)
@@ -56,16 +48,20 @@ class ReplayWindow(tk.Toplevel):
         self.rowconfigure(0, weight=1)
 
 
-class ReplayerFileBrowser(BaseFileBrowser):
+class ReplayerFileBrowser(BaseLocalFileBrowser):
     def __init__(self, master, log_frame):
-        BaseFileBrowser.__init__(
-            self, master, True, "tools.replayer_last_browser_folder"
-        )
+        super().__init__(master, True)
         self.log_frame = log_frame
         self.configure(border=1, relief=tk.GROOVE)
 
+        user_logs_path = os.path.join(THONNY_USER_DIR, "user_logs")
+        if os.path.exists(user_logs_path):
+            self.focus_into(user_logs_path)
+        else:
+            self.focus_into(os.path.expanduser("~"))
+
     def on_double_click(self, event):
-        self.save_current_folder()
+        # self.save_current_folder()
         path = self.get_selected_path()
         if path:
             self.log_frame.load_log(path)
@@ -78,12 +74,8 @@ class ControlFrame(ttk.Frame):
         self.toggle_button = ttk.Button(self, text="Play")
         self.speed_scale = ttk.Scale(self, from_=1, to=100, orient=tk.HORIZONTAL)
 
-        self.toggle_button.grid(
-            row=0, column=0, sticky=tk.NSEW, pady=(10, 0), padx=(0, 5)
-        )
-        self.speed_scale.grid(
-            row=0, column=1, sticky=tk.NSEW, pady=(10, 0), padx=(5, 0)
-        )
+        self.toggle_button.grid(row=0, column=0, sticky=tk.NSEW, pady=(10, 0), padx=(0, 5))
+        self.speed_scale.grid(row=0, column=1, sticky=tk.NSEW, pady=(10, 0), padx=(5, 0))
 
         self.columnconfigure(1, weight=1)
 
@@ -137,7 +129,10 @@ class LogFrame(ui_utils.TreeFrame):
         # print("log replay", event)
 
         if "text_widget_id" in event:
-            if event.get("text_widget_context", None) == "shell":
+            if (
+                event.get("text_widget_context", None) == "shell"
+                or event.get("text_widget_class") == "ShellText"
+            ):
                 self.shell.replay_event(event)
             else:
                 self.editor_notebook.replay_event(event)
@@ -319,9 +314,7 @@ class ShellFrame(ReplayerEditor):
         vert_spacing = 10
         io_indent = 16
         self.code_view.text.tag_configure("toplevel", font="EditorFont")
-        self.code_view.text.tag_configure(
-            "prompt", foreground="purple", font="BoldEditorFont"
-        )
+        self.code_view.text.tag_configure("prompt", foreground="purple", font="BoldEditorFont")
         self.code_view.text.tag_configure("command", foreground="black")
         self.code_view.text.tag_configure("version", foreground="DarkGray")
         self.code_view.text.tag_configure("automagic", foreground="DarkGray")
@@ -331,18 +324,12 @@ class ShellFrame(ReplayerEditor):
         self.code_view.text.tag_configure("error", foreground="Red")
 
         self.code_view.text.tag_configure(
-            "io",
-            lmargin1=io_indent,
-            lmargin2=io_indent,
-            rmargin=io_indent,
-            font="IOFont",
+            "io", lmargin1=io_indent, lmargin2=io_indent, rmargin=io_indent, font="IOFont"
         )
         self.code_view.text.tag_configure("stdin", foreground="Blue")
         self.code_view.text.tag_configure("stdout", foreground="Black")
         self.code_view.text.tag_configure("stderr", foreground="Red")
-        self.code_view.text.tag_configure(
-            "hyperlink", foreground="#3A66DD", underline=True
-        )
+        self.code_view.text.tag_configure("hyperlink", foreground="#3A66DD", underline=True)
 
         self.code_view.text.tag_configure("vertically_spaced", spacing1=vert_spacing)
         self.code_view.text.tag_configure("inactive", foreground="#aaaaaa")
@@ -364,5 +351,5 @@ def load_plugin() -> None:
     get_workbench().set_default("tools.replayer_last_browser_folder", None)
     if get_workbench().get_ui_mode() == "expert":
         get_workbench().add_command(
-            "open_replayer", "tools", "Open replayer...", open_replayer, group=110
+            "open_replayer", "tools", _("Open replayer..."), open_replayer, group=110
         )
