@@ -7,6 +7,7 @@ import os.path
 
 from thonny.assistance import ErrorHelper, Suggestion, name_similarity, add_error_helper
 from thonny import assistance
+from thonny.misc_utils import running_on_linux, running_on_windows
 
 
 class SyntaxErrorHelper(ErrorHelper):
@@ -18,16 +19,12 @@ class SyntaxErrorHelper(ErrorHelper):
 
         if self.error_info["message"] == "EOL while scanning string literal":
             self.intro_text = (
-                "You haven't properly closed the string on line %s."
-                % self.error_info["lineno"]
+                "You haven't properly closed the string on line %s." % self.error_info["lineno"]
                 + "\n(If you want a multi-line string, then surround it with"
                 + " `'''` or `\"\"\"` at both ends.)"
             )
 
-        elif (
-            self.error_info["message"]
-            == "EOF while scanning triple-quoted string literal"
-        ):
+        elif self.error_info["message"] == "EOF while scanning triple-quoted string literal":
             # lineno is not useful, as it is at the end of the file and user probably
             # didn't want the string to end there
             self.intro_text = "You haven't properly closed a triple-quoted string"
@@ -40,29 +37,31 @@ class SyntaxErrorHelper(ErrorHelper):
                             self.tokens.append(t)
                     except tokenize.TokenError as e:
                         self.token_error = e
-                
-                if (not self.tokens 
-                    or self.tokens[-1].type not in [token.ERRORTOKEN, token.ENDMARKER]):
+
+                if not self.tokens or self.tokens[-1].type not in [
+                    token.ERRORTOKEN,
+                    token.ENDMARKER,
+                ]:
                     self.tokens.append(tokenize.TokenInfo(token.ERRORTOKEN, "", None, None, ""))
             else:
                 self.tokens = []
 
             unbalanced = self._sug_unbalanced_parens()
             if unbalanced:
-                self.intro_text = "Unbalanced parentheses, brackets or braces:\n\n" + unbalanced.body
+                self.intro_text = (
+                    "Unbalanced parentheses, brackets or braces:\n\n" + unbalanced.body
+                )
                 self.intro_confidence = 5
             else:
                 self.intro_text = "Python doesn't know how to read your program."
-                
+
                 if "^" in str(self.error_info):
                     self.intro_text += (
                         "\n\nSmall `^` in the original error message shows where it gave up,"
                         + " but the actual mistake can be before this."
                     )
-    
-                self.suggestions = [
-                    self._sug_missing_or_misplaced_colon(),
-                ]
+
+                self.suggestions = [self._sug_missing_or_misplaced_colon()]
 
     def _sug_missing_or_misplaced_colon(self):
         i = 0
@@ -120,8 +119,7 @@ class SyntaxErrorHelper(ErrorHelper):
                     body = "Nothing is allowed between `%s` and colon." % t.string
                     relevance = 9
                     if (
-                        self.tokens[keyword_pos + 1].type
-                        not in (token.NEWLINE, tokenize.COMMENT)
+                        self.tokens[keyword_pos + 1].type not in (token.NEWLINE, tokenize.COMMENT)
                         and t.string == "else"
                     ):
                         body = "If you want to specify a conditon, then use `elif` or nested `if`."
@@ -130,18 +128,13 @@ class SyntaxErrorHelper(ErrorHelper):
             i += 1
 
         return Suggestion("missing-or-misplaced-colon", title, body, relevance)
-    
+
     def _sug_unbalanced_parens(self):
         problem = self._find_first_braces_problem()
         if not problem:
             return None
-        
-        return Suggestion(
-            "missing-or-misplaced-colon", 
-            "Unbalanced brackets", 
-            problem[1],
-            8
-        )
+
+        return Suggestion("missing-or-misplaced-colon", "Unbalanced brackets", problem[1], 8)
 
     def _sug_wrong_increment_op(self):
         pass
@@ -167,7 +160,7 @@ class SyntaxErrorHelper(ErrorHelper):
                 level -= 1
 
             token_index += 1
-            
+
             if level <= 0:
                 return token_index
 
@@ -188,10 +181,12 @@ class SyntaxErrorHelper(ErrorHelper):
                         t,
                         "Found '`%s`' at `line %d <%s>`_ without preceding matching '`%s`'"
                         % (
-                            t.string, 
+                            t.string,
                             t.start[0],
-                            assistance.format_file_url(self.error_info["filename"], t.start[0], t.start[1]),
-                            openers[t.string]
+                            assistance.format_file_url(
+                                self.error_info["filename"], t.start[0], t.start[1]
+                            ),
+                            openers[t.string],
                         ),
                     )
                 elif brace_stack[-1].string != openers[t.string]:
@@ -201,12 +196,16 @@ class SyntaxErrorHelper(ErrorHelper):
                         % (
                             t.string,
                             t.start[0],
-                            assistance.format_file_url(self.error_info["filename"], t.start[0], t.start[1]),
+                            assistance.format_file_url(
+                                self.error_info["filename"], t.start[0], t.start[1]
+                            ),
                             brace_stack[-1].string,
                             brace_stack[-1].start[0],
-                            assistance.format_file_url(self.error_info["filename"], 
-                                                       brace_stack[-1].start[0],
-                                                       brace_stack[-1].start[1]),
+                            assistance.format_file_url(
+                                self.error_info["filename"],
+                                brace_stack[-1].start[0],
+                                brace_stack[-1].start[1],
+                            ),
                         ),
                     )
                 else:
@@ -219,10 +218,12 @@ class SyntaxErrorHelper(ErrorHelper):
                 % (
                     brace_stack[-1].string,
                     brace_stack[-1].start[0],
-                    assistance.format_file_url(self.error_info["filename"], 
-                                               brace_stack[-1].start[0],
-                                               brace_stack[-1].start[1]),
-                )
+                    assistance.format_file_url(
+                        self.error_info["filename"],
+                        brace_stack[-1].start[0],
+                        brace_stack[-1].start[1],
+                    ),
+                ),
             )
 
         return None
@@ -246,11 +247,7 @@ class NameErrorHelper(ErrorHelper):
         ]
 
     def _sug_missing_quotes(self):
-        if (
-            self._is_attribute_value()
-            or self._is_call_function()
-            or self._is_subscript_value()
-        ):
+        if self._is_attribute_value() or self._is_call_function() or self._is_subscript_value():
             relevance = 0
         else:
             relevance = 5
@@ -297,9 +294,7 @@ class NameErrorHelper(ErrorHelper):
                 + " Don't forget that case of the letters matters!"
             )
 
-        return Suggestion(
-            "bad-spelling-name", "Did you misspell it (somewhere)?", body, relevance
-        )
+        return Suggestion("bad-spelling-name", "Did you misspell it (somewhere)?", body, relevance)
 
     def _sug_missing_import(self):
         likely_importable_functions = {
@@ -377,22 +372,15 @@ class NameErrorHelper(ErrorHelper):
             relevance = 3
 
         if body is None:
-            body = (
-                "Some functions/variables need to be imported before they can be used."
-            )
+            body = "Some functions/variables need to be imported before they can be used."
 
-        return Suggestion(
-            "missing-import", "Did you forget to import it?", body, relevance
-        )
+        return Suggestion("missing-import", "Did you forget to import it?", body, relevance)
 
     def _sug_local_from_global(self):
         relevance = 0
         body = None
 
-        if (
-            self.last_frame.code_name == "<module>"
-            and self.last_frame_module_ast is not None
-        ):
+        if self.last_frame.code_name == "<module>" and self.last_frame_module_ast is not None:
             function_names = set()
             for node in ast.walk(self.last_frame_module_ast):
                 if isinstance(node, (ast.FunctionDef, ast.AsyncFunctionDef)):
@@ -408,10 +396,7 @@ class NameErrorHelper(ErrorHelper):
                             and isinstance(localnode.ctx, ast.Store)
                         ):
                             function_names.add(node.name)
-                        elif (
-                            isinstance(localnode, ast.Global)
-                            and self.name in localnode.names
-                        ):
+                        elif isinstance(localnode, ast.Global) and self.name in localnode.names:
                             declared_global = True
 
                     if node.name in function_names and declared_global:
@@ -537,8 +522,7 @@ class AttributeErrorHelper(ErrorHelper):
         return Suggestion(
             "wrong-type-attribute",
             "Did you expect another type?",
-            "If you didn't mean %s %s, "
-            % (action, _get_phrase_for_object(self.type_name))
+            "If you didn't mean %s %s, " % (action, _get_phrase_for_object(self.type_name))
             + "then step through your program to see "
             + "why this type appears here.",
             3,
@@ -550,13 +534,70 @@ class AttributeErrorHelper(ErrorHelper):
         )
 
 
+class OSErrorHelper(ErrorHelper):
+    def __init__(self, error_info):
+        super().__init__(error_info)
+
+        if "Address already in use" in self.error_info["message"]:
+            self.intro_text = "Your programs tries to listen on a port which is already taken."
+            self.suggestions = [
+                Suggestion(
+                    "kill-by-port-type-error",
+                    "Want to close the other process?",
+                    self.get_kill_process_instructions(),
+                    5,
+                ),
+                Suggestion(
+                    "use-another-type-error",
+                    "Can you use another port?",
+                    "If you don't want to mess with the other process, then check whether"
+                    + " you can configure your program to use another port.",
+                    3,
+                ),
+            ]
+
+        else:
+            self.intro_text = "No specific information is available for this error."
+
+    def get_kill_process_instructions(self):
+        s = (
+            "Let's say you need port 5000. If you don't know which process is using it,"
+            + " then enter following system command into Thonny's Shell:\n\n"
+        )
+
+        if running_on_windows():
+            s += (
+                "``!netstat -ano | findstr :5000``\n\n"
+                + "You should see the process ID in the last column.\n\n"
+            )
+        else:
+            s += (
+                "``!lsof -i:5000``\n\n" + "You should see the process ID under the heading PID.\n\n"
+            )
+
+        s += (
+            "Let's pretend the ID is 12345."
+            " You can try hard-killing the process with following command:\n\n"
+        )
+
+        if running_on_windows():
+            s += "``!tskill 12345``\n"
+        else:
+            s += (
+                "``!kill -9 12345``\n\n"
+                + "Both steps can be combined into single command:\n\n"
+                + "``!kill -9 $(lsof -t -i:5000)``\n\n"
+            )
+
+        return s
+
+
 class TypeErrorHelper(ErrorHelper):
     def __init__(self, error_info):
         super().__init__(error_info)
 
         self.intro_text = (
-            "Python was asked to do an operation with an object which "
-            + "doesn't support it."
+            "Python was asked to do an operation with an object which " + "doesn't support it."
         )
 
         self.suggestions = [
@@ -614,11 +655,7 @@ class TypeErrorHelper(ErrorHelper):
                     "Did you mean to treat both sides as numbers and produce a sum?",
                     "In this case you should first convert the string to a number "
                     + "using either function `float` or `int`, eg:\n\n"
-                    + (
-                        "`float('3.14') + 22`"
-                        if string_first
-                        else "`22 + float('3.14')`"
-                    ),
+                    + ("`float('3.14') + 22`" if string_first else "`22 + float('3.14')`"),
                     7,
                 )
             )
