@@ -52,7 +52,7 @@ class RstText(TweakableText):
         # Underline on font looks better than underline on tag
         underline_font = main_font.copy()
         underline_font.configure(underline=True)
-        
+
         self.tag_configure("h1", font=h1_font, spacing3=5)
         self.tag_configure("h2", font=h2_font, spacing3=5)
         self.tag_configure("h3", font=h3_font, spacing3=5)
@@ -60,10 +60,12 @@ class RstText(TweakableText):
         self.tag_configure("line_block", spacing1=0, spacing3=10, spacing2=0)
         self.tag_configure("em", font=italic_font)
         self.tag_configure("strong", font=bold_font)
-        
-        self.tag_configure("a", 
-                           **{**get_syntax_options_for_tag("hyperlink"), 'underline' : False}, 
-                           font=underline_font)
+
+        self.tag_configure(
+            "a",
+            **{**get_syntax_options_for_tag("hyperlink"), "underline": False},
+            font=underline_font
+        )
         self.tag_configure("small", font=small_font)
         self.tag_configure("light", foreground="gray")
         self.tag_configure("remark", font=small_italic_font)
@@ -75,6 +77,7 @@ class RstText(TweakableText):
         self.tag_configure(
             "code",
             font="TkFixedFont",
+            # wrap="none", # TODO: needs automatic hor-scrollbar and better padding mgmt
             # background="#eeeeee"
         )
         # if ui_utils.get_tk_version_info() >= (8,6,6):
@@ -85,8 +88,7 @@ class RstText(TweakableText):
 
         toti_code_font = bold_font.copy()
         toti_code_font.configure(
-            family=tk.font.nametofont("TkFixedFont").cget("family"),
-            size=bold_font.cget("size"),
+            family=tk.font.nametofont("TkFixedFont").cget("family"), size=bold_font.cget("size")
         )
         self.tag_configure("topic_title_code", font=toti_code_font)
         self.tag_raise("topic_title_code", "code")
@@ -259,13 +261,9 @@ class TkTextRenderingVisitor(docutils.nodes.GenericNodeVisitor):
                 self.text.tag_remove("sel", "1.0", "end")
 
             if elide:
-                label.configure(
-                    image=get_workbench().get_image(get_toggler_image_name("boxplus"))
-                )
+                label.configure(image=get_workbench().get_image(get_toggler_image_name("boxplus")))
             else:
-                label.configure(
-                    image=get_workbench().get_image(get_toggler_image_name("boxminus"))
-                )
+                label.configure(image=get_workbench().get_image(get_toggler_image_name("boxminus")))
 
         assert isinstance(node.children[0], docutils.nodes.title)
 
@@ -290,13 +288,11 @@ class TkTextRenderingVisitor(docutils.nodes.GenericNodeVisitor):
         raise docutils.nodes.SkipNode()
 
     def _visit_empty_topic(self, node):
-        img = get_workbench().get_image("boxdot_light" if get_workbench().uses_dark_ui_theme() else "boxdot")
+        img = get_workbench().get_image(
+            "boxdot_light" if get_workbench().uses_dark_ui_theme() else "boxdot"
+        )
         label = tk.Label(
-            self.text,
-            image=img,
-            borderwidth=0,
-            background=self.text["background"],
-            cursor="arrow",
+            self.text, image=img, borderwidth=0, background=self.text["background"], cursor="arrow"
         )
         self._append_window(label)
         assert isinstance(node.children[0], docutils.nodes.title)
@@ -411,7 +407,7 @@ class TkTextRenderingVisitor(docutils.nodes.GenericNodeVisitor):
 
     def depart_block_quote(self, node):
         self._pop_tag("code")
-    
+
     def default_visit(self, node):
         self._append_text(self._node_to_text(node))
         print("skipping children", type(node), node)
