@@ -25,15 +25,9 @@ class TweakableText(tk.Text):
         self.tk.createcommand(self._w, self._dispatch_tk_operation)
         self._tk_proxies = {}
 
-        self._original_insert = self._register_tk_proxy_function(
-            "insert", self.intercept_insert
-        )
-        self._original_delete = self._register_tk_proxy_function(
-            "delete", self.intercept_delete
-        )
-        self._original_mark = self._register_tk_proxy_function(
-            "mark", self.intercept_mark
-        )
+        self._original_insert = self._register_tk_proxy_function("insert", self.intercept_insert)
+        self._original_delete = self._register_tk_proxy_function("delete", self.intercept_delete)
+        self._original_mark = self._register_tk_proxy_function("mark", self.intercept_mark)
 
     def _register_tk_proxy_function(self, operation, function):
         self._tk_proxies[operation] = function
@@ -55,8 +49,7 @@ class TweakableText(tk.Text):
         except TclError as e:
             # Some Tk internal actions (eg. paste and cut) can cause this error
             if (
-                str(e).lower()
-                == '''text doesn't contain any characters tagged with "sel"'''
+                str(e).lower() == '''text doesn't contain any characters tagged with "sel"'''
                 and operation in ["delete", "index", "get"]
                 and args in [("sel.first", "sel.last"), ("sel.first",)]
             ):
@@ -64,10 +57,7 @@ class TweakableText(tk.Text):
                 pass
             else:
                 exception(
-                    "[_dispatch_tk_operation] operation: "
-                    + operation
-                    + ", args:"
-                    + repr(args)
+                    "[_dispatch_tk_operation] operation: " + operation + ", args:" + repr(args)
                 )
                 # traceback.print_exc()
 
@@ -109,9 +99,7 @@ class TweakableText(tk.Text):
 
     def intercept_insert(self, index, chars, tags=None, **kw):
         assert isinstance(chars, str)
-        if (
-            chars >= "\uf704" and chars <= "\uf70d"
-        ):  # Function keys F1..F10 in Mac cause these
+        if chars >= "\uf704" and chars <= "\uf70d":  # Function keys F1..F10 in Mac cause these
             pass
         elif self.is_read_only():
             self.bell()
@@ -182,8 +170,16 @@ class EnhancedText(TweakableText):
     Most of the code is adapted from idlelib.EditorWindow.
     """
 
-    def __init__(self, master=None, style="Text", tag_current_line=False,
-                 indent_with_tabs=False, replace_tabs=False, cnf={}, **kw):
+    def __init__(
+        self,
+        master=None,
+        style="Text",
+        tag_current_line=False,
+        indent_with_tabs=False,
+        replace_tabs=False,
+        cnf={},
+        **kw
+    ):
         # Parent class shouldn't autoseparate
         # TODO: take client provided autoseparators value into account
         kw["autoseparators"] = False
@@ -261,9 +257,7 @@ class EnhancedText(TweakableText):
         self.bind("<Prior>", self.perform_page_up, True)
 
     def _bind_selection_aids(self):
-        self.bind(
-            "<Command-a>" if _running_on_mac() else "<Control-a>", self.select_all, True
-        )
+        self.bind("<Command-a>" if _running_on_mac() else "<Control-a>", self.select_all, True)
 
     def _bind_undo_aids(self):
         self.bind("<<Undo>>", self._on_undo, True)
@@ -416,9 +410,7 @@ class EnhancedText(TweakableText):
             row, _ = map(int, last_visible_idx.split("."))
             line_count = self.get_line_count()
 
-            if (
-                row == line_count or row == line_count - 1
-            ):  # otherwise tk doesn't show last line
+            if row == line_count or row == line_count - 1:  # otherwise tk doesn't show last line
                 self.mark_set("insert", "end")
         except Exception:
             traceback.print_exc()
@@ -499,9 +491,7 @@ class EnhancedText(TweakableText):
 
     def perform_tab(self, event=None):
         self._log_keypress_for_undo(event)
-        if (
-            event.state & 0x0001
-        ):  # shift is pressed (http://stackoverflow.com/q/32426250/261181)
+        if event.state & 0x0001:  # shift is pressed (http://stackoverflow.com/q/32426250/261181)
             return self.dedent_region(event)
         else:
             # check whether there are letters before cursor on this line
@@ -637,7 +627,7 @@ class EnhancedText(TweakableText):
     def _on_paste(self, e):
         if self.is_read_only():
             return
-        
+
         try:
             if self.has_selection():
                 self.direct_delete("sel.first", "sel.last")
@@ -719,23 +709,28 @@ class EnhancedText(TweakableText):
     def direct_insert(self, index, chars, tags=None, **kw):
         chars = self.check_convert_tabs_to_spaces(chars)
         super().direct_insert(index, chars, tags, **kw)
-    
+
     def check_convert_tabs_to_spaces(self, chars):
+        if not self.replace_tabs:
+            return chars
         tab_count = chars.count("\t")
-        if not self.replace_tabs or tab_count == 0:
+        if tab_count == 0:
             return chars
         else:
-            
-            if messagebox.askyesno("Convert tabs to spaces?",
-                                   "Thonny (according to Python recommendation) uses spaces for indentation, "
-                                   + "but the text you are about to insert/open contains %d tab characters. " % tab_count
-                                   + "To avoid confusion, it's better to convert them into spaces (unless you know they should be kept as tabs).\n\n" 
-                                   + "Do you want me to replace each tab with %d spaces?\n\n" % self.indent_width,
-                                   parent=tk._default_root):
+
+            if messagebox.askyesno(
+                "Convert tabs to spaces?",
+                "Thonny (according to Python recommendation) uses spaces for indentation, "
+                + "but the text you are about to insert/open contains %d tab characters. "
+                % tab_count
+                + "To avoid confusion, it's better to convert them into spaces (unless you know they should be kept as tabs).\n\n"
+                + "Do you want me to replace each tab with %d spaces?\n\n" % self.indent_width,
+            ):
                 return chars.expandtabs(self.indent_width)
             else:
                 return chars
 
+
 class TextFrame(ttk.Frame):
     "Decorates text with scrollbars, line numbers and print margin"
 
@@ -792,9 +787,11 @@ class TextFrame(ttk.Frame):
             undo=False,
             wrap="none",
         )
-        self._gutter.bind("<ButtonRelease-1>", self.on_gutter_click)
-        self._gutter.bind("<Button-1>", self.on_gutter_click)
-        self._gutter.bind("<Button1-Motion>", self.on_gutter_motion)
+        self._gutter_is_gridded = False
+        self._gutter.bind("<Double-Button-1>", self.on_gutter_double_click, True),
+        self._gutter.bind("<ButtonRelease-1>", self.on_gutter_click, True)
+        self._gutter.bind("<Button-1>", self.on_gutter_click, True)
+        self._gutter.bind("<Button1-Motion>", self.on_gutter_motion, True)
         self._gutter["yscrollcommand"] = self._gutter_scroll
 
         # need tags for justifying and rmargin
@@ -825,9 +822,7 @@ class TextFrame(ttk.Frame):
         self.rowconfigure(0, weight=1)
 
         self._recommended_line_length = line_length_margin
-        margin_line_color = ttk.Style().lookup(
-            "Gutter", "background", default="LightGray"
-        )
+        margin_line_color = ttk.Style().lookup("Gutter", "background", default="LightGray")
         self._margin_line = tk.Canvas(
             self.text,
             borderwidth=0,
@@ -852,43 +847,52 @@ class TextFrame(ttk.Frame):
         self.text.focus_set()
 
     def set_gutter_visibility(self, value):
-        if value and not self._gutter.winfo_ismapped():
+        if value and not self._gutter_is_gridded:
             self._gutter.grid(row=0, column=0, sticky=tk.NSEW)
-        elif not value and self._gutter.winfo_ismapped():
+            self._gutter_is_gridded = True
+        elif not value and self._gutter_is_gridded:
             self._gutter.grid_forget()
+            self._gutter_is_gridded = False
+        else:
+            return
 
+        """
         # insert first line number (NB! Without trailing linebreak. See update_gutter)
         self._gutter.config(state="normal")
         self._gutter.delete("1.0", "end")
         for content, tags in self.compute_gutter_line(self._first_line_number):
             self._gutter.insert("end", content, ("content",) + tags)
         self._gutter.config(state="disabled")
-
-        self.update_gutter()
+        """
+        self.update_gutter(True)
 
     def set_line_length_margin(self, value):
         self._recommended_line_length = value
         self.update_margin_line()
 
     def _text_changed(self, event):
-        "# TODO: make more efficient"
         self.update_gutter()
-        self.update_margin_line()
 
     def _cursor_moved(self, event):
         self._update_gutter_active_line()
 
     def _vertical_scrollbar_update(self, *args):
+        if not hasattr(self, "_vbar"):
+            return
+
         self._vbar.set(*args)
         self._gutter.yview(tk.MOVETO, args[0])
+        self.text.event_generate("<<VerticalScroll>>")
 
     def _gutter_scroll(self, *args):
-        # FIXME: this doesn't work properly
-        # Can't scroll to bottom when line numbers are not visible
-        # and can't type normally at the bottom, when line numbers are visible
-        return
-        # self._vbar.set(*args)
-        # self.text.yview(tk.MOVETO, args[0])
+        if not hasattr(self, "_vbar"):
+            return
+
+        try:
+            self._vbar.set(*args)
+            self.text.yview(tk.MOVETO, args[0])
+        except TclError:
+            pass
 
     def _horizontal_scrollbar_update(self, *args):
         self._hbar.set(*args)
@@ -897,15 +901,13 @@ class TextFrame(ttk.Frame):
     def _vertical_scroll(self, *args):
         self.text.yview(*args)
         self._gutter.yview(*args)
+        self.text.event_generate("<<VerticalScroll>>")
 
     def _horizontal_scroll(self, *args):
         self.text.xview(*args)
         self.update_margin_line()
 
-    def update_gutter(self, clean=True):
-        # TODO: make it more efficient
-        # by default clean only if line counts in gutter and text differ
-
+    def update_gutter(self, clean=False):
         if clean:
             self._gutter.config(state="normal")
             self._gutter.delete("1.0", "end")
@@ -927,10 +929,21 @@ class TextFrame(ttk.Frame):
             if text_line_count > gutter_line_count:
                 delta = text_line_count - gutter_line_count
                 start = gutter_line_count + self._first_line_number - 1
-                for i in range(start, start + delta):
-                    self._gutter.insert("end-1c", "\n", ("content",))
-                    for content, tags in self.compute_gutter_line(i):
-                        self._gutter.insert("end-1c", content, ("content",) + tags)
+
+                if not clean and text_line_count > 10 and gutter_line_count < 3:
+                    # probably initial load, do bulk insert
+                    parts = []
+                    for i in range(start, start + delta):
+                        parts.append("\n")
+                        for content, tags in self.compute_gutter_line(i, plain=True):
+                            parts.append(content)
+
+                    self._gutter.insert("end-1c", "".join(parts), ("content",) + tags)
+                else:
+                    for i in range(start, start + delta):
+                        self._gutter.insert("end-1c", "\n", ("content",))
+                        for content, tags in self.compute_gutter_line(i):
+                            self._gutter.insert("end-1c", content, ("content",) + tags)
             else:
                 self._gutter.delete(line2index(text_line_count) + "-1c", "end-1c")
 
@@ -942,13 +955,18 @@ class TextFrame(ttk.Frame):
         self._gutter.yview_moveto(first)
         self._update_gutter_active_line()
 
+        if text_line_count > 9998:
+            self._gutter.configure(width=7)
+        elif text_line_count > 998:
+            self._gutter.configure(width=6)
+
     def _update_gutter_active_line(self):
         self._gutter.tag_remove("active", "1.0", "end")
         insert = self.text.index("insert")
         self._gutter.tag_add("active", insert + " linestart", insert + " lineend")
 
-    def compute_gutter_line(self, lineno):
-        yield str(lineno), ("line_number",)
+    def compute_gutter_line(self, lineno, plain=False):
+        yield str(lineno), ()
 
     def update_margin_line(self):
         if self._recommended_line_length == 0:
@@ -962,9 +980,7 @@ class TextFrame(ttk.Frame):
                 bbox = self.text.bbox(first_visible_idx)
                 first_visible_col_x = bbox[0]
 
-                margin_line_visible_col = (
-                    self._recommended_line_length - first_visible_col
-                )
+                margin_line_visible_col = self._recommended_line_length - first_visible_col
                 delta = first_visible_col_x
             except Exception:
                 # fall back to ignoring scroll position
@@ -973,9 +989,7 @@ class TextFrame(ttk.Frame):
 
             if margin_line_visible_col > -1:
                 x = (
-                    get_text_font(self.text).measure(
-                        (margin_line_visible_col - 1) * "M"
-                    )
+                    get_text_font(self.text).measure((margin_line_visible_col - 1) * "M")
                     + delta
                     + self.text["padx"]
                 )
@@ -998,17 +1012,22 @@ class TextFrame(ttk.Frame):
         except tk.TclError:
             exception("on_gutter_click")
 
+    def on_gutter_double_click(self, event=None):
+        try:
+            self._gutter.mark_unset("gutter_selection_start")
+            self.text.tag_remove("sel", "1.0", "end")
+            self._gutter.tag_remove("sel", "1.0", "end")
+        except tk.TclError:
+            exception("on_gutter_click")
+
     def on_gutter_motion(self, event=None):
         try:
-            linepos = int(
-                self._gutter.index("@%s,%s" % (event.x, event.y)).split(".")[0]
-            )
-            gutter_selection_start = int(
-                self._gutter.index("gutter_selection_start").split(".")[0]
-            )
+            if "gutter_selection_start" not in self._gutter.mark_names():
+                return
+            linepos = int(self._gutter.index("@%s,%s" % (event.x, event.y)).split(".")[0])
+            gutter_selection_start = int(self._gutter.index("gutter_selection_start").split(".")[0])
             self.text.select_lines(
-                min(gutter_selection_start, linepos),
-                max(gutter_selection_start - 1, linepos - 1),
+                min(gutter_selection_start, linepos), max(gutter_selection_start - 1, linepos - 1)
             )
             self.text.mark_set("insert", "%s.0" % linepos)
         except tk.TclError:
@@ -1068,11 +1087,8 @@ def fixwordbreaks(root):
     # Make sure that Tk's double-click and next/previous word
     # operations use our definition of a word (i.e. an identifier)
     root.tk.call("tcl_wordBreakAfter", "a b", 0)  # make sure word.tcl is loaded
-    # TODO: IDLE updated following to
-    # root.tk.call('set', 'tcl_wordchars', r'\w')
-    # root.tk.call('set', 'tcl_nonwordchars', r'\W')
-    root.tk.call("set", "tcl_wordchars", u"[a-zA-Z0-9_À-ÖØ-öø-ÿĀ-ſƀ-ɏА-я]")
-    root.tk.call("set", "tcl_nonwordchars", u"[^a-zA-Z0-9_À-ÖØ-öø-ÿĀ-ſƀ-ɏА-я]")
+    root.tk.call("set", "tcl_wordchars", r"\w")
+    root.tk.call("set", "tcl_nonwordchars", r"\W")
 
 
 def rebind_control_a(root):
