@@ -11,16 +11,56 @@ import textwrap
 import threading
 import time
 import tkinter as tk
+import tkinter.font
 import traceback
 from tkinter import filedialog, messagebox, ttk
 from typing import Callable, List, Optional, Tuple, Union  # @UnusedImport
 
 from thonny import get_workbench, misc_utils, tktextext
 from thonny.common import TextRange
-from thonny.misc_utils import running_on_linux, running_on_mac_os, running_on_windows
+from thonny.misc_utils import (
+    running_on_linux,
+    running_on_mac_os,
+    running_on_windows,
+    running_on_rpi,
+)
 from thonny.tktextext import TweakableText
 import sys
 from _tkinter import TclError
+import webbrowser
+
+PARENS_REGEX = re.compile(r"[\(\)\{\}\[\]]")
+
+
+class CommonDialog(tk.Toplevel):
+    def __init__(self, master=None, cnf={}, **kw):
+        super().__init__(master=master, cnf=cnf, **kw)
+        self.bind("<FocusIn>", self._unlock_on_focus_in, True)
+
+    def _unlock_on_focus_in(self, event):
+        if not self.winfo_ismapped():
+            print("unlocking")
+            focussed_widget = self.focus_get()
+            self.deiconify()
+            if focussed_widget:
+                focussed_widget.focus_set()
+
+
+class CommonDialogEx(CommonDialog):
+    def __init__(self, master=None, cnf={}, **kw):
+        super().__init__(master=master, cnf=cnf, **kw)
+
+        # Need to fill the dialog with a frame to gain theme support
+        self.main_frame = ttk.Frame(self)
+        self.main_frame.grid(row=0, column=0, sticky="nsew")
+        self.columnconfigure(0, weight=1)
+        self.rowconfigure(0, weight=1)
+
+        self.bind("<Escape>", self.on_close, True)
+        self.protocol("WM_DELETE_WINDOW", self.on_close)
+
+    def on_close(self, event=None):
+        self.destroy()
 
 
 class CustomMenubar(ttk.Frame):
@@ -33,10 +73,7 @@ class CustomMenubar(ttk.Frame):
             "CustomMenubarLabel.TLabel",
             background=[
                 ("!active", lookup_style_option("Menubar", "background", "gray")),
-                (
-                    "active",
-                    lookup_style_option("Menubar", "activebackground", "LightYellow"),
-                ),
+                ("active", lookup_style_option("Menubar", "activebackground", "LightYellow")),
             ],
             foreground=[
                 ("!active", lookup_style_option("Menubar", "foreground", "black")),
@@ -326,12 +363,10 @@ class ClosableNotebook(ttk.Notebook):
         # self._check_update_style()
 
     def create_tab_menu(self):
-        menu = tk.Menu(
-            self.winfo_toplevel(), tearoff=False, **get_style_configuration("Menu")
-        )
-        menu.add_command(label="Close", command=self._close_tab_from_menu)
-        menu.add_command(label="Close others", command=self._close_other_tabs)
-        menu.add_command(label="Close all", command=self.close_tabs)
+        menu = tk.Menu(self.winfo_toplevel(), tearoff=False, **get_style_configuration("Menu"))
+        menu.add_command(label=_("Close"), command=self._close_tab_from_menu)
+        menu.add_command(label=_("Close others"), command=self._close_other_tabs)
+        menu.add_command(label=_("Close all"), command=self.close_tabs)
         return menu
 
     def _letf_btn_press(self, event):
@@ -427,8 +462,7 @@ class ClosableNotebook(ttk.Notebook):
                 "img_tab_close", file=os.path.join(img_dir, "tab_close.gif")
             )
             ClosableNotebook._close_active_img = tk.PhotoImage(
-                "img_tab_close_active",
-                file=os.path.join(img_dir, "tab_close_active.gif"),
+                "img_tab_close_active", file=os.path.join(img_dir, "tab_close_active.gif")
             )
 
         style.element_create(
@@ -481,6 +515,24 @@ class ClosableNotebook(ttk.Notebook):
             ],
         )
 
+    def _check_remove_padding(self, kw):
+        # Windows themes produce 1-pixel padding to the bottom of the pane
+        # Don't know how to get rid of it using themes
+        if "padding" not in kw and ttk.Style().theme_use().lower() in (
+            "windows",
+            "xpnative",
+            "vista",
+        ):
+            kw["padding"] = (0, 0, 0, -1)
+
+    def add(self, child, **kw):
+        self._check_remove_padding(kw)
+        super().add(child, **kw)
+
+    def insert(self, pos, child, **kw):
+        self._check_remove_padding(kw)
+        super().insert(pos, child, **kw)
+
 
 class AutomaticNotebook(ClosableNotebook):
     """
@@ -489,7 +541,7 @@ class AutomaticNotebook(ClosableNotebook):
     """
 
     def __init__(self, master, position_key, preferred_size_in_pw=None):
-        if get_workbench().get_ui_mode() == "simple":
+        if get_workbench().in_simple_mode():
             style = "TNotebook"
         else:
             style = "ButtonNotebook.TNotebook"
@@ -618,10 +670,7 @@ def sequence_to_accelerator(sequence):
         return sequence
 
     accelerator = (
-        sequence.strip("<>")
-        .replace("Key-", "")
-        .replace("KeyPress-", "")
-        .replace("Control", "Ctrl")
+        sequence.strip("<>").replace("Key-", "").replace("KeyPress-", "").replace("Control", "Ctrl")
     )
 
     # Tweaking individual parts
@@ -665,6 +714,28 @@ def set_zoomed(toplevel, value):
 
 
 class EnhancedTextWithLogging(tktextext.EnhancedText):
+    def __init__(
+        self,
+        master=None,
+        style="Text",
+        tag_current_line=False,
+        indent_with_tabs=False,
+        replace_tabs=False,
+        cnf={},
+        **kw
+    ):
+        super().__init__(
+            master=master,
+            style=style,
+            tag_current_line=tag_current_line,
+            indent_with_tabs=indent_with_tabs,
+            replace_tabs=replace_tabs,
+            cnf=cnf,
+            **kw
+        )
+
+        self._last_event_changed_line_count = False
+
     def direct_insert(self, index, chars, tags=None, **kw):
         try:
             # try removing line numbers
@@ -672,17 +743,24 @@ class EnhancedTextWithLogging(tktextext.EnhancedText):
             # TODO: does it occur when opening a file with line numbers in it?
             # if self._propose_remove_line_numbers and isinstance(chars, str):
             #    chars = try_remove_linenumbers(chars, self)
+
             concrete_index = self.index(index)
-            return tktextext.EnhancedText.direct_insert(
-                self, index, chars, tags=tags, **kw
-            )
+            line_before = self.get(concrete_index + " linestart", concrete_index + " lineend")
+            self._last_event_changed_line_count = "\n" in chars
+            return tktextext.EnhancedText.direct_insert(self, index, chars, tags=tags, **kw)
         finally:
+            line_after = self.get(concrete_index + " linestart", concrete_index + " lineend")
+            trivial_for_coloring, trivial_for_parens = self._is_trivial_edit(
+                chars, line_before, line_after
+            )
             get_workbench().event_generate(
                 "TextInsert",
                 index=concrete_index,
                 text=chars,
                 tags=tags,
                 text_widget=self,
+                trivial_for_coloring=trivial_for_coloring,
+                trivial_for_parens=trivial_for_parens,
             )
 
     def direct_delete(self, index1, index2=None, **kw):
@@ -694,17 +772,69 @@ class EnhancedTextWithLogging(tktextext.EnhancedText):
             else:
                 concrete_index2 = None
 
-            return tktextext.EnhancedText.direct_delete(
-                self, index1, index2=index2, **kw
+            chars = self.get(index1, index2)
+            self._last_event_changed_line_count = "\n" in chars
+            line_before = self.get(
+                concrete_index1 + " linestart",
+                (concrete_index1 if concrete_index2 is None else concrete_index2) + " lineend",
             )
+            return tktextext.EnhancedText.direct_delete(self, index1, index2=index2, **kw)
         finally:
+            line_after = self.get(
+                concrete_index1 + " linestart",
+                (concrete_index1 if concrete_index2 is None else concrete_index2) + " lineend",
+            )
+            trivial_for_coloring, trivial_for_parens = self._is_trivial_edit(
+                chars, line_before, line_after
+            )
             get_workbench().event_generate(
                 "TextDelete",
                 index1=concrete_index1,
                 index2=concrete_index2,
                 text_widget=self,
+                trivial_for_coloring=trivial_for_coloring,
+                trivial_for_parens=trivial_for_parens,
             )
 
+    def _is_trivial_edit(self, chars, line_before, line_after):
+        # line is taken after edit for insertion and before edit for deletion
+        if not chars.strip():
+            # linebreaks, including with automatic indent
+            # check it doesn't break a triple-quote
+            trivial_for_coloring = line_before.count("'''") == line_after.count(
+                "'''"
+            ) and line_before.count('"""') == line_after.count('"""')
+            trivial_for_parens = trivial_for_coloring
+        elif len(chars) > 1:
+            # paste, cut, load or something like this
+            trivial_for_coloring = False
+            trivial_for_parens = False
+        elif chars == "#":
+            trivial_for_coloring = "''''" not in line_before and '"""' not in line_before
+            trivial_for_parens = trivial_for_coloring and not re.search(PARENS_REGEX, line_before)
+        elif chars in "()[]{}":
+            trivial_for_coloring = line_before.count("'''") == line_after.count(
+                "'''"
+            ) and line_before.count('"""') == line_after.count('"""')
+            trivial_for_parens = False
+        elif chars == "'":
+            trivial_for_coloring = "'''" not in line_before and "'''" not in line_after
+            trivial_for_parens = False  # can put parens into open string
+        elif chars == '"':
+            trivial_for_coloring = '"""' not in line_before and '"""' not in line_after
+            trivial_for_parens = False  # can put parens into open string
+        elif chars == "\\":
+            # can shorten closing quote
+            trivial_for_coloring = '"""' not in line_before and '"""' not in line_after
+            trivial_for_parens = False
+        else:
+            trivial_for_coloring = line_before.count("'''") == line_after.count(
+                "'''"
+            ) and line_before.count('"""') == line_after.count('"""')
+            trivial_for_parens = trivial_for_coloring
+
+        return trivial_for_coloring, trivial_for_parens
+
 
 class SafeScrollbar(ttk.Scrollbar):
     def __init__(self, master=None, **kw):
@@ -756,9 +886,7 @@ class VerticallyScrollableFrame(ttk.Frame):
 
         # set up scrolling with canvas
         vscrollbar = ttk.Scrollbar(self, orient=tk.VERTICAL)
-        self.canvas = tk.Canvas(
-            self, bd=0, highlightthickness=0, yscrollcommand=vscrollbar.set
-        )
+        self.canvas = tk.Canvas(self, bd=0, highlightthickness=0, yscrollcommand=vscrollbar.set)
         vscrollbar.config(command=self.canvas.yview)
         self.canvas.xview_moveto(0)
         self.canvas.yview_moveto(0)
@@ -768,9 +896,7 @@ class VerticallyScrollableFrame(ttk.Frame):
         self.rowconfigure(0, weight=1)
 
         self.interior = ttk.Frame(self.canvas)
-        self.interior_id = self.canvas.create_window(
-            0, 0, window=self.interior, anchor=tk.NW
-        )
+        self.interior_id = self.canvas.create_window(0, 0, window=self.interior, anchor=tk.NW)
         self.bind("<Configure>", self._configure_interior, "+")
         self.bind("<Expose>", self._expose, "+")
 
@@ -803,9 +929,7 @@ class ScrollableFrame(ttk.Frame):
         # set up scrolling with canvas
         vscrollbar = ttk.Scrollbar(self, orient=tk.VERTICAL)
         hscrollbar = ttk.Scrollbar(self, orient=tk.HORIZONTAL)
-        self.canvas = tk.Canvas(
-            self, bd=0, highlightthickness=0, yscrollcommand=vscrollbar.set
-        )
+        self.canvas = tk.Canvas(self, bd=0, highlightthickness=0, yscrollcommand=vscrollbar.set)
         vscrollbar.config(command=self.canvas.yview)
         hscrollbar.config(command=self.canvas.xview)
 
@@ -822,9 +946,7 @@ class ScrollableFrame(ttk.Frame):
         self.interior = ttk.Frame(self.canvas)
         self.interior.columnconfigure(0, weight=1)
         self.interior.rowconfigure(0, weight=1)
-        self.interior_id = self.canvas.create_window(
-            0, 0, window=self.interior, anchor=tk.NW
-        )
+        self.interior_id = self.canvas.create_window(0, 0, window=self.interior, anchor=tk.NW)
         self.bind("<Configure>", self._configure_interior, "+")
         self.bind("<Expose>", self._expose, "+")
 
@@ -896,7 +1018,7 @@ class ToolTip:
             return
         x, y, _, cy = self.widget.bbox("insert")
         x = x + self.widget.winfo_rootx() + 27
-        y = y + cy + self.widget.winfo_rooty() + 27
+        y = y + cy + self.widget.winfo_rooty() + self.widget.winfo_height() + 2
         self.tipwindow = tw = tk.Toplevel(self.widget)
         tw.wm_overrideredirect(1)
         if running_on_mac_os():
@@ -905,24 +1027,21 @@ class ToolTip:
         tw.wm_geometry("+%d+%d" % (x, y))
         try:
             # For Mac OS
-            tw.tk.call(
-                "::tk::unsupported::MacWindowStyle",
-                "style",
-                tw._w,
-                "help",
-                "noActivates",
-            )
+            tw.tk.call("::tk::unsupported::MacWindowStyle", "style", tw._w, "help", "noActivates")
         except tk.TclError:
             pass
         label = tk.Label(tw, text=self.text, **self.options)
         label.pack()
+        # get_workbench().bind("WindowFocusOut", self.hidetip, True)
 
-    def hidetip(self):
+    def hidetip(self, event=None):
         tw = self.tipwindow
         self.tipwindow = None
         if tw:
             tw.destroy()
 
+        # get_workbench().unbind("WindowFocusOut", self.hidetip)
+
 
 def create_tooltip(widget, text, **kw):
     options = get_style_configuration("Tooltip").copy()
@@ -945,7 +1064,7 @@ def create_tooltip(widget, text, **kw):
     widget.bind("<Leave>", leave)
 
 
-class NoteBox(tk.Toplevel):
+class NoteBox(CommonDialog):
     def __init__(self, master=None, max_default_width=300, **kw):
         super().__init__(master=master, highlightthickness=0, **kw)
 
@@ -958,11 +1077,7 @@ class NoteBox(tk.Toplevel):
         try:
             # For Mac OS
             self.tk.call(
-                "::tk::unsupported::MacWindowStyle",
-                "style",
-                self._w,
-                "help",
-                "noActivates",
+                "::tk::unsupported::MacWindowStyle", "style", self._w, "help", "noActivates"
             )
         except tk.TclError:
             pass
@@ -1054,9 +1169,7 @@ class NoteBox(tk.Toplevel):
                     "%d.%d lineend" % (focus.end_lineno - 1, focus.end_lineno - 1)
                 )
             else:
-                botright = target.bbox(
-                    "%d.%d" % (focus.end_lineno, focus.end_col_offset)
-                )
+                botright = target.bbox("%d.%d" % (focus.end_lineno, focus.end_col_offset))
 
             if topleft and botright:
                 focus_x += topleft[0]
@@ -1093,8 +1206,7 @@ class NoteBox(tk.Toplevel):
         line_height = font.metrics()["linespace"]
 
         self.wm_geometry(
-            "%dx%d+%d+%d"
-            % (width, line_count * line_height, focus_x, focus_y + focus_height)
+            "%dx%d+%d+%d" % (width, line_count * line_height, focus_x, focus_y + focus_height)
         )
 
         # TODO: detect the situation when note doesn't fit under
@@ -1102,9 +1214,7 @@ class NoteBox(tk.Toplevel):
 
         self.deiconify()
 
-    def show_note(
-        self, *content_items: Union[str, List], target=None, focus=None
-    ) -> None:
+    def show_note(self, *content_items: Union[str, List], target=None, focus=None) -> None:
 
         self.set_content(*content_items)
         self.place(target, focus)
@@ -1240,8 +1350,6 @@ def try_remove_linenumbers(text, master):
             title="Remove linenumbers",
             message="Do you want to remove linenumbers from pasted text?",
             default=messagebox.YES,
-            master=master,
-            parent=master,
         ):
             return remove_line_numbers(text)
         else:
@@ -1253,9 +1361,7 @@ def try_remove_linenumbers(text, master):
 
 def has_line_numbers(text):
     lines = text.splitlines()
-    return len(lines) > 2 and all(
-        [len(split_after_line_number(line)) == 2 for line in lines]
-    )
+    return len(lines) > 2 and all([len(split_after_line_number(line)) == 2 for line in lines])
 
 
 def split_after_line_number(s):
@@ -1280,36 +1386,54 @@ def remove_line_numbers(s):
 
 
 def center_window(win, master=None):
-    # looks like it doesn't take window border into account
-    win.update_idletasks()
+    # for backward compat
+    return assign_geometry(win, master)
 
-    if getattr(master, "initializing", False):
-        # can't get reliable positions when main window is not in mainloop yet
-        left = (win.winfo_screenwidth() - 600) // 2
-        top = (win.winfo_screenheight() - 400) // 2
+
+def assign_geometry(win, master=None):
+
+    if master is None:
+        master = tk._default_root
+
+    size = get_workbench().get_option(get_size_option_name(win))
+    if size:
+        width, height = size
+        saved_size = True
     else:
-        if master is None:
-            left = win.winfo_screenwidth() - win.winfo_width() // 2
-            top = win.winfo_screenheight() - win.winfo_height() // 2
+        fallback_width = 600
+        fallback_height = 400
+        # need to wait until size is computed
+        # (unfortunately this causes dialog to jump)
+        if getattr(master, "initializing", False):
+            # can't get reliable positions when main window is not in mainloop yet
+            width = fallback_width
+            height = fallback_height
         else:
-            left = (
-                master.winfo_rootx()
-                + master.winfo_width() // 2
-                - win.winfo_width() // 2
-            )
-            top = (
-                master.winfo_rooty()
-                + master.winfo_height() // 2
-                - win.winfo_height() // 2
-            )
+            if not running_on_linux():
+                # better to avoid in Linux because it causes ugly jump
+                win.update_idletasks()
+            # looks like it doesn't take window border into account
+            width = win.winfo_width()
+            height = win.winfo_height()
 
-    win.geometry("+%d+%d" % (left, top))
+            if width < 10:
+                # ie. size measurement is not correct
+                width = fallback_width
+                height = fallback_height
+
+        saved_size = False
+
+    left = master.winfo_rootx() + master.winfo_width() // 2 - width // 2
+    top = master.winfo_rooty() + master.winfo_height() // 2 - height // 2
+
+    if saved_size:
+        win.geometry("%dx%d+%d+%d" % (width, height, left, top))
+    else:
+        win.geometry("+%d+%d" % (left, top))
 
 
-class WaitingDialog(tk.Toplevel):
-    def __init__(
-        self, master, async_result, description, title="Please wait!", timeout=None
-    ):
+class WaitingDialog(CommonDialog):
+    def __init__(self, master, async_result, description, title="Please wait!", timeout=None):
         self._async_result = async_result
         super().__init__(master)
         if misc_utils.running_on_mac_os():
@@ -1352,7 +1476,7 @@ def run_with_waiting_dialog(master, action, args=(), description="Working"):
     return async_result.get()
 
 
-class FileCopyDialog(tk.Toplevel):
+class FileCopyDialog(CommonDialog):
     def __init__(self, master, source, destination, description=None, fsync=True):
         self._source = source
         self._destination = destination
@@ -1370,22 +1494,18 @@ class FileCopyDialog(tk.Toplevel):
         self.rowconfigure(0, weight=1)
         self.columnconfigure(0, weight=1)
 
-        self.title("Copying")
+        self.title(_("Copying"))
 
         if description is None:
-            description = "Copying\n  %s\nto\n  %s" % (source, destination)
+            description = _("Copying\n  %s\nto\n  %s") % (source, destination)
 
         label = ttk.Label(main_frame, text=description)
         label.grid(row=0, column=0, columnspan=2, sticky="nw", padx=15, pady=15)
 
-        self._bar = ttk.Progressbar(
-            main_frame, maximum=os.path.getsize(source), length=200
-        )
+        self._bar = ttk.Progressbar(main_frame, maximum=os.path.getsize(source), length=200)
         self._bar.grid(row=1, column=0, columnspan=2, sticky="nsew", padx=15, pady=0)
 
-        self._cancel_button = ttk.Button(
-            main_frame, text="Cancel", command=self._cancel
-        )
+        self._cancel_button = ttk.Button(main_frame, text=_("Cancel"), command=self._cancel)
         self._cancel_button.grid(row=2, column=1, sticky="ne", padx=15, pady=15)
         self._bar.focus_set()
 
@@ -1393,9 +1513,7 @@ class FileCopyDialog(tk.Toplevel):
 
         self._update_progress()
 
-        self.bind(
-            "<Escape>", self._cancel, True
-        )  # escape-close only if process has completed
+        self.bind("<Escape>", self._cancel, True)  # escape-close only if process has completed
         self.protocol("WM_DELETE_WINDOW", self._cancel)
         self._start()
 
@@ -1440,7 +1558,7 @@ class FileCopyDialog(tk.Toplevel):
         self._close()
 
 
-class ChoiceDialog(tk.Toplevel):
+class ChoiceDialog(CommonDialog):
     def __init__(
         self, master=None, title="Choose one", question: str = "Choose one:", choices=[]
     ) -> None:
@@ -1453,9 +1571,7 @@ class ChoiceDialog(tk.Toplevel):
 
         row = 0
         question_label = ttk.Label(self, text=question)
-        question_label.grid(
-            row=row, column=0, columnspan=2, sticky="w", padx=20, pady=20
-        )
+        question_label.grid(row=row, column=0, columnspan=2, sticky="w", padx=20, pady=20)
         row += 1
 
         self.var = tk.StringVar()
@@ -1488,55 +1604,56 @@ class ChoiceDialog(tk.Toplevel):
         self.result = None
         self.destroy()
 
-class LongTextDialog(tk.Toplevel):
+
+class LongTextDialog(CommonDialog):
     def __init__(self, title, text_content, parent=None):
         if parent is None:
             parent = tk._default_root
-        
+
         super().__init__(master=parent)
-        self.title(title)    
-        
+        self.title(title)
+
         main_frame = ttk.Frame(self)
         main_frame.grid(row=0, column=0, sticky="nsew")
         self.columnconfigure(0, weight=1)
         self.rowconfigure(0, weight=1)
-        
+
         default_font = tk.font.nametofont("TkDefaultFont")
-        self._text = tktextext.TextFrame(main_frame, read_only=True, 
-                                         wrap="none",
-                                         font=default_font,
-                                         width=80,
-                                         height=10,
-                                         relief="sunken",
-                                         borderwidth=1)
+        self._text = tktextext.TextFrame(
+            main_frame,
+            read_only=True,
+            wrap="none",
+            font=default_font,
+            width=80,
+            height=10,
+            relief="sunken",
+            borderwidth=1,
+        )
         self._text.grid(row=1, column=0, columnspan=2, sticky="nsew", padx=20, pady=20)
         self._text.text.direct_insert("1.0", text_content)
         self._text.text.see("1.0")
-        
-        copy_button = ttk.Button(main_frame, command=self._copy, 
-                                 text="Copy to clipboard",
-                                 width=20)
-        copy_button.grid(row=2, column=0, sticky="w", padx=20, pady=(0,20))        
-        
-        close_button = ttk.Button(main_frame, command=self._close, text="Close")
-        close_button.grid(row=2, column=1, sticky="w", padx=20, pady=(0,20))
-        
+
+        copy_button = ttk.Button(
+            main_frame, command=self._copy, text=_("Copy to clipboard"), width=20
+        )
+        copy_button.grid(row=2, column=0, sticky="w", padx=20, pady=(0, 20))
+
+        close_button = ttk.Button(main_frame, command=self._close, text=_("Close"))
+        close_button.grid(row=2, column=1, sticky="w", padx=20, pady=(0, 20))
+
         main_frame.columnconfigure(0, weight=1)
         main_frame.rowconfigure(1, weight=1)
-        
+
         self.protocol("WM_DELETE_WINDOW", self._close)
-        self.bind("<Escape>", self._close, True)        
-        
+        self.bind("<Escape>", self._close, True)
+
     def _copy(self, event=None):
         self.clipboard_clear()
         self.clipboard_append(self._text.text.get("1.0", "end"))
-    
+
     def _close(self, event=None):
         self.destroy()
-    
-    
-        
-        
+
 
 def ask_one_from_choices(
     master=None, title="Choose one", question: str = "Choose one:", choices=[]
@@ -1546,18 +1663,12 @@ def ask_one_from_choices(
     return dlg.result
 
 
-class SubprocessDialog(tk.Toplevel):
+class SubprocessDialog(CommonDialog):
     """Shows incrementally the output of given subprocess.
     Allows cancelling"""
 
     def __init__(
-        self,
-        master,
-        proc,
-        title,
-        long_description=None,
-        autoclose=True,
-        conclusion="Done.",
+        self, master, proc, title, long_description=None, autoclose=True, conclusion="Done."
     ):
         self._closed = False
         self._proc = proc
@@ -1571,7 +1682,7 @@ class SubprocessDialog(tk.Toplevel):
         self._event_queue = collections.deque()
         self._conclusion = conclusion
 
-        tk.Toplevel.__init__(self, master)
+        super().__init__(master)
 
         self.rowconfigure(0, weight=1)
         self.columnconfigure(0, weight=1)
@@ -1596,7 +1707,7 @@ class SubprocessDialog(tk.Toplevel):
         if long_description is not None:
             self.text.direct_insert("1.0", long_description + "\n\n")
 
-        self.button = ttk.Button(main_frame, text="Cancel", command=self._close)
+        self.button = ttk.Button(main_frame, text=_("Cancel"), command=self._close)
         self.button.grid(row=1, column=0, pady=(0, 15))
 
         main_frame.rowconfigure(0, weight=1)
@@ -1627,9 +1738,7 @@ class SubprocessDialog(tk.Toplevel):
 
             self.returncode = self._proc.wait()
 
-        self._stdout_thread = threading.Thread(
-            target=listen_stream, args=["stdout"], daemon=True
-        )
+        self._stdout_thread = threading.Thread(target=listen_stream, args=["stdout"], daemon=True)
         self._stdout_thread.start()
         if self._proc.stderr is not None:
             self._stderr_thread = threading.Thread(
@@ -1650,7 +1759,7 @@ class SubprocessDialog(tk.Toplevel):
             if self.returncode == None:
                 self.after(200, poll_output_events)
             else:
-                self.button["text"] = "OK"
+                self.button["text"] = _("OK")
                 self.button.focus_set()
                 if self.returncode != 0:
                     self.text.direct_insert("end", "\n\nReturn code: ", ("stderr",))
@@ -1669,16 +1778,14 @@ class SubprocessDialog(tk.Toplevel):
     def _close(self, event=None):
         if self._proc.poll() is None:
             if messagebox.askyesno(
-                "Cancel the process?",
-                "The process is still running.\nAre you sure you want to cancel?",
-                parent=self,
+                _("Cancel the process?"),
+                _("The process is still running.\nAre you sure you want to cancel?"),
+                parent=None if running_on_mac_os() else self,
             ):
                 # try gently first
                 try:
                     if running_on_windows():
-                        os.kill(
-                            self._proc.pid, signal.CTRL_BREAK_EVENT
-                        )  # @UndefinedVariable
+                        os.kill(self._proc.pid, signal.CTRL_BREAK_EVENT)  # @UndefinedVariable
                     else:
                         os.kill(self._proc.pid, signal.SIGINT)
 
@@ -1784,33 +1891,43 @@ def _get_dialog_provider():
 
 def asksaveasfilename(**options):
     # https://tcl.tk/man/tcl8.6/TkCmd/getSaveFile.htm
-    _ensure_parent(options)
+    _tweak_file_dialog_parent(options)
     return _get_dialog_provider().asksaveasfilename(**options)
 
 
 def askopenfilename(**options):
     # https://tcl.tk/man/tcl8.6/TkCmd/getOpenFile.htm
-    _ensure_parent(options)
+    _tweak_file_dialog_parent(options)
     return _get_dialog_provider().askopenfilename(**options)
 
 
 def askopenfilenames(**options):
     # https://tcl.tk/man/tcl8.6/TkCmd/getOpenFile.htm
-    _ensure_parent(options)
+    _tweak_file_dialog_parent(options)
     return _get_dialog_provider().askopenfilenames(**options)
 
 
 def askdirectory(**options):
     # https://tcl.tk/man/tcl8.6/TkCmd/chooseDirectory.htm
-    _ensure_parent(options)
+    _tweak_file_dialog_parent(options)
     return _get_dialog_provider().askdirectory(**options)
 
-def _ensure_parent(options):
-    if "parent" not in options:
+
+def _tweak_file_dialog_parent(options):
+    if running_on_mac_os():
+        # used to require master/parent (https://bugs.python.org/issue34927)
+        # but this is deprecated in Catalina (https://github.com/thonny/thonny/issues/840)
         if "master" in options:
-            options["parent"] = options["master"]
-        else:
-            options["parent"] = tk._default_root
+            del options["master"]
+        if "parent" in options:
+            del options["parent"]
+    else:
+        if "parent" not in options:
+            if "master" in options:
+                options["parent"] = options["master"]
+            else:
+                options["parent"] = tk._default_root
+
 
 class _ZenityDialogProvider:
     # https://www.writebash.com/bash-gui/zenity-create-file-selection-dialog-224.html
@@ -1870,18 +1987,20 @@ class _ZenityDialogProvider:
             if pattern.startswith("."):
                 pattern = "*" + pattern
 
+            if pattern == "*.*":
+                # ".*" was provided to make the pattern safe for Tk dialog
+                # not required with Zenity
+                pattern = "*"
+
             args.append("--file-filter=%s | %s" % (desc, pattern))
-        
+
         return args
 
     @classmethod
     def _call(cls, args):
         args = ["zenity", "--name=Thonny", "--class=Thonny"] + args
         result = subprocess.run(
-            args,
-            stdout=subprocess.PIPE,
-            stderr=subprocess.PIPE,
-            universal_newlines=True,
+            args, stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True
         )
         if result.returncode == 0:
             return result.stdout.strip()
@@ -1931,7 +2050,7 @@ def handle_mistreated_latin_shortcuts(registry, event):
 
     # print(simplified_state, event.keycode)
     if (simplified_state, event.keycode) in registry:
-        if event.keycode != ord(event.char):
+        if event.keycode != ord(event.char) and event.keysym in (None, "??"):
             # keycode and char doesn't match,
             # this means non-latin keyboard
             for handler, tester in registry[(simplified_state, event.keycode)]:
@@ -1939,42 +2058,59 @@ def handle_mistreated_latin_shortcuts(registry, event):
                     handler()
 
 
-def show_dialog(dlg, master=None, center=True):
+def show_dialog(dlg, master=None, geometry=True):
     if master is None:
         master = tk._default_root
 
+    get_workbench().event_generate("WindowFocusOut")
+    # following order seems to give most smooth appearance
     focused_widget = master.focus_get()
-    dlg.transient(master)
-    dlg.grab_set()
+    dlg.transient(master.winfo_toplevel())
+
+    if geometry:
+        # dlg.withdraw() # unfortunately inhibits size calculations in assign_geometry
+        if isinstance(geometry, str):
+            dlg.geometry(geometry)
+        else:
+            assign_geometry(dlg, master)
+        # dlg.wm_deiconify()
+
+    try:
+        dlg.grab_set()
+    except:
+        pass
+
     dlg.lift()
     dlg.focus_set()
-    if center:
-        center_window(dlg, master)
-    master.wait_window(dlg)
+
+    master.winfo_toplevel().wait_window(dlg)
     dlg.grab_release()
-    master.lift()
-    master.focus_force()
+    master.winfo_toplevel().lift()
+    master.winfo_toplevel().focus_force()
+    master.winfo_toplevel().grab_set()
+
     if focused_widget is not None:
-        focused_widget.focus_set()
+        try:
+            focused_widget.focus_force()
+        except TclError:
+            pass
 
 
-def popen_with_ui_thread_callback(
-    *Popen_args, on_completion, poll_delay=0.1, **Popen_kwargs
-):
+def popen_with_ui_thread_callback(*Popen_args, on_completion, poll_delay=0.1, **Popen_kwargs):
     if "encoding" not in Popen_kwargs:
-        if "env" not in Popen_kwargs: 
+        if "env" not in Popen_kwargs:
             Popen_kwargs["env"] = os.environ.copy()
         Popen_kwargs["env"]["PYTHONIOENCODING"] = "utf-8"
-        if sys.version_info >= (3,6): 
+        if sys.version_info >= (3, 6):
             Popen_kwargs["encoding"] = "utf-8"
-    
+
     proc = subprocess.Popen(*Popen_args, **Popen_kwargs)
-    
+
     # Need to read in thread in order to avoid blocking because
     # of full pipe buffer (see https://bugs.python.org/issue1256)
     out_lines = []
     err_lines = []
-    
+
     def read_stream(stream, target_list):
         while True:
             line = stream.readline()
@@ -1982,14 +2118,12 @@ def popen_with_ui_thread_callback(
                 target_list.append(line)
             else:
                 break
-            
-    t_out = threading.Thread(target=read_stream, daemon=True,
-                             args=(proc.stdout, out_lines))
-    t_err = threading.Thread(target=read_stream, daemon=True,
-                             args=(proc.stderr, err_lines))
+
+    t_out = threading.Thread(target=read_stream, daemon=True, args=(proc.stdout, out_lines))
+    t_err = threading.Thread(target=read_stream, daemon=True, args=(proc.stderr, err_lines))
     t_out.start()
     t_err.start()
-    
+
     def poll():
         if proc.poll() is not None:
             t_out.join(3)
@@ -2002,16 +2136,17 @@ def popen_with_ui_thread_callback(
     poll()
     return proc
 
+
 class MenuEx(tk.Menu):
     def __init__(self, target):
         self._testers = {}
-        super().__init__(target, tearoff=False,
-                         postcommand=self.on_post, 
-                         **get_style_configuration("Menu"))
-    
+        super().__init__(
+            target, tearoff=False, postcommand=self.on_post, **get_style_configuration("Menu")
+        )
+
     def on_post(self, *args):
         self.update_item_availability()
-    
+
     def update_item_availability(self):
         for i in range(self.index("end") + 1):
             item_data = self.entryconfigure(i)
@@ -2021,73 +2156,112 @@ class MenuEx(tk.Menu):
                     self.entryconfigure(i, state=tk.DISABLED)
                 else:
                     self.entryconfigure(i, state=tk.NORMAL)
-                
-    
+
     def add(self, kind, cnf={}, **kw):
         cnf = cnf or kw
         tester = cnf.get("tester")
         if "tester" in cnf:
             del cnf["tester"]
-        
+
         super().add(kind, cnf)
-        
+
         itemdata = self.entryconfigure(self.index("end"))
         labeldata = itemdata.get("label")
-        if labeldata: 
+        if labeldata:
             self._testers[labeldata] = tester
 
+
 class TextMenu(MenuEx):
     def __init__(self, target):
         self.text = target
         MenuEx.__init__(self, target)
         self.add_basic_items()
         self.add_extra_items()
-    
+
     def add_basic_items(self):
-        self.add_command(label="Cut", command=self.on_cut, tester=self.can_cut)
-        self.add_command(label="Copy", command=self.on_copy, tester=self.can_copy)
-        self.add_command(label="Paste", command=self.on_paste, tester=self.can_paste)
-    
+        self.add_command(label=_("Cut"), command=self.on_cut, tester=self.can_cut)
+        self.add_command(label=_("Copy"), command=self.on_copy, tester=self.can_copy)
+        self.add_command(label=_("Paste"), command=self.on_paste, tester=self.can_paste)
+
     def add_extra_items(self):
         self.add_separator()
-        self.add_command(label="Select All", command=self.on_select_all)
-    
+        self.add_command(label=_("Select All"), command=self.on_select_all)
+
     def on_cut(self):
         self.text.event_generate("<<Cut>>")
-    
+
     def on_copy(self):
         self.text.event_generate("<<Copy>>")
-    
+
     def on_paste(self):
         self.text.event_generate("<<Paste>>")
-    
+
     def on_select_all(self):
         self.text.event_generate("<<SelectAll>>")
-    
+
     def can_cut(self):
-        return (
-            self.get_selected_text()
-            and not self.selection_is_read_only()
-        )
-    
+        return self.get_selected_text() and not self.selection_is_read_only()
+
     def can_copy(self):
         return self.get_selected_text()
-    
+
     def can_paste(self):
         return not self.selection_is_read_only()
-    
+
     def get_selected_text(self):
         try:
             return self.text.get("sel.first", "sel.last")
         except TclError:
             return ""
-    
+
     def selection_is_read_only(self):
         if hasattr(self.text, "is_read_only"):
             return self.text.is_read_only()
-        
+
         return False
-        
+
+
+def create_url_label(master, url, text=None):
+    url_font = tkinter.font.nametofont("TkDefaultFont").copy()
+    url_font.configure(underline=1)
+    url_label = ttk.Label(
+        master, text=text if text else url, style="Url.TLabel", cursor="hand2", font=url_font
+    )
+    url_label.grid()
+    url_label.bind("<Button-1>", lambda _: webbrowser.open(url))
+    return url_label
+
+
+def get_size_option_name(window):
+    return "layout." + type(window).__name__ + "_size"
+
+
+def get_default_theme():
+    if running_on_windows():
+        return "Windows"
+    elif running_on_rpi():
+        return "Raspberry Pi"
+    else:
+        return "Enhanced Clam"
+
+
+def get_default_basic_theme():
+    if running_on_windows():
+        return "xpnative"
+    else:
+        return "clam"
+
+
+EM_WIDTH = None
+
+
+def ems_to_pixels(x):
+    global EM_WIDTH
+    if EM_WIDTH is None:
+        EM_WIDTH = tkinter.font.nametofont("TkDefaultFont").measure("m")
+    return int(EM_WIDTH * x)
+
+
 if __name__ == "__main__":
     root = tk.Tk()
     closa = ClosableNotebook(root)
